/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package actions;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.beans.PropertyVetoException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import javax.swing.JFileChooser;
import javax.swing.KeyStroke;

import dialogs.MtgFileView;
import objets.CMathGrafDoc;
import types.NatureObjet;
import entreesSorties.MathGraphDataInputStream;
import mathgraph32.Global;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import mathgraph32.MtgMenuFileItem;

/**
 * 
 * @author Yves
 */
@SuppressWarnings("serial")
public class ActionFichierOuvrir extends MtgAction {
	public ActionFichierOuvrir(MtgFrame frameInit) {
		super(frameInit, null, getString("menuFichierOuvrir"), getString("tipFichierOuvrir"),
				iconeAssociee(), KeyStroke.getKeyStroke("control O"), true);
	}

	public static String iconeAssociee() {
		return "images/Open24";
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		MtgInternalFrame cadre;
		class MyFilter extends javax.swing.filechooser.FileFilter {
			public boolean accept(File file) {
				if (file.isDirectory())
					return true;
				String filename = file.getName();
				return filename.endsWith(Global.GLOBAL_COMMA_MGF) || filename.endsWith(Global.GLOBAL_COMMA_MGJ);
			}

			public String getDescription() {
				return "*.mgf;*.mgj";
			}
		}
		String ch1;
		if (frame.repertoireFichiers.length() == 0) ch1 = frame.pref_fileDirectory;
		else ch1 = frame.repertoireFichiers;
		JFileChooser fc = new JFileChooser(ch1);
		boolean hres = MtgFrame.screenIsHighResolution;
		fc.setPreferredSize(new Dimension(hres ? 1200 : 800, hres ? 800 : 600));
		fc.setFileView(new MtgFileView());
		fc.setFileFilter(new MyFilter());
		int rVal = fc.showOpenDialog(frame);
    if(rVal == JFileChooser.APPROVE_OPTION) {		
			File file = fc.getSelectedFile();
			String ch = file.getAbsolutePath();
			String chlc = ch.toLowerCase();
			if (!chlc.endsWith(Global.GLOBAL_COMMA_MGJ) && !chlc.endsWith(Global.GLOBAL_COMMA_MGF)) {
				ch = ch + Global.GLOBAL_COMMA_MGJ;
				file = new File(ch);
			}			
			if (file != null) {
				MtgInternalFrame cad = frame.cadreParChemin(file.getAbsolutePath());
				if (cad == null) { // On n'ouvre pas un fichier dj ouvert
					try {
						// On actualise la variable repertoireFichiers du MtgFrame pre pour
						// qu' la prochaine ouverture
						// on soit plac dans le mme rpertoire
						// String ch = file.getAbsolutePath();
						int ind = ch.lastIndexOf(File.separatorChar);
						frame.repertoireFichiers = ch.substring(0, ind);
						FileInputStream fis = new FileInputStream(file);
						MathGraphDataInputStream inps = new MathGraphDataInputStream(fis);
						try {
							// MtgInternalFrame cadreActif = frame.cadreActif();
							// if (cadreActif != null) cadreActif.cacheDialogueAffichageVariables();
							String titre = ch.substring(ind + 1);
	
							CMathGrafDoc doc = new CMathGrafDoc(frame);
							doc.path = ch;
							doc.chargeDepuisFlux(inps);
							cadre = new MtgInternalFrame(frame, "", doc, MtgInternalFrame.largeurNouvelleFigure, MtgInternalFrame.hauteurNouvelleFigure);
							cadre.setTitle(titre);
							cadre.path = file.getAbsolutePath();
							cadre.gestionnaire.initialise();
							// cadre.setVisible(true);
							frame.mtgDesktopPane.add(cadre);
							inps.close();
							// On adapte les icnes et le menu au document charg
							// Test
							// frame.adapteMenuADocument(doc);  // Version 4.1.2 : Inutile car fait dans metAJoutIconesEtMenu
							// Test
							// frame.metAJourIconesEtMenus(doc);  // Version 4.1.2 : Inutile car fait dans MtgInternalFrame.internalFrameActivated
							// Modification version 3.5.1
							frame.mtgDesktopPane.getDesktopManager().activateFrame(cadre);
							try {
								cadre.setSelected(true);
							} catch (PropertyVetoException e1) {
							}
							Dimension sizefen = new Dimension(doc.sizeWindowx, doc.sizeWindowy);
							// Attention : dans la version C++ on enregistrait dans le flux la taille totale de la fentre
							// Dans la version java on enregistree que la taille de la figure (la taille du MtgFigurePanel)
							if (doc.ancienneVersion) cadre.setPreferredSize(sizefen);
							else cadre.paneCenter.setPreferredSize(sizefen); // Modifi version 3.5
							cadre.pack();
							doc.sizeWindowx = cadre.paneFigure.getWidth();
							doc.sizeWindowy = cadre.paneFigure.getHeight();
							int width = cadre.getWidth();
							int height = cadre.getHeight();
							boolean b = frame.preparePositionPourFenenetre(new Dimension(width, height));
							if (b) {
								cadre.reshape(frame.xfenetre, frame.yfenetre, width, height);
							}
							else {
								try {
									cadre.setMaximum(true);
								} catch (PropertyVetoException e1) {
									frame.mtgDesktopPane.getDesktopManager().maximizeFrame(cadre);
								}
							}
							if (frame.outilActif != null) frame.outilActif.select(); // Ncessaire dans le cas o des objets sont fichs
							// On cre si ncessaire un item de menu dans le menu Fenetres pour activer cete fentre
							int nfen = frame.nombreCadresOuverts();
							if (nfen <= MtgFrame.nombreFenetresDansMenu) {
								MtgMenuFileItem item = new MtgMenuFileItem(cadre);
								frame.menuFenetre.add(item);
							}
							else if (nfen == MtgFrame.nombreFenetresDansMenu + 1) frame.menuFenetre.add(frame.itemChoixFenetre);
							// Les deux lignes suivantes rajoutes version 4.4
							if (cadre.listePr.nombreObjetsParNature(NatureObjet.NMacro) != 0) frame.activeOutilExecutionMacro();
							else frame.activeOutilCapture();
						} catch (IOException ioerr) {
							frame.dialogueAvertissement(getString("ch35"));
							return;
						}
					} catch (FileNotFoundException err) {
						frame.dialogueAvertissement(getString("ch35"));
						return;
					}
				} else {
					frame.activeCadre(cad);
				}
			}
    }
	}
}
