/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import objets.CPrototype;
import entreesSorties.MathGraphDataInputStream;
import mathgraph32.Global;
import mathgraph32.MtgFrame;

/**
 * Classe affichant une bote de dialogue d'ouverture de fichier classique (fichier mgc de prototype)
 * et affichant dans deux champs les caractristiques du prototype
 * @author Yves
 *
 */
@SuppressWarnings("serial")
public class ChoixPrototypeDansRepertoireDlg extends MtgDlg {
	public JFileChooser fileChooser;
	private MtgLabel label1, label2;
	private MtgTextField infoNomPrototype;
	private MtgTextArea infoPrototype;
	private JScrollPane scrollPaneInfo;
	
	class MyFilter extends javax.swing.filechooser.FileFilter {
		public boolean accept(File file) {
			if (file.isDirectory())
				return true;
			String filename = file.getName().toLowerCase();
			return filename.endsWith(Global.GLOBAL_COMMA_MGC);
		}

		public String getDescription() {
			return Global.GLOBAL_STAR_COMMA_MGC;
		}
	}
	
  public ChoixPrototypeDansRepertoireDlg(MtgFrame frameInit) {
    super (getString("choixPrototypeDansRepertoireDlg1"), true, frameInit);
    initComponents();
		fileChooser.setFileFilter(new MyFilter());
    // fileChooser.setAcceptAllFileFilterUsed(true);
    fileChooser.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
          if (evt.getActionCommand().equals(JFileChooser.CANCEL_SELECTION)){
              dispose();
          }else
          	if (evt.getActionCommand().equals(JFileChooser.APPROVE_SELECTION)){
           	 		setRetourOK(true);
           	 		dispose();
        }
      }
    });
    fileChooser.addPropertyChangeListener(new PropertyChangeListener() {

			public void propertyChange(PropertyChangeEvent evt) {
				// TODO Auto-generated method stub
				if (evt.getPropertyName().equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY)) metAJourInfos();
			}
  	});
    pack();
  }
  
  private void initComponents() {
  	label1 = new MtgLabel(frame, getString("choixPrototypeDansRepertoireDlg2"));
  	label2 = new MtgLabel(frame, getString("Infos"));
  	infoNomPrototype = new MtgTextField(frame, 30);
    infoPrototype = new MtgTextArea(frame);
    scrollPaneInfo = new JScrollPane(infoPrototype, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    infoPrototype.setEditable(false);
    infoPrototype.setColumns(20);
    infoPrototype.setEditable(false);
    infoPrototype.setLineWrap(false);
    infoPrototype.setRows(4);
    // Modification version 5.0 pour tenir compte des crans haute dfinition
   double k = MtgFrame.coefMultForDlgBoxIcons;
    scrollPaneInfo.setPreferredSize(new Dimension((int)(k*450), (int)(k*90)));    
  	
  	fileChooser = new JFileChooser(path());
  	fileChooser.setFileView(new MtgFileView());
  	fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
    
    JPanel panePrincipal = new JPanel();
    JPanel paneBas = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneBas.setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 100, 60, GridBagConstraints.NONE, GridBagConstraints.WEST);
    panePrincipal.add(fileChooser, constraints);

    setConstraints(constraints, 0, 0, 1, 1, 30, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(label1, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 70, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneBas.add(infoNomPrototype, constraints);
    setConstraints(constraints, 0, 1, 2, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(label2, constraints);
    setConstraints(constraints, 0, 2, 2, 1, 0, 60, GridBagConstraints.BOTH, GridBagConstraints.WEST);
    paneBas.add(scrollPaneInfo, constraints);

    setConstraints(constraints, 0, 1, 1, 1, 100, 40, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
    panePrincipal.add(paneBas, constraints);
    
    setContentPane(panePrincipal);
  }
  	
  private void metAJourInfos() {
		File file = fileChooser.getSelectedFile();
		if (file != null) {
			FileInputStream fis;
			try {
				fis = new FileInputStream(file);
				MathGraphDataInputStream inps = new MathGraphDataInputStream(fis);
				CPrototype proto = new CPrototype();
				try {
					proto.read(inps);
					inps.close();
					infoNomPrototype.setText(proto.nom);
					infoPrototype.setText(proto.commentaire);
					infoPrototype.setCaretPosition(0);
				} catch (IOException ioerr) {
					infoNomPrototype.setText("");
					infoPrototype.setText(getString("ch146"));
					return;
				}
			} catch (FileNotFoundException e1) {
				infoNomPrototype.setText("");
				infoPrototype.setText(getString("ch146"));
				return;			
			}
		}
		else {
				infoNomPrototype.setText("");
				infoPrototype.setText("");
			
		}
	}
  
  /**
   * Chemin d'accs au dossier o on va chercher les constructions
   * @return
   */
  protected String path() {
  	return frame.pref_constDirectory;
  }
	
}