
/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.util.Vector;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import mathgraph32.MtgFrame;
import objets.CListeObjets;
import objets.CTranslationParCoord;
import objets.CValeurAssocieeAVariable;

@SuppressWarnings("serial")
public class CoordonneesVecteurTranslationDlg extends MtgDlg {
	private boolean modification; // True si on modifie les coordonnes d'un point dj cr
	private CTranslationParCoord translationAssociee;
	public ListeReperes listeReperes;
  private EditeurValeurDynamique editeurx;
  private EditeurValeurDynamique editeury;
  private MtgButton btnOK, btnCancel;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsAbs;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsOrd;
  private CValeurAssocieeAVariable valx;
  private CValeurAssocieeAVariable valy;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;

  public CoordonneesVecteurTranslationDlg(MtgFrame frameInit, CTranslationParCoord translationAssocieeInit, boolean modificationInit) {
    super(getString("coordonneesVecteurDlg1"), true, frameInit);
    CListeObjets liste = frame.cadreActif().listePr;
    setResizable(false);
    translationAssociee = translationAssocieeInit;
    modification = modificationInit;
    valx = new CValeurAssocieeAVariable(translationAssocieeInit.x);
    valy = new CValeurAssocieeAVariable(translationAssocieeInit.y);
    MtgLabel label1 = new MtgLabel(frame, getString("choixRepere"));
    MtgLabel label2 = new MtgLabel(frame, getString("coordonneesVecteurDlg2"));
    MtgLabel label3 = new MtgLabel(frame, getString("coordonneesVecteurDlg3"));
    editeurx = new EditeurValeurDynamique(frame, valx, translationAssociee);
    if (modification) editeurx.setText(translationAssocieeInit.x.chaineInfo());
    editeury = new EditeurValeurDynamique(frame, valy, translationAssociee);
    if (modification) editeury.setText(translationAssocieeInit.y.chaineInfo());
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setPreferredSize(btnCancel.getPreferredSize());
    int indmax;
    if (modification) indmax = liste.indexOf(translationAssociee) - 1;
    else indmax = liste.size() - 1;
    paneListeValeurEtFonctionsAbs = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurx, true, indmax, false);
    paneListeValeurEtFonctionsOrd = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeury, true, indmax, false);
    int indiceObj;
    if (modification) indiceObj = liste.indexOf(translationAssociee);
    else indiceObj = -1;
    listeReperes = new ListeReperes(frame, frame.cadreActif().listePr, indiceObj);
    if (modification) listeReperes.setSel(translationAssociee.rep);
    listeReperes.setBorder(BorderFactory.createEmptyBorder(1,5,1,5));
    JScrollPane scrollList = new JScrollPane(listeReperes, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());

    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 2;
    constraints.insets.top = 0;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 15, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label2, constraints);
    setConstraints(constraints, 1, 0, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(editeurx, constraints);
    setConstraints(constraints, 2, 0, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(btnOK, constraints);
    setConstraints(constraints, 1, 1, 1, 1, 0, 15, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(paneListeValeurEtFonctionsAbs, constraints);

    setConstraints(constraints, 0, 2, 1, 1, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label3, constraints);
    setConstraints(constraints, 1, 2, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(editeury, constraints);
    setConstraints(constraints, 2, 1, 1, 1, 30, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(btnCancel, constraints);
    setConstraints(constraints, 1, 3, 1, 1, 0, 15, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(paneListeValeurEtFonctionsOrd, constraints);
    
    setConstraints(constraints, 0, 4, 1, 1, 0, 15, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    setConstraints(constraints, 1, 4, 1, 1, 0, 40, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(scrollList, constraints);

     
    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(6);
    order.add(editeurx);
    order.add(editeury);
    order.add(btnOK);
    order.add(paneListeValeurEtFonctionsAbs);
    order.add(paneListeValeurEtFonctionsOrd);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    
    pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
    setContentPane(pane);    
    
    getRootPane().setDefaultButton(btnOK);  
    editeurx.setSelectionStart(0);
    editeurx.setSelectionEnd(editeurx.getText().length());
    pack();
  }
  
  private void btnOKActionPerformed() {
    if (editeurx.verifieValidite()) {
    	if (editeury.verifieValidite()) {
        setRetourOK(true);
        // On abandonne les objets CValeurAssocieeAVariable de pointAssocie pour les remplacer par les locaaux
        translationAssociee.x = valx;
        translationAssociee.y = valy;
        translationAssociee.rep = listeReperes.repereChoisi();
        dispose();
    	}
    	else editeury.requestFocus();
    }
    else editeurx.requestFocus();  	
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
}
