/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.File;
import java.io.IOException;
import java.util.Vector;
import javax.imageio.ImageIO;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JPanel;

import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.Global;
import mathgraph32.MtgFrame;
import objets.CImage;

@SuppressWarnings("serial")
public class ImageDlg extends MtgDlg {
	private JFileChooser fc; // Ajout version 4.4 car les images sont dsormais enregistres dans le flux.
	private CImage image;
	private boolean modification;
	private MtgButton btnOK, btnCancel;
	private PaneModeAffichage paneModeAffichage;
	private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
	
	class MyFilter extends javax.swing.filechooser.FileFilter {
		public boolean accept(File file) {
			if (file.isDirectory())
				return true;
			String filename = file.getName().toLowerCase();
			return filename.endsWith(Global.GLOBAL_JPG) || filename.endsWith(Global.GLOBAL_JPEG)
				|| filename.endsWith(Global.GLOBAL_GIF)	|| filename.endsWith(Global.GLOBAL_TIF) 
				|| filename.endsWith(Global.GLOBAL_TIFF) || filename.endsWith(Global.GLOBAL_PNG);
		}
		public String getDescription() {
			return Global.GLOBAL_ALL_GRAPHIC_EXTENSIONS;
		}
	}	
	
  public ImageDlg(MtgFrame frameInit, CImage imageInit, boolean modificationInit) {
    super (getString("imageDlg1"), true, frameInit);
    image = imageInit;
    modification = modificationInit;
    initComponents();
    if (modification) getRootPane().setDefaultButton(btnOK);    
  }
	
	public void initComponents() {
		if (!modification) {
			fc = new JFileChooser(frame.repertoireFichiers);
			fc.setApproveButtonText(getString("btnChoisir"));
			fc.setFileFilter(new MyFilter());
			// On ajoute un panneau de visualisation des fichiers image
			fc.setAccessory(new ImagePreview(fc));
	    fc.addActionListener(new java.awt.event.ActionListener() {
	      public void actionPerformed(java.awt.event.ActionEvent evt) {
	          if (evt.getActionCommand().equals(JFileChooser.CANCEL_SELECTION)){
	              dispose();
	          }else
	          	if (evt.getActionCommand().equals(JFileChooser.APPROVE_SELECTION)){
	           	 		setImage();
	        }
	      }
	    });
		}
		
	  paneModeAffichage = new PaneModeAffichage(frame, image, true); // Modifi version 4.5.2
	  btnOK = new MtgButton(frame, getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      setImage();
	    }
	  });    
	  btnCancel = new MtgButton(frame, getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());
    JPanel panePrincipal = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 5;
		constraints.insets.top = 5;
		constraints.insets.left = 5;
		constraints.insets.right = 5;

		int line = 0;
		if (!modification) {
			setConstraints(constraints, 0, 0, 2, 1, 60, 50, GridBagConstraints.NONE, GridBagConstraints.WEST);
			panePrincipal.add(fc, constraints);
			line++;
		}
		setConstraints(constraints, 0, line++, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneModeAffichage, constraints);
    if (modification) {
	    JPanel paneRight = new JPanel();
	    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
	    paneRight.add(btnOK);
	    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
	    paneRight.add(btnCancel);
	 		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
			panePrincipal.add(paneRight, constraints);	    
    }
		
		setContentPane(panePrincipal);		
		// On change l'ordre par dfaut des composants
		Vector<Component> order = new Vector<Component>(4);
		order.add(paneModeAffichage.checkEffacementFond);
		order.add(paneModeAffichage.radioSansEncadrement);
		order.add(paneModeAffichage.radioEncadrementSimple);
		order.add(paneModeAffichage.radioEncadrement3D);
		mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
		setFocusTraversalPolicy(mtgFocusTraversalPolicy);
		pack();
		
	}

  private void btnCancelActionPerformed() {
  	dispose();
  }
 
  private void setImage() {
  	if (!modification) {
			File file = fc.getSelectedFile();
	  	try {
				image.image = ImageIO.read(file);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				dialogueAvertissement(getString("echecImportationImage"));
				return;
			}
			String fileName = file.getName();
			image.natImage = MathGraphDataOutputStream.getNatImage(fileName);
  	}
		image.encadrement = paneModeAffichage.styleEncadrement();
		image.effacementFond = paneModeAffichage.effacementFond();
		image.couleurFondAffichage = paneModeAffichage.couleurFond();
  	image.rectangleAffichage.width = image.image.getWidth(null);
   	image.rectangleAffichage.height = image.image.getHeight(null);		
		// Ajouts version 4.5
		image.alignementHorizontal = paneModeAffichage.alignementHorizontal();
		image.alignementVertical = paneModeAffichage.alignementVertical();
		//
  	setRetourOK(true);
  	dispose();  	
 	
  }
}
