/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CListeObjets;
import objets.CMacroModificationVariable;
import objets.CValeurAssocieeAVariable;
import objets.CVariableBornee;
import types.NatureCarSpe;

@SuppressWarnings("serial")
public class MacroModificationVariableDlg extends MtgDlg {
	private boolean modification;
	private CMacroModificationVariable macro;
	private MtgLabel label1, label2, label3, label4, label5, label6, label7;
	private TextFieldAvecCarSpeciaux	editeurIntitule;
	private MtgTextArea editeurCommentaire;
	private EditeurValeurDynamique editeurValeurMini;
	private ListeVariables listeVariables;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsValMini;
	private EditeurValeurDynamique editeurValeurMaxi;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsValMaxi;
	private EditeurValeurDynamique editeurValeurActuelle;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsValActuelle;
	private EditeurValeurDynamique editeurValeurPas;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctionsValPas;
	private PaneTaillePolice paneTaillePolice;
	private PaneModeAffichagePourMacro paneModeAffichage;
	private MtgButton btnOK, btnCancel;
	private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
	// Les 4 objets suivants pour stocker temporairement les donnes choisies en cas d'annulation
  private CValeurAssocieeAVariable valMini, valMaxi, valPas, valActuelle;

  public MacroModificationVariableDlg(MtgFrame frameInit, CMacroModificationVariable macroInit, boolean modificationInit) {
    super (getString("macroModificationVariableDlg1"), true, frameInit);
    macro = macroInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);
  }
  
  private void initComponents() {
    valMini = new CValeurAssocieeAVariable(macro.valeurMiniAssociee);
    valMaxi = new CValeurAssocieeAVariable(macro.valeurMaxiAssociee);
    valPas = new CValeurAssocieeAVariable(macro.valeurPasAssociee);
    valActuelle = new CValeurAssocieeAVariable(macro.valeurActuelleAssociee);

		label1 = new MtgLabel(frame, getString("macroApparitionDlg2"));
		label2 = new MtgLabel(frame, getString("macroApparitionDlg3"));
		label3 = new MtgLabel(frame, getString("macroModificationVariableDlg2"));
		label4 = new MtgLabel(frame, getString("macroModificationVariableDlg3"));
		label5 = new MtgLabel(frame, getString("macroModificationVariableDlg4"));
		label6 = new MtgLabel(frame, getString("macroModificationVariableDlg5"));
		label7 = new MtgLabel(frame, getString("macroModificationVariableDlg6"));
		editeurIntitule = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
		editeurIntitule.setColumns(30); // Ajout version 4.9.2 pour viter que la fentre dgnre quand on agit sur le PaneModeAffichagePourMacro
		editeurCommentaire = new MtgTextArea(frame);
		editeurCommentaire.setColumns(20);
		editeurCommentaire.setLineWrap(false);
		editeurCommentaire.setRows(4);
    JScrollPane scrollPaneCommentaire = new JScrollPane(editeurCommentaire,
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    // Modification version 5.0 pour tenir compte des crans haute dfinition
    double k = MtgFrame.coefMultForDlgBoxIcons;
    scrollPaneCommentaire.setPreferredSize(new Dimension((int)(k*300), (int)(k*90)));
    editeurValeurMini = new EditeurValeurDynamique(frame, valMini, macro);
    editeurValeurMaxi = new EditeurValeurDynamique(frame, valMaxi, macro);
    editeurValeurPas = new EditeurValeurDynamique(frame, valPas, macro);
    editeurValeurActuelle = new EditeurValeurDynamique(frame, valActuelle, macro);
    
    CListeObjets liste = frame.cadreActif().listePr;
    listeVariables = new ListeVariables(frame, liste, liste.size()-1, false);
    listeVariables.setVisibleRowCount(5);
    JScrollPane scrollPaneListeVariables = new JScrollPane(listeVariables, 
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    
    if (modification)  {
    	editeurIntitule.setText(macro.intitule);
    	editeurCommentaire.setText(macro.commentaireMacro);
			editeurCommentaire.setCaretPosition(0);
    	editeurValeurMini.setText(macro.valeurMiniAssociee.chaineInfo());
     	editeurValeurMaxi.setText(macro.valeurMaxiAssociee.chaineInfo());
     	editeurValeurPas.setText(macro.valeurPasAssociee.chaineInfo());
     	editeurValeurActuelle.setText(macro.valeurActuelleAssociee.chaineInfo());
     	listeVariables.setSel(macro.variableAssociee);
    }
    
    
    btnOK = new MtgButton(frame, getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new MtgButton(frame, getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getMaximumSize());
  	int indmax;
  	if (modification) indmax = frame.cadreActif().listePr.indexOf(macro) - 1;
  	else indmax = frame.cadreActif().listePr.size() - 1;
    paneListeValeurEtFonctionsValMini = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurMini, true, indmax, false);
    paneListeValeurEtFonctionsValMaxi = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurMaxi, true, indmax, false);
    paneListeValeurEtFonctionsValPas = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurPas, true, indmax, false);
    paneListeValeurEtFonctionsValActuelle = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurValeurActuelle, true, indmax, false);

	  paneTaillePolice = new PaneTaillePolice(macro.indiceTaillePolice);
	  paneModeAffichage = new PaneModeAffichagePourMacro(frame, macro);
    
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    JPanel paneLeft = new JPanel();
    JPanel paneBas = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    paneBas.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		
		// paneMilieu est le JPanel qui contient l'diteur de valeur avec dessous le bouton
		// de liste des valeurs et  droite un JLabel avec au-dessous la liste des variables
		paneLeft.setLayout(new GridBagLayout());
		setConstraints(constraints, 0, 0, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneLeft.add(label3, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 60, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneLeft.add(editeurValeurMini, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneLeft.add(paneListeValeurEtFonctionsValMini, constraints);
		setConstraints(constraints, 0, 2, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneLeft.add(label4, constraints);
		setConstraints(constraints, 1, 2, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneLeft.add(editeurValeurMaxi, constraints);
		setConstraints(constraints, 1, 3, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneLeft.add(paneListeValeurEtFonctionsValMaxi, constraints);
		setConstraints(constraints, 0, 4, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneLeft.add(label5, constraints);
		setConstraints(constraints, 1, 4, 1, 1, 60, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneLeft.add(editeurValeurPas, constraints);
		setConstraints(constraints, 1, 5, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneLeft.add(paneListeValeurEtFonctionsValPas, constraints);
		setConstraints(constraints, 0, 6, 1, 1, 40, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneLeft.add(label6, constraints);
		setConstraints(constraints, 1, 6, 1, 1, 60, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneLeft.add(editeurValeurActuelle, constraints);
		setConstraints(constraints, 1, 7, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneLeft.add(paneListeValeurEtFonctionsValActuelle, constraints);
		
		setConstraints(constraints, 0, 0, 1, 2, 25, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneBas.add(paneTaillePolice, constraints);
		setConstraints(constraints, 1, 0, 1, 2, 25, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneBas.add(paneModeAffichage, constraints);
		setConstraints(constraints, 2, 0, 1, 1, 25, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(label7, constraints);
		setConstraints(constraints, 2, 1, 1, 1, 0, 30, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneBas.add(scrollPaneListeVariables, constraints);
		
		
		setConstraints(constraints, 0, 0, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 60, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurIntitule, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneCommentaire, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 0, 50, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneLeft, constraints);
		setConstraints(constraints, 0, 3, 2, 1, 0, 20, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneBas, constraints);
		
		setConstraints(constraints, 2, 0, 1, 5, 40, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.NORTHEAST);
		panePrincipal.add(paneRight, constraints);

		setContentPane(panePrincipal);
		// On change l'ordre par dfaut des composants
		Vector<Component> order = new Vector<Component>(9);
		order.add(editeurIntitule);
		order.add(editeurCommentaire);
		order.add(editeurValeurMini);
		order.add(editeurValeurMaxi);
		order.add(editeurValeurPas);
		order.add(editeurValeurActuelle);
		order.add(listeVariables);
		order.add(btnOK);
		order.add(btnCancel);
		mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
		setFocusTraversalPolicy(mtgFocusTraversalPolicy);
		pack();			    
  	
  }
  
  private void btnOKActionPerformed() {
  	String chintitule = editeurIntitule.getText();
  	if (chintitule.length() == 0) {
  		dialogueAvertissement(getString("ch115"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	CListeObjets liste = frame.cadreActif().listePr;
  	if (!liste.validationIntituleMacro(chintitule, macro)) {
  		dialogueAvertissement(getString("ch116"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	CVariableBornee variableChoisie = listeVariables.variableSelectionnee();
    if (editeurValeurMini.verifieValidite()) {
    	if (editeurValeurMaxi.verifieValidite()) {
      	if (editeurValeurPas.verifieValidite()) {
        	if (editeurValeurActuelle.verifieValidite()) {
        		if	(valMini.pointeSur(variableChoisie))
        		{
        			dialogueAvertissement(getString("ch126"));
        			editeurValeurMini.requestFocus();
        			return;
        		}
        		if	(valMaxi.pointeSur(variableChoisie))
        		{
        			dialogueAvertissement(getString("ch126"));
        			editeurValeurMaxi.requestFocus();
        			return;
        		}
        		if	(valPas.pointeSur(variableChoisie))
        		{
        			dialogueAvertissement(getString("ch126"));
        			editeurValeurPas.requestFocus();
        			return;
        		}
        		if	(valActuelle.pointeSur(variableChoisie))
        		{
        			dialogueAvertissement(getString("ch126"));
        			editeurValeurActuelle.requestFocus();
        			return;
        		}        		
        		MtgInternalFrame cadre = frame.cadreActif();
        		valMini.positionne(false, cadre.dimf);
        		valMaxi.positionne(false, cadre.dimf);
        		valPas.positionne(false, cadre.dimf);
        		valActuelle.positionne(false, cadre.dimf);
        		
        		if (
      					(valMini.valeur >= valMaxi.valeur)
      					||	(valPas.valeur > (valMaxi.valeur - valMini.valeur))
      					||	(valActuelle.valeur < valMini.valeur)
      					||	(valActuelle.valeur > valMaxi.valeur))
        		{
        			if (!frame.dialogueConfirmation_YES_NO(getString("ch127"))) {
        				editeurValeurMini.requestFocus();
        				return;
        			}
        		}
        		
		        // On abandonne les objets CValeurAssocieeAVariable de pointAssocie pour les remplacer par les locaux
		        macro.intitule = chintitule;
		        macro.commentaireMacro = editeurCommentaire.getText();
		      	macro.indiceTaillePolice = paneTaillePolice.indiceTaille();
		      	macro.effacementFond = paneModeAffichage.effacementFond();
		      	macro.couleurFondAffichage = paneModeAffichage.couleurFond();		        
		    		// Ajouts version 4.5
		    		macro.alignementHorizontal = paneModeAffichage.alignementHorizontal();
		    		macro.alignementVertical = paneModeAffichage.alignementVertical();
		    		//
		        macro.variableAssociee = listeVariables.variableSelectionnee();
		        macro.valeurMiniAssociee = valMini;
		        macro.valeurMaxiAssociee = valMaxi;
		        macro.valeurPasAssociee = valPas;
		        macro.valeurActuelleAssociee = valActuelle;
		    		frame.pref_IndiceTaillePoliceAffichages = macro.indiceTaillePolice;
		    		frame.cadreActif().couleurFondAffichage = macro.couleurFondAffichage;
		        setRetourOK(true);
		        dispose();
        	}
      	}
    	}
    }  	
  	
  }

  private void btnCancelActionPerformed() {
  	dispose();
  }
  

  
}
