/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URL;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import outils.OutilProtocole;
import objets.CBarycentre;
import objets.CElementBase;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CTransformation;
import objets.CValeurDynamique;
import types.NatureCalcul;
import types.NatureObjet;
import mathgraph32.MtgFrame;
import mathgraph32.MtgImageIcon;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class ProtocoleDialog extends MtgDlg {
	private ListeTousObjets listeTousObjets;
	private JButton btnHaut,btnBas,btnDebut,btnFin,btnRecHaut, btnRecBas,btnRecDebut,btnRecFin;
	private MtgButton btnFermer;
	// private JTextArea info;
	private CListeObjets listeObjetsNommes = new CListeObjets();
  private MtgTextArea textArea;
  private JButton btnSupprimer,btnSupprimerJusqueFin,btnModifierObjet,btnGomme,btnRideau;
  private MtgCheckBox checkAffObjInterm;
  private MtgCheckBox checkAffichagePartiel;
  private MtgCheckBox checkAffichageObjetsCaches;
  public static Insets insetsZero = new Insets(0,0,0,0);
  private MtgInternalFrame cadre;
  private JPanel paneBoutonsReclassement;
  private boolean reclassementsEffectues;

  public ProtocoleDialog(MtgFrame frameInit) {
  	super(getString("menuProtocole"), true, frameInit);
  	reclassementsEffectues = false;
  	frame.montreBarreOutil(false); // On cache la barre d'outils de gauche pour avoir de la place
  	cadre = frame.cadreActif();
		CListeObjets liste = cadre.listePr;
		liste.genereNoms(listeObjetsNommes, false);
    initComponents();
    getRootPane().setDefaultButton(btnFermer);
    initialiseSelection();
    metAJourInfo();
		cadre.redessineSansCalcul();
    addWindowListener(new java.awt.event.WindowAdapter() {
      public void windowClosing(java.awt.event.WindowEvent evt) {
        fermeture();
      } 
    });
  }
  
  private void initialiseSelection() {
    int ind = listeTousObjets.pointeurs.size() - 1;
		listeTousObjets.setSelectedIndex(ind);
		listeTousObjets.ensureIndexIsVisible(ind);
  }
  
  private void initComponents() {
		CListeObjets liste = cadre.listePr;
		// Modification pour tenir compte des crans haute dfition
		double k = MtgFrame.coefMultForDlgBoxIcons/2; // Pour les icnes flches qui sont de petites dimensions 16 x 16
    double k2 = MtgFrame.coefMultForDlgBoxIcons; // Pour la taille des champs d'dition
    if (k2 > 2.25) k2 = 2.25; // Pas trop grand quand mme.
    double k3 = MtgFrame.coefMultForDlgBoxIcons; // Pour la taille des champs d'dition
   //
		URL adr = MtgFrame.class.getResource("images/UpArrow16.gif");    
		btnHaut = new JButton(new MtgImageIcon(k3, adr));
    btnHaut.setToolTipText(getString("tipPrecedent"));
		btnHaut.addActionListener(new java.awt.event.ActionListener() {
		  public void actionPerformed(java.awt.event.ActionEvent evt) {
		  	btnHautActionPerformed();
		  }
		});
		adr = MtgFrame.class.getResource("images/DownArrow16.gif");    
		btnBas = new JButton(new MtgImageIcon(k3, adr));
    btnBas.setToolTipText(getString("tipSuivant"));
		btnBas.addActionListener(new java.awt.event.ActionListener() {
		  public void actionPerformed(java.awt.event.ActionEvent evt) {
		    btnBasActionPerformed();
		  }
		});
    adr = MtgFrame.class.getResource("images/UpMaxArrow16.gif");    
    btnDebut = new JButton(new MtgImageIcon(k3, adr));
    btnDebut.setToolTipText(getString("tipDebut"));
    btnDebut.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	btnDebutActionPerformed();
      }
    });
    adr = MtgFrame.class.getResource("images/DownMaxArrow16.gif");
    btnFin = new JButton(new MtgImageIcon(k3, adr));
    btnFin.setToolTipText(getString("tipFin"));
    btnFin.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	btnFinActionPerformed();
      }
    });
		btnFermer = new MtgButton(frame, getString("btnFermer"));
		btnFermer.addActionListener(new java.awt.event.ActionListener() {
		  public void actionPerformed(java.awt.event.ActionEvent evt) {
		    btnFermerActionPerformed();
		  }
		});
		adr = MtgFrame.class.getResource("images/Up16.gif");    
		btnRecHaut = new JButton(new MtgImageIcon(k3, adr));
		btnRecHaut.setToolTipText(getString("tipRecHaut"));
		btnRecHaut.addActionListener(new java.awt.event.ActionListener() {
		  public void actionPerformed(java.awt.event.ActionEvent evt) {
		  	btnRecHautActionPerformed();
		  }
		});
		adr = MtgFrame.class.getResource("images/Down16.gif");    
		btnRecBas = new JButton(new MtgImageIcon(k3, adr));
		btnRecBas.setToolTipText(getString("tipRecBas"));
		btnRecBas.addActionListener(new java.awt.event.ActionListener() {
		  public void actionPerformed(java.awt.event.ActionEvent evt) {
		    btnRecBasActionPerformed();
		  }
		});
    adr = MtgFrame.class.getResource("images/UpMax16.gif");    
    btnRecDebut = new JButton(new MtgImageIcon(k3, adr));
    btnRecDebut.setToolTipText(getString("tipRecHautMax"));
    btnRecDebut.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	btnRecDebutActionPerformed();
      }
    });
    adr = MtgFrame.class.getResource("images/DownMax16.gif");
    btnRecFin = new JButton(new MtgImageIcon(k3, adr));
    btnRecFin.setToolTipText(getString("tipRecBasMax"));
    btnRecFin.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
      	btnRecFinActionPerformed();
      }
    });
    adr = MtgFrame.class.getResource("images/ModificationObjetGraphique.png");    
    btnModifierObjet = new JButton(new MtgImageIcon(k, adr));
    btnModifierObjet.setMargin(insetsZero);
    btnModifierObjet.setToolTipText(getString("tipModifierObjet"));
    btnModifierObjet.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnModifierObjetActionPerformed();
      }
    });
    adr = MtgFrame.class.getResource("images/Delete24.png");    
    btnSupprimer = new JButton(new MtgImageIcon(k, adr));
    btnSupprimer.setMargin(insetsZero);
    btnSupprimer.setToolTipText(getString("tipSuppressionObjet"));
    btnSupprimer.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnSupprimerActionPerformed();
      }
    });
    adr = MtgFrame.class.getResource("images/SuppressionJusqueFin.png");    
    btnSupprimerJusqueFin = new JButton(new MtgImageIcon(k, adr));
    btnSupprimerJusqueFin.setMargin(insetsZero);
    btnSupprimerJusqueFin.setToolTipText(getString("tipSuppressionJusqueFin"));
    btnSupprimerJusqueFin.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnSupprimerJusqueFinActionPerformed();
      }
    });
    adr = MtgFrame.class.getResource("images/Gomme.png");    
    btnGomme = new JButton(new MtgImageIcon(k, adr));
    btnGomme.setMargin(insetsZero);
    btnGomme.setToolTipText(getString("btnMasquer"));
    btnGomme.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnGommeActionPerformed();
      }
    });
    adr = MtgFrame.class.getResource("images/Rideau.png");    
    btnRideau = new JButton(new MtgImageIcon(k, adr));
    btnRideau.setMargin(insetsZero);
    btnRideau.setToolTipText(getString("btnRendreVisible"));
    btnRideau.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnRideauActionPerformed();
      }
    });
    checkAffObjInterm = new MtgCheckBox(frame, getString("ProtocoleDlg2")); 
    checkAffObjInterm.setSelected(false);
    checkAffObjInterm.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				CListeObjets liste = cadre.listePr; 
				// liste.effaceNoms(listeObjetsNommes); // Modfi version 5.1
       	listeObjetsNommes.effaceNoms();
				boolean affObjInterm = checkAffObjInterm.isSelected();
				listeObjetsNommes.retireTout();
				liste.genereNoms(listeObjetsNommes, affObjInterm);
				listeTousObjets.objetsIntermediaires = affObjInterm;
				listeTousObjets.metAJourListe();
				initialiseSelection();
				frame.outilPointageProtocole.affichageObjetsInterm = affObjInterm;
				frame.cadreActif().redessineSansCalcul();
			}
		});
    checkAffichagePartiel = new MtgCheckBox(frame, getString("ProtocoleDlg3"));
    checkAffichagePartiel.setSelected(true);
    checkAffichagePartiel.addActionListener(new ActionListener() {
			
			public void actionPerformed(ActionEvent e) {
				frame.outilPointageProtocole.affichagePartiel = checkAffichagePartiel.isSelected();
				frame.cadreActif().redessineSansCalcul();
			}
		});
    checkAffichageObjetsCaches = new MtgCheckBox(frame, getString("ProtocoleDlg4"));
    checkAffichageObjetsCaches.setSelected(false);
    checkAffichageObjetsCaches.addActionListener(new ActionListener() {		
			public void actionPerformed(ActionEvent e) {
				frame.outilPointageProtocole.affichageObjetsCaches = checkAffichageObjetsCaches.isSelected();
				frame.cadreActif().redessineSansCalcul();
			}
		});
		listeTousObjets = new ListeTousObjets(frame, liste);
    JScrollPane scrollPaneListeObjets = new JScrollPane(listeTousObjets, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    		JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
    // Modification version 5.0 pour tenir compte des crans haute rsolution
    scrollPaneListeObjets.setPreferredSize(new Dimension((int) (k2*450),(int) (450*k2)));
    listeTousObjets.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent evt) {
				listeObjetsMouseClicked();
		    if (evt.getClickCount() == 2) {
		    	CElementBase elb = listeTousObjets.selectedObject();
		    	if (elb != null) { 
			  		if (elb.modifiableParMenu()) btnModifierObjetActionPerformed();
		    	}
		    }
			}

			public void mouseEntered(MouseEvent evt) {
			}

			public void mouseExited(MouseEvent evt) {
			}

			public void mousePressed(MouseEvent evt) {
			}

			public void mouseReleased(MouseEvent evt) {
			}
    });
    listeTousObjets.addListSelectionListener(new ListSelectionListener() {		
			public void valueChanged(ListSelectionEvent e) {
				metAJourInfo();
			}
		});
    listeTousObjets.addKeyListener(new KeyListener() {
			
			public void keyTyped(KeyEvent e) {
			}
			
			public void keyReleased(KeyEvent e) {
			}
			
			public void keyPressed(KeyEvent e) {
				if ((e.getKeyCode() == KeyEvent.VK_DELETE) && (btnSupprimer.isVisible()))
					btnSupprimerActionPerformed();
			}
		});
    textArea = new MtgTextArea(frame);
    textArea.setColumns(MtgFrame.screenIsHighResolution ? 50 : 25);
    textArea.setEditable(false);
    textArea.setLineWrap(true);
    textArea.setRows(5);
    JScrollPane scrollPaneInfo = new JScrollPane(textArea, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    		JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
    scrollPaneInfo.setPreferredSize(new Dimension((int)(k2*350), (int)(k2*150)));

		JPanel panePrincipal = new JPanel();
		panePrincipal.setLayout(new GridBagLayout());
		JPanel paneRight = new JPanel();
		// JPanel paneBas = new JPanel();
		paneBoutonsReclassement = new JPanel();
		paneBoutonsReclassement.setLayout(new BoxLayout(paneBoutonsReclassement, BoxLayout.Y_AXIS));
		JPanel paneBoutonsDeplacement = new JPanel();
		paneBoutonsDeplacement.setLayout(new BoxLayout(paneBoutonsDeplacement, BoxLayout.Y_AXIS));
		JPanel paneBoutons1 = new JPanel();
		JPanel paneBoutons2 = new JPanel();
		paneBoutons2.setLayout(new GridBagLayout());
		JPanel paneBoutons = new JPanel();
		paneBoutons.setLayout(new BoxLayout(paneBoutons, BoxLayout.Y_AXIS));
		paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
		paneRight.add(btnFermer);
		paneRight.add(Box.createRigidArea(new Dimension(0,5)));
		JPanel paneOptions = new JPanel();
		paneOptions.setLayout(new GridBagLayout());
		JPanel paneBas = new JPanel();
		
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		
		paneBoutonsReclassement.add(btnRecDebut);
		paneBoutonsReclassement.add(Box.createRigidArea(new Dimension(0,3)));
		paneBoutonsReclassement.add(btnRecHaut);
		paneBoutonsReclassement.add(Box.createRigidArea(new Dimension(0,3)));
		paneBoutonsReclassement.add(btnRecBas);
		paneBoutonsReclassement.add(Box.createRigidArea(new Dimension(0,3)));
		paneBoutonsReclassement.add(btnRecFin);
		paneBoutonsDeplacement.add(btnDebut);
		paneBoutonsDeplacement.add(Box.createRigidArea(new Dimension(0,3)));
		paneBoutonsDeplacement.add(btnHaut);
		paneBoutonsDeplacement.add(Box.createRigidArea(new Dimension(0,3)));
		paneBoutonsDeplacement.add(btnBas);
		paneBoutonsDeplacement.add(Box.createRigidArea(new Dimension(0,3)));
		paneBoutonsDeplacement.add(btnFin);
		paneBoutons1.add(paneBoutonsDeplacement);
		paneBoutons1.add(Box.createRigidArea(new Dimension(5,0)));
		paneBoutons1.add(paneBoutonsReclassement);
		JPanel paneGommeRideau = new JPanel();
		paneGommeRideau.add(btnGomme);
		paneGommeRideau.add(btnRideau);
		
		setConstraints(constraints, 0, 0, 1, 1, 30, 100, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneBoutons2.add(btnSupprimer, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 30, 100, GridBagConstraints.NONE, GridBagConstraints.EAST);
		paneBoutons2.add(btnSupprimerJusqueFin, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 30, 100, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneBoutons2.add(btnModifierObjet, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 30, 100, GridBagConstraints.NONE, GridBagConstraints.EAST);
		paneBoutons2.add(paneGommeRideau, constraints);
		paneBoutons.add(paneBoutons1);
		paneBoutons.add(paneBoutons2);
		setConstraints(constraints, 0, 0, 1, 1, 100, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneOptions.add(checkAffObjInterm, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 100, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneOptions.add(checkAffichagePartiel, constraints);
		setConstraints(constraints, 0, 2, 1, 1, 100, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneOptions.add(checkAffichageObjetsCaches, constraints);
		paneBas.add(scrollPaneInfo);
		paneBas.add(paneOptions);


		setConstraints(constraints, 0, 0, 1, 2, 90, 100, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneListeObjets, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 10, 0, GridBagConstraints.NONE, GridBagConstraints.NORTH);
		panePrincipal.add(btnFermer, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 10, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneBoutons, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 100, 20, GridBagConstraints.NONE, GridBagConstraints.WEST); 
		panePrincipal.add(paneBas, constraints);
    setContentPane(panePrincipal);
		pack();
		
  }
  
  public final void listeObjetsMouseClicked() {
  	metAJourInfo();
  }
  
  public final void btnRecHautActionPerformed() {
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste = cadre.listePr;
  	CElementBase elb = listeTousObjets.selectedObject();
  	if (elb != null) {
  		if (!liste.remonteVersDebutAvecDependants(elb)) Toolkit.getDefaultToolkit().beep();
  		else {
  			listeTousObjets.metAJourListe();
	  		cadre.redessineAvecCalcul();
	  		listeTousObjets.selectObject(elb);
	  		reclassementsEffectues = true;
  		}
  	}
  }
  public final void btnRecBasActionPerformed() {
  	// Ligne suivante ajout version 5.3 pour ne pas reclasser vers la fin la constante pi qui doit rester en dbut de liste
   	if (listeTousObjets.getSelectedIndex() == 0) return;
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste = cadre.listePr;
  	CElementBase elb = listeTousObjets.selectedObject();
  	if (elb != null) {
  		if (!liste.remonteVersFinAvecDependants(elb)) Toolkit.getDefaultToolkit().beep(); 
  		else {
  			listeTousObjets.metAJourListe();
	  		cadre.redessineAvecCalcul();
	  		listeTousObjets.selectObject(elb);
	  		reclassementsEffectues = true;
  		}
  	}
  }
  private final void btnRecDebutActionPerformed() {
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste = cadre.listePr;
  	CElementBase elb = listeTousObjets.selectedObject();
  	if (elb != null)
    	if (liste.reclasseVersDebutAvecDependants(elb)) {
  			listeTousObjets.metAJourListe();
	  		cadre.redessineAvecCalcul();
	  		listeTousObjets.selectObject(elb);    		
	  		reclassementsEffectues = true;
    	}
    	else Toolkit.getDefaultToolkit().beep(); 	
  }
  private final void btnRecFinActionPerformed() {
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste = cadre.listePr;
  	CElementBase elb = listeTousObjets.selectedObject();
  	if (elb != null)
    	if (liste.reclasseVersFinAvecDependants(elb)) {
  			listeTousObjets.metAJourListe();
	  		cadre.redessineAvecCalcul();
	  		listeTousObjets.selectObject(elb);    		
	  		reclassementsEffectues = true;
    	}
    	else Toolkit.getDefaultToolkit().beep(); 	
  }
  private final void btnHautActionPerformed() {
  	int ind = listeTousObjets.getSelectedIndex();
  	if(ind != 0) {
  		listeTousObjets.setSelectedIndex(ind-1);
  		listeTousObjets.ensureIndexIsVisible(ind-1); 
  	}
  }
  private final void btnBasActionPerformed() {
  	int indmax = listeTousObjets.pointeurs.size() - 1;
  	int ind = listeTousObjets.getSelectedIndex();
  	if(ind != indmax) {
  		listeTousObjets.setSelectedIndex(ind+1);
  		listeTousObjets.ensureIndexIsVisible(ind+1);
  	}
  }
  private final void btnDebutActionPerformed() {
 		listeTousObjets.setSelectedIndex(0);
		listeTousObjets.ensureIndexIsVisible(0); 
  }
  private final void btnFinActionPerformed() {
  	int indmax = listeTousObjets.pointeurs.size();
		listeTousObjets.setSelectedIndex(indmax-1);
		listeTousObjets.ensureIndexIsVisible(indmax-1); 
  }

  private final void btnFermerActionPerformed() {
  	fermeture();
  }
  
  private final void metAJourInfo() {
    CElementBase elb = listeTousObjets.selectedObject();
    if (elb == null) return;
    paneBoutonsReclassement.setVisible(!elb.estElementIntermediaire() && !elb.estDeNature(NatureObjet.NImplementationProto));
    frame.outilPointageProtocole.indiceElementEnCours = elb.index;
    String ch = elb.infoPourHistorique();
    if (!elb.estElementIntermediaire() && elb.modifiableParMenu()) btnModifierObjet.setVisible(true);
    else btnModifierObjet.setVisible(false);
    if (elb.estDeNature(NatureObjet.NToutObjet)) {
    	CElementGraphique elg = (CElementGraphique) elb;
    	if (elg.masque) {
    		ch += "\n" + getString("ch164");
    		btnGomme.setVisible(false);
    		btnRideau.setVisible(true);
    	}
    	else {
    		btnGomme.setVisible(true);
    		btnRideau.setVisible(false);
    	}
    }
    else {
    	btnGomme.setVisible(false);
  		btnRideau.setVisible(false);
    }
    if ((!elb.existe) && (elb.getNatureCalcul() == NatureCalcul.NAucunCalcul)) ch = ch + "\n" + getString("ch18");
    if (elb.estElementIntermediaire()) ch = ch + "\n" + getString("ch159");
    else if (elb.estElementFinal) ch = ch + "\n" + getString("ch160");  
    ch += "\n" + getString("Nhtml" ) + " "  + elb.index; // Ajout version 4.9.2 pour communication html
    textArea.setText(ch);
    textArea.setCaretPosition(0);
    OutilProtocole outil = frame.outilProtocole;
    outil.annuleClignotement();
    if ((elb.getNature() & NatureObjet.NToutObjet) != 0) { 
      outil.ajouteClignotementDe((CElementGraphique)elb);
      elb.ajouteAntecedents(frame, frame.listeClignotante);
      // Il faut positionner la liste clignotante car pour les translations
      // on lui a rajout des lments qui ne sont pas des objets suppmentaires
      // ne faisant pas partie de la figure
      frame.listeClignotante.positionne(false, frame.cadreActif().dimf);
      outil.resetClignotement();
    }
    else {
    	if (elb.estDeNatureCalcul(NatureCalcul.NTouteMesure)) {
        elb.ajouteAntecedents(frame, frame.listeClignotante);
        outil.resetClignotement();
    	}
    }
  }
  private final int indiceObjetEnCours() {
  	int ind = listeTousObjets.getSelectedIndex();
  	return listeTousObjets.pointeurs.get(ind).index;
  }
  private final CElementBase objetEnCours() {
  	int ind = listeTousObjets.getSelectedIndex();
  	return listeTousObjets.pointeurs.get(ind);
  	
  }
  private final void btnSupprimerActionPerformed() {
		boolean affObjInterm = checkAffObjInterm.isSelected();
  	int ind = listeTousObjets.getSelectedIndex();
    if (indiceObjetEnCours() == 0) {
      dialogueAvertissement(getString("ch24"));
      return;
    }
    MtgInternalFrame cadre = frame.cadreActif();    
    CListeObjets liste = cadre.listePr;
    CElementBase elb = objetEnCours();
    // Destruction de l'objet en cours
    CElementBase antecedentDirect = elb.antecedentDirect();
    if (antecedentDirect.getNature() == NatureObjet.NTransformation) {
    	// Si l'objet antcdent direct est une transformation, on regarde s'il y a d'autres images par
    	// cette transformation  dtruire. Si oui, on ne dtruit pas la transformation.
    	// Sinon on dtruit la transformation et ses dpendants.
    	// On peut avoir par exemple cr un point image d'un point par une transformation puis l'iamge de cette image
    	// Dans ce cas il ne faut pas dtruite la transformation
    	if (liste.nombreImagesParTransformationNonDepDe((CTransformation) antecedentDirect, elb) > 0) antecedentDirect = elb;
    }
    if (antecedentDirect.getNature() == NatureObjet.NBipoint)  {
      if (!frame.dialogueConfirmation_YES_NO(getString("ch38"))) {
      	return;
      }
    }
    
    if (liste.nombreDependants(elb) >= 2) {
      if (frame.dialogueConfirmation_YES_NO(getString("ch37"))) {
        // Correction dfaut version 4.9.9.3. Avant d'enregistrer il faut retirer les noms des points
      	// liste.effaceNoms(listeObjetsNommes); // Modifi version 5.1
       	listeObjetsNommes.effaceNoms();
        frame.retireDependants(antecedentDirect);
        cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil16"));
        // Correction dfaut version 4.9.9.3. Puis les renommer
        listeObjetsNommes.retireTout();
    		liste.genereNoms(listeObjetsNommes, affObjInterm);

        listeTousObjets.metAJourListe();
        if (ind >= listeTousObjets.pointeurs.size()) listeTousObjets.setSelectedIndex(ind-1);
        else listeTousObjets.setSelectedIndex(ind);
        cadre.effacegbuffer();
        cadre.repaintFigureSansCalcul();
        listeTousObjets.metAJourListe();
        int size = listeTousObjets.pointeurs.size();
        if (ind < size) listeTousObjets.setSelectedIndex(ind);
        else listeTousObjets.setSelectedIndex(size-1);
        metAJourInfo();
      }      
    }
    else {
      // Correction dfaut version 4.9.9.3. Avant d'enregistrer il faut retirer les noms des points
    	// liste.effaceNoms(listeObjetsNommes); // Modifi version 5.1
     	listeObjetsNommes.effaceNoms();
      frame.retireDependants(antecedentDirect);
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil16"));
      // Correction dfaut version 4.9.9.3. Puis les renommer
      listeObjetsNommes.retireTout();
  		liste.genereNoms(listeObjetsNommes, affObjInterm);
  		
      if (ind >= listeTousObjets.pointeurs.size()) listeTousObjets.setSelectedIndex(ind-1);
      else listeTousObjets.setSelectedIndex(ind);
      cadre.effacegbuffer();
      cadre.repaintFigureSansCalcul();
      listeTousObjets.metAJourListe();
      int size = listeTousObjets.pointeurs.size();
      if (ind < size) listeTousObjets.setSelectedIndex(ind);
      else listeTousObjets.setSelectedIndex(size-1);
      metAJourInfo();
    }   
  }
  private final void btnSupprimerJusqueFinActionPerformed() {
   	if (indiceObjetEnCours() == 0) {
      dialogueAvertissement(getString("ch24"));
      return;
    }
		boolean affObjInterm = checkAffObjInterm.isSelected();
  	int ind = listeTousObjets.getSelectedIndex();
    MtgInternalFrame cadre = frame.cadreActif();    
    CListeObjets liste = cadre.listePr;
    CElementBase elb = objetEnCours();
    // Destruction de l'objet en cours
    CElementBase antecedentDirect = elb.antecedentDirect();
    if (antecedentDirect.getNature() == NatureObjet.NBipoint)  {
      if (frame.dialogueConfirmation_YES_NO(getString("ch38"))) {
        // Correction dfaut version 4.9.9.3. Avant d'enregistrer il faut retirer les noms des points
      	// liste.effaceNoms(listeObjetsNommes); // Modifi version 5.1
       	listeObjetsNommes.effaceNoms();
		    frame.retireNDerniersElements(liste.size() - indiceObjetEnCours());
		    // Si l'antcdent direct n'tait pas le mme que l'lment, on regarde :
		    // Si c'est une transformation, on regarde s'il reset d'autres objets dpendant de cette transformation.
		    // S'il n'y en a pas, on dtruit la transformation, sinon on la garde
		    // Dans les autres cas (destruction d'une intersection d'objets) on dtruit l'antcdent direct et ses dpendants
		    if (antecedentDirect != elb) {
		    	if (antecedentDirect.getNature() == NatureObjet.NTransformation) {
		    		if (liste.nombreDependants(antecedentDirect) == 1) {
		    			frame.retireDependants(antecedentDirect);
		    		}
		    	}
		  		else frame.retireDependants(antecedentDirect);
		    }
		    frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil16"));
	      // Correction dfaut version 4.9.9.3. Puis les renommer
        listeObjetsNommes.retireTout();
    		liste.genereNoms(listeObjetsNommes, affObjInterm);
    		
		    listeTousObjets.metAJourListe();
		    listeTousObjets.setSelectedIndex(ind-1);
		    cadre.effacegbuffer();
        cadre.repaintFigureSansCalcul();
	      metAJourInfo();		    
		  }
    }
    else {
      // Correction dfaut version 4.9.9.3. Avant d'enregistrer il faut retirer les noms des points
    	// liste.effaceNoms(listeObjetsNommes); // Modifi version 5.1
     	listeObjetsNommes.effaceNoms();
	    frame.retireNDerniersElements(liste.size() - indiceObjetEnCours());
	    // Si l'antcdent direct n'tait pas le mme que l'lemnt, on regarde :
	    // Si c'est une transformation, on regarde s'il reset d'autres objets dpendant de cette transformation.
	    // S'il n'y en a pas, on dtruit la transformation, sinon on la garde
	    // Dans les autres cas (destruction d'une intersection d'objets) on dtruit l'antcdent direct et ses dpendants
	    if (antecedentDirect != elb) {
	    	if (antecedentDirect.getNature() == NatureObjet.NTransformation) {
	    		if (liste.nombreDependants(antecedentDirect) == 1) {
	    			frame.retireDependants(antecedentDirect);
	    		}
	    	}
	  		else frame.retireDependants(antecedentDirect);
	    }
	    frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil16"));
      // Correction dfaut version 4.9.9.3. Puis les renommer
      listeObjetsNommes.retireTout();
  		liste.genereNoms(listeObjetsNommes, affObjInterm);
  		
	    listeTousObjets.setSelectedIndex(ind-1);
	    listeTousObjets.metAJourListe();
      cadre.effacegbuffer();
      cadre.repaintFigureSansCalcul();
      metAJourInfo();
	  }
  	
  }
  private final void btnModifierObjetActionPerformed() {
    CElementBase elb = objetEnCours();
    MtgDlg dlg;
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets liste = cadre.listePr;
    if (elb.modifiableParMenu()) {
    	if (elb.getNature() == NatureObjet.NBarycentre) {
    		dlg = new ModificationBarycentreDlg(frame, (CBarycentre) elb);
      	dlg.setLocationRelativeTo(this);
      	dlg.setVisible(true);
      	if (dlg.retourOK()) {
      		// Ajout version 5.1.3
         	listeObjetsNommes.effaceNoms();
          listeObjetsNommes.retireTout();
      		cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil66"));
      		liste.genereNoms(listeObjetsNommes, checkAffObjInterm.isSelected());
      		// Fin ajout version 5.1.3
      		liste.metAJourObjetsDependantDe(elb);
      		cadre.redessineAvecCalcul();
      		int ind = listeTousObjets.getSelectedIndex();
      		listeTousObjets.metAJourListe();
      		listeTousObjets.setSelectedIndex(ind);
      		listeTousObjets.ensureIndexIsVisible(ind);
      		metAJourInfo();
      	}
    	}
    	else {
	    	if (elb.estDeNature(NatureObjet.NToutObjet)) {
	    		if (elb.antecedentDirect().getNature() == NatureObjet.NTransformation) {
	    			// Cas o l'objet cliqu est l'image d'une autre objet par une transformation
	    			CTransformation trans = (CTransformation)elb.antecedentDirect();
						if (trans.modificationImage(frame, (CElementGraphique) elb, false)) {
		      		// Ajout version 5.1.3
		         	listeObjetsNommes.effaceNoms();
		          listeObjetsNommes.retireTout();
		      		cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil166"));
		      		liste.genereNoms(listeObjetsNommes, checkAffObjInterm.isSelected());
		      		// Fin ajout version 5.1.3
		      		int ind = listeTousObjets.getSelectedIndex();
		      		listeTousObjets.metAJourListe();
		      		listeTousObjets.setSelectedIndex(ind);
		      		listeTousObjets.ensureIndexIsVisible(ind);
	    			}
	    		}
	    		else {
		  			dlg = elb.dialogueModification(frame);
		  			dlg.setLocationRelativeTo(frame);
		  			dlg.setVisible(true);
		  			if (dlg.retourOK()) {
		      		// Ajout version 5.1.3
		         	listeObjetsNommes.effaceNoms();
		          listeObjetsNommes.retireTout();
		      		cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil166"));
		      		liste.genereNoms(listeObjetsNommes, checkAffObjInterm.isSelected());
		      		// Fin ajout version 5.1.3
		    		  cadre.listePr.metAJourObjetsDependantDe(elb);
		      		cadre.redessineAvecCalcul();
		      		int ind = listeTousObjets.getSelectedIndex();
		      		listeTousObjets.metAJourListe();
		      		listeTousObjets.setSelectedIndex(ind);
		      		listeTousObjets.ensureIndexIsVisible(ind);
			    		metAJourInfo();
			  		}
	    		}
	    	}
	    	else {
	    		CValeurDynamique val = (CValeurDynamique) elb;
	      	dlg = val.dialogueModification(frame);
	      	dlg.setLocationRelativeTo(this);
	      	dlg.setVisible(true);
	      	if (dlg.retourOK()) {
	      		// Ajout version 5.1.3
	         	listeObjetsNommes.effaceNoms();
	          listeObjetsNommes.retireTout();
	      		cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil48"));
	      		liste.genereNoms(listeObjetsNommes, checkAffObjInterm.isSelected());
	      		// Fin ajout version 5.1.3
	    		  cadre.listePr.metAJourObjetsDependantDe(val);      		
	      		cadre.redessineAvecCalcul();
	      		int ind = listeTousObjets.getSelectedIndex();
	      		listeTousObjets.metAJourListe();
	      		listeTousObjets.setSelectedIndex(ind);
	      		listeTousObjets.ensureIndexIsVisible(ind);
	      		metAJourInfo();
	      	}
	    	}
    	}
  	}
  }
  
  private final void btnGommeActionPerformed() {
    CElementBase elb = objetEnCours();
    ((CElementGraphique) elb).masque = true;
    metAJourInfo();
  }
  private final void btnRideauActionPerformed() {
    CElementBase elb = objetEnCours();
    ((CElementGraphique) elb).masque = false;
    metAJourInfo();
  }
  private final void fermeture() {
  	// cadre.listePr.effaceNoms(listeObjetsNommes); // Modifi  version 5.1
   	listeObjetsNommes.effaceNoms();
  	listeObjetsNommes.retireTout();
  	frame.montreBarreOutil(true); // On cache la barre d'outils de gauche pour avoir de la place
  	frame.cadreActif().listePr.retablitComposants();
    frame.outilProtocole.deselect();
    cadre.effacegbuffer();
    frame.cadreActif().repaintFigureSansCalcul();
    frame.activeOutilCapture();
    if (reclassementsEffectues) cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil98"));
  	dispose();
  }
}