/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import javax.swing.BorderFactory;
import objets.CCalculBase;
import objets.CListeObjets;
import types.ErreurCalculException;
import types.Int;

import mathgraph32.MtgFrame;

// Modifi version 4.6.4
@SuppressWarnings("serial")
public class panePreferencesPressePapier extends PaneOptionsPreferences {
	private MtgLabel label1, label2;
	// Ajout version 4.6.4
	public MtgCheckBox cbAttenuation;
	private MtgTextField editeurCoeffMultImages;
	private double ratio;
	
	public panePreferencesPressePapier(MtgFrame frameInit) {
		super(frameInit);
		initComponents();
	}

	private void initComponents() {
		label1 = new MtgLabel(frame, MtgFrame.getString("coeffMultPressePapier1"));
		label2 = new MtgLabel(frame, MtgFrame.getString("coeffMultPressePapier2"));
		cbAttenuation = new MtgCheckBox(frame, MtgFrame.getString("coeffMultPressePapier3"), frame.pref_attenuation);
		editeurCoeffMultImages = new MtgTextField(frame, 6);
		setLayout(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    MtgDlg.setConstraints(constraints, 0, 0, 2, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    add(label1, constraints);
    MtgDlg.setConstraints(constraints, 0, 1, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    add(label2, constraints);
    MtgDlg.setConstraints(constraints, 1, 1, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    add(editeurCoeffMultImages, constraints);
    MtgDlg.setConstraints(constraints, 0, 2, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    add(cbAttenuation, constraints);

		editeurCoeffMultImages.setText(Double.toString(frame.pref_ClipboardImagesRatio));
		setBorder(BorderFactory.createEmptyBorder(10, 20, 10, 20));		
	}

	@Override
	public void sauveDonnees() {
		frame.pref_ClipboardImagesRatio = ratio;
		// Ajout version 4.6.4
		frame.pref_attenuation = cbAttenuation.isSelected();
	}

	@Override
	public boolean donneesValides() {
    CListeObjets liste = frame.cadreActif().listePr;
    String rep = editeurCoeffMultImages.getText();
    Int inderr = new Int();    
  	if (CCalculBase.verifieSyntaxe(liste, rep, inderr, 0, null)) {
      CCalculBase calc = CCalculBase.creeCalculBase(rep ,liste, 0, rep.length()-1, null);
      try {
        calc.listeProprietaire.initialiseNombreIterations();
        ratio = calc.resultat(false);
      }
      catch (ErreurCalculException e) {
      	if (e.depassementCapacite)
      		frame.dialogueAvertissement(MtgFrame.getString("ch179"));
      	else frame.dialogueAvertissement(MtgFrame.getString("ch44"));
      	editeurCoeffMultImages.requestFocus();
        return false;
      }
      // Modifi version 4.6.4. Seuelement agrandissements autoriss.
      // if ((ratio >= 0.1) && (ratio <= 4))  return true;
      // Modifi version 4.8.1 car aussi utilis pour exportation Tikz
      // if ((ratio >= 1) && (ratio <= 4))  return true;
      if ((ratio >= 0.25) && (ratio <= 4))  return true;
      else {
      	frame.dialogueAvertissement(MtgFrame.getString("ch194"));
      	editeurCoeffMultImages.requestFocus();
      	return false;
      }
  	}
  	else {
  		frame.dialogueAvertissement(MtgFrame.getString("ch5"));
  		editeurCoeffMultImages.requestFocus();  		
  		return false;
  	}
	}

	@Override
	public String titreOnglet() {
		// TODO Auto-generated method stub
		return MtgFrame.getString("ongletPressePapier");
	}
}
