/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package entreesSorties;

import java.awt.image.BufferedImage;
import java.io.*;
import java.util.*;

import javax.imageio.ImageIO;

import mathgraph32.Global;

import objets.CObjetBase;

public class MathGraphDataOutputStream extends DataOutputStream {
	public static final byte nat_png = 0;
	public static final byte nat_gif = 1;
	public static final byte nat_jpg = 2;
	public static final byte nat_noImage = -1;
	
	public static final String[] suffix = {Global.GLOBAL_PNG,Global.GLOBAL_GIF,Global.GLOBAL_JPG};
	
  ArrayList<String> listeNomsClasse = new ArrayList<String>();
  /** Creates new MathGraphDataOutputStream */
    public MathGraphDataOutputStream(OutputStream out) {
      super(out);
  }
  /* Nouvelle version de serialization en java. Ecriture */
  public final void writeObject(CObjetBase ob)
    throws IOException {
    int i;
    String nomClasse = ob.getClass().getName();
    int numVer = ob.numeroVersion(); // Numro de version
    // On regarde d'abord si un objet de cette classe a dj t enregistr dans le flux.
    // Si oui on enregistre dans le flux son indice dans la liste des classes dj
    // charges dans la liste listeNomsClasses.
    // Sinon on enregistre un short FFFF suivi du numro de version de l'objet
    // suivi du nom de la classe
    boolean trouve = false;
    for (i = 0; (i < listeNomsClasse.size()) && !trouve; i++) {
      String ch = (String) listeNomsClasse.get(i);
      if (ch.equals(nomClasse)) {
        trouve = true;
        break;
      }
    }
    if (trouve) {
      writeInt(i);
    }
    else {
      writeInt(-1);
      writeInt(numVer);
      writeUTF(nomClasse);
      listeNomsClasse.add(nomClasse);      
    }
    ob.write(this, ob.listeProprietaire);
  }
  
  /**
   * Appel pour enregistrer les caractres successifs d'une chane dans le flux
   * Ne sera appel que pour enregistrer la chane d'identification
   * De cette faon le document pourra tre lu par l'applet de la mme faon
   * que les figures issues du C++
   * @param chaine
   * @return
   * @throws IOException
   */
  public final void writeCString(String chaine) throws IOException {
    for (int i = 0; i < chaine.length(); i++) {
      this.writeByte(chaine.charAt(i));
    }
  }
  
  public final void writeBufferedImage(BufferedImage image, int natImage) throws IOException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream(2000);
    ImageIO.write(image, suffix[natImage], baos);
    baos.flush();
    byte[] b = baos.toByteArray();
    // On enregistre dans le flux la taille du tableau de byte reprsentant l'image puis le tableau lui-mme
    writeInt(b.length);
    write(b);  	
  }
  
  public static final byte getNatImage(String path) {
		int indpoint = path.lastIndexOf(".");
		String ext;
		// On informe l'image de sa nature pour qu'elle la respecte en s'enregistrant dans les fluxs de donnes
		if (indpoint != -1) ext = path.substring(path.lastIndexOf(".")+1);
		else ext = Global.GLOBAL_COMMA_PNG;
		if (ext.equalsIgnoreCase(Global.GLOBAL_GIF)) return MathGraphDataOutputStream.nat_gif;
		else if (ext.equalsIgnoreCase(Global.GLOBAL_JPEG) || ext.equalsIgnoreCase(Global.GLOBAL_JPG)) return MathGraphDataOutputStream.nat_jpg;
		else return MathGraphDataOutputStream.nat_png;
  	
  }
}
