/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package mathgraph32;

import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;

@SuppressWarnings("serial")
public class MtgIconArrow extends JButton implements ActionListener {
	private MtgIconButton iconButton;
	public MtgIconArrow(MtgIconButton iconButtonInit) {
		super();
		iconButton = iconButtonInit;
		setIcon(getImageIcon());
		/*
    setBorder(BorderFactory.createCompoundBorder(
    		BorderFactory.createEtchedBorder(EtchedBorder.RAISED),BorderFactory.createCompoundBorder(
    				BorderFactory.createMatteBorder(1,1,1,1,Color.white), BorderFactory.createLineBorder(Color.white,1))));
    				*/
		// La bordure doit avoir les mmes dimensions verticales que celle de MtgIconButton
    setBorder(BorderFactory.createCompoundBorder(
				BorderFactory.createMatteBorder(1,0,1,0,Color.white), BorderFactory.createLineBorder(Color.white,1)));
		addActionListener(this);
	}
	
	@Override
	public void paintComponent(Graphics g) {
		// TODO Auto-generated method stub
		super.paintComponent(g);
		Graphics2D pDC= (Graphics2D) g;
		int w = getWidth();
		int h = getHeight()/2;
		int a = (int)(0.25*w);
		int b = (int)(0.25*h);
		int[] abs = {w-a, 2, 2, w-a};
		int[] ord = {h, b, 2*h-b, h};
		Polygon pol = new Polygon(abs, ord, 4);
		// Ligne suivante modifi version 5.3 pour changer la couleur des flches.
		// GradientPaint gp = new GradientPaint(0, 0, Color.cyan, 16, 0, Color.blue, true);
    GradientPaint gp = new GradientPaint(0, 0, Color.lightGray, 30, 0, Color.yellow, true);
    pDC.setPaint(gp);
		pDC.fillPolygon(pol);
		pDC.setColor(Color.black);
		pDC.drawPolygon(pol);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		iconButton.frame.fermeMenusHorizontauxSauf(iconButton);
		if ((iconButton.menu == null) || !iconButton.menu.isVisible()) {
			iconButton.menu = new MtgHorizontalMenu(iconButton.frame, iconButton.tabActions, iconButton);
			iconButton.menu.setLocation(getLocationOnScreen().x+getWidth(), getLocationOnScreen().y);
			iconButton.menu.setVisible(true);
		}
		else {
			if ((iconButton.menu != null) && iconButton.menu.isVisible()) {
				iconButton.menu.setVisible(false);
				iconButton.menu.removeAll();
				iconButton.menu = null;		 
			}
			else {
				iconButton.menu = new MtgHorizontalMenu(iconButton.frame, iconButton.tabActions, iconButton);
				iconButton.menu.setLocation(getLocationOnScreen().x+getWidth(), getLocationOnScreen().y);
				iconButton.menu.setVisible(true);
			}
		}
	}

  private final ImageIcon getImageIcon() {
  	return new MtgImageIcon(iconButton.frame.coefMultVertIcons(), MtgFrame.class.getResource("images/IconArrow.png"));
  }

}