/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

// L'objet barycentre maintient une liste de noeuds pondrs.


import java.awt.Color;
import java.io.IOException;
import java.util.ArrayList;
import java.util.ListIterator;

import types.DimensionFenetre;
import types.NatureObjet;
import types.StyleEncadrement;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;


public class CBarycentre extends CPointAncetre {
  public java.util.ArrayList<CNoeudPondere> col;
  /** Creates new CBarycentre */
  public CBarycentre() {
    col = new ArrayList<CNoeudPondere>();
  }
  
  // Constructeur pour clonage
  public CBarycentre(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit , java.util.ArrayList<CNoeudPondere> colInit, boolean marquePourTraceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
      dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
    col = colInit;
  }
    
  // Il faut cloner chaque lment de la liste de noeuds pondrs.
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    java.util.ArrayList<CNoeudPondere> colClone = new java.util.ArrayList<CNoeudPondere>();
    ListIterator<CNoeudPondere> li = col.listIterator();
    while(li.hasNext()) {
      CNoeudPondere noeud = li.next();
      CNoeudPondere noeudClone = noeud.getClone(listeSource, listeCible);
      colClone.add(noeudClone);
    }
   int ind1 = listeSource.indexOf(impProto);        
   CElementBase ptelb = new CBarycentre(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
     estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
          indiceTailleNom, motif, colClone, marquePourTrace);
    return ptelb;
  }
    
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
		// Pour chacun des points pondrs, on cr un commentaire associ qui affichera le coefficient
		// Tous ces commentaires seront rajouts  la liste des objets clignotants
		int nbPoints = col.size();
		for (int i = 0; i < nbPoints; i++) {
			CNoeudPondere noeud  = col.get(i);
			CPointAncetre point = noeud.pointAssocie;
			liste.add(point);
			/* Modifi version 4.9.9. On met le commentaire en haut du point pour qu'on voie son nom dans le protocole
    	CCommentaire com = new CCommentaire(listeProprietaire, null, false, Color.blue, 0d, 0d, 0d, 10d, false, point, 2,
    			StyleEncadrement.Simple, true,  new Color(230,230,246), noeud.poids.chaineInfo(),
    			CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalHaut);
    	*/
			/* Modifi version 5.0
    	CCommentaire com = new CCommentaire(listeProprietaire, null, false, Color.blue, 0d, 0d, 0d, -25d, false, point, 2,
    			StyleEncadrement.Simple, true,  new Color(230,230,246), noeud.poids.chaineInfo(),
    			CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalHaut);
    	*/
    	CCommentaire com = new CCommentaire(listeProprietaire, null, false, Color.blue, 0d, 0d, 0d, -25d, false, point, 13,
    			StyleEncadrement.Simple, true,  new Color(230,230,246), noeud.poids.chaineInfo(),
    			CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalHaut);
    	com.positionne(false, frame.cadreActif().dimf);
    	liste.add(com);
		}
  }  
  
  @Override
  public String infoPourHistorique() {
  	String ch = nom() + " : "+ getString("chinfo75") + " " + getString("ch31") + " : \n";
    ListIterator<CNoeudPondere> li = col.listIterator();
    while(li.hasNext()) {
      CNoeudPondere noeud = li.next();
      ch += "(" + noeud.pointAssocie.nom() +";" + noeud.poids.chaineInfo() + ") ";
    }
    return ch;
  }
	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	String ch = nom() + " : "+ getString("chinfo75") + " " + getString("ch31") + " ";
    ListIterator<CNoeudPondere> li = col.listIterator();
    while(li.hasNext()) {
      CNoeudPondere noeud = li.next();
      ch += "(" + noeud.pointAssocie.nom() +";" + noeud.poids.chaineInfo() + ") ";
    }
    return ch;
	}  
  
  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
    ListIterator<CNoeudPondere> li = col.listIterator(); 
    while(li.hasNext()) {
      CNoeudPondere noeud = li.next();
      noeud.initialisePourDependance();
    }
  }
  // Optimis version 4.7.6
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    boolean resultat = false;
    ListIterator<CNoeudPondere> li = col.listIterator();
    while(li.hasNext() && !resultat) {
      CNoeudPondere noeud = li.next();
      resultat = resultat || noeud.pointAssocie.dependDe(p) || noeud.poids.dependDe(p);
    }
    return memoriseDependance(resultat || super.dependDe(p));
  }

  // Optimis version 4.7.6
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    boolean resultat = false;
    ListIterator<CNoeudPondere> li = col.listIterator();
    while(li.hasNext() && !resultat) {
      CNoeudPondere noeud = li.next();
      resultat = resultat || noeud.pointAssocie.dependDePourReclassement(p) || noeud.poids.dependDePourReclassement(p);
    }
    return memoriseDependancePourRec(resultat || super.dependDePourReclassement(p));
  }
  
  // Optimis version 4.7.6
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    boolean resultat = false;
    ListIterator<CNoeudPondere> li = col.listIterator();
    while(li.hasNext() && !resultat) {
      CNoeudPondere noeud = li.next();
      resultat = resultat || noeud.pointAssocie.dependDePourBoucle(p) || noeud.poids.dependDePourBoucle(p);
    }
    return ((p == this) || resultat);
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double mt;// masse totale
    double xg,yg;

		existe = true;
		mt = 0;
		xg = 0;
		yg = 0;
    ListIterator<CNoeudPondere> li = col.listIterator();
    while(li.hasNext()) {
      CNoeudPondere noeud = li.next();
      //Ligne suivante pour version 3.0
      noeud.poids.positionne(infoRandom, dimfen);
			existe = existe && (noeud.pointAssocie.existe) && (noeud.poids.existe());
			if (!existe) return;
			double p = noeud.poids.rendValeur();
			mt = mt + p;
			xg = xg + p*noeud.pointAssocie.x;
			yg = yg + p*noeud.pointAssocie.y;
		}
		if (mt == 0) {
			existe = false;
			return;
		}
		else
		{
			xg = xg / mt;
			yg = yg / mt;
			super.placeEn(xg,yg);
			super.positionne(infoRandom, dimfen); // Ajout verion 4.6.4
		}
	}
 
  @Override
	public boolean confonduAvec(CElementBase p) {
    CBarycentre pt;
    if (p.getClass() == this.getClass()) {
      pt = (CBarycentre) p;
      if (col.size() != pt.col.size()) return false;
      else {
  			int nbElementsCommuns = 0;
  			// On cre une copie de col
  	    java.util.ArrayList<CNoeudPondere> colClone = new java.util.ArrayList<CNoeudPondere>();
  	    ListIterator<CNoeudPondere> li = col.listIterator();
  	    while(li.hasNext()) {
  	      CNoeudPondere noeud = li.next();
  	      CNoeudPondere noeudClone = noeud.getClone(listeProprietaire, listeProprietaire);
  	      colClone.add(noeudClone);
  	    }
  			for (int i = 0; i < pt.col.size(); i++) {
  				CNoeudPondere no1 = (CNoeudPondere)(pt.col.get(i));
  				for (int j = 0; j < colClone.size(); j++) {
  					CNoeudPondere no2 = (CNoeudPondere)(colClone.get(j));
  					if ((no2.pointAssocie == no1.pointAssocie) && (no2.poids.confonduAvec(no1.poids))) {
  						nbElementsCommuns++;
  						colClone.remove(j);
  					}
  				}
  			}
  			return nbElementsCommuns == col.size();
      }
    }
    else return false;
	}
  
  public double sommeCoefficients(boolean infoRandom)
  {
    double resultat = 0;
    ListIterator<CNoeudPondere> li = col.listIterator();
    while(li.hasNext()) {
      CNoeudPondere noeud = li.next();
      resultat = resultat + noeud.poids.rendValeur();
    }
    return resultat;
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NBarycentre;
  }
  
  // Optimis version 4.7.6
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	boolean resultat = true;
    ListIterator<CNoeudPondere> li = col.listIterator();
    while(li.hasNext() && resultat) {
      CNoeudPondere noeud = li.next();
  		resultat = resultat 
  		&&
  		noeud.pointAssocie.estDefiniParObjetDans(listeOb)
  		&&
  		noeud.poids.estDefiniParObjetDans(listeOb);
  	}
  	return resultat;
	}  
  
  /**
   * Fonction qui renverra true si la classe de l'objet possde un menu pour le modifier
   * Danq le cas particulier d'un barycentre, il y aura d'abord activation de l'outil de modification
   * d'un barycentre
   * @return
   */
  @Override
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    // On lit d'abord le nombre d'objets de la liste
    int nombreObjets = inps.readCCount();
    for (int i = 0; i < nombreObjets; i++) {
      CNoeudPondere noeud = (CNoeudPondere) inps.readCObject(list);;
      col.add(noeud);
    }
  }
  /* Enregistrement dans un flux java */  
  // modifi par rapport  chrageDepuisFlux n'emploit pas readObject
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    // On lit d'abord le nombre d'objets de la liste
    int nombreObjets = inps.readInt();
    for (int i = 0; i < nombreObjets; i++) {
      CNoeudPondere noeud = new CNoeudPondere();
      noeud.read(inps, list);
      col.add(noeud);
    }
  }  
  /* Enregistrement dans un flux java */
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int nombreObjets = col.size();
    oups.writeInt(nombreObjets);
    ListIterator<CNoeudPondere> li = col.listIterator();
    while(li.hasNext()) {
      CNoeudPondere noeud = li.next();
      noeud.write(oups, list);
    }
  }   

  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    ListIterator<CNoeudPondere> li = col.listIterator();
    while(li.hasNext()) {
      CNoeudPondere noeud = li.next();
      noeud.remplacePoint(ancienPoint, nouveauPoint);
    }
  }
}
