/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.Color;
import java.io.IOException;

import types.DimensionFenetre;
import types.StyleTrait;
import dialogs.CoefficientDirecteurDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;


public class CDroiteOm extends CDroite {
	public CRepere rep;
	public CPointAncetre O;
	public CValeurAssocieeAVariable m;

  /** Creates new CDroiteOm */
  public CDroiteOm() {
  }
  
  // Constructeur pour clonage
  public CDroiteOm(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit, StyleTrait styleInit,
      double abscisseNomInit, CRepere repInit, CPointAncetre OInit, CValeurAssocieeAVariable mInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit,
      masqueInit, nomInit, indiceTailleNomInit, styleInit, abscisseNomInit);
    rep = repInit;
    O = OInit;
    m = mInit;     
  }
  
  // Constructeur plus court pour cration d'objet
  public CDroiteOm(CListeObjets liste, Color couleurInit, StyleTrait styleInit, CRepere repInit,
  		CPointAncetre OInit, CValeurAssocieeAVariable mInit, int indiceTailleNomInit) {
  	super(liste, null, false, couleurInit, false, styleInit, indiceTailleNomInit);
    rep = repInit;
    O = OInit;
    m = mInit;       	
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(rep);
    int ind2 = listeSource.indexOf(O);
    int ind3 = listeSource.indexOf(impProto);     
    CValeurAssocieeAVariable mClone = (CValeurAssocieeAVariable) m.getClone(listeSource, listeCible);
    CElementBase ptelb = new CDroiteOm(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y,
      masque, nom, indiceTailleNom, style, abscisseNom, (CRepere)(listeCible.get(ind1, CRepere.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)), mClone);
    return ptelb;
  }
  
  @Override
  // Amlior version 3.8
  public String infoPourHistorique() {
  	return nom() + " : " + getString("chinfo152") + " " + m.chaineInfo() + " " + getString("chinfo196") + " " + O.nom() +
				"\n" + getString("chinfo32")	+ " " + rep.nom();
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo152") + " " + m.chaineInfo() + " " + getString("chinfo196") + " " + O.nom() +
  				" " + getString("chinfo32")	+ " " + rep.nom();
	}

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(O);
  }

  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	m.initialisePourDependance();
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || O.dependDe(p) || rep.dependDe(p) || m.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || O.dependDePourReclassement(p) 
    	|| rep.dependDePourReclassement(p) || m.dependDePourReclassement(p));
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || O.dependDePourBoucle(p) || rep.dependDePourBoucle(p)
      || m.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	m.positionne(infoRandom, dimfen);
		existe = rep.existe && O.existe && m.existe();
		if (existe)
		{
			point_x = O.x;
			point_y = O.y;
			double k = m.rendValeur();
			double unitex = rep.unitex;
			double unitey = rep.unitey;
			vecteur.x = rep.u.x/unitex + k*rep.v.x/unitey;
			vecteur.y = rep.u.y/unitex + k*rep.v.y/unitey;
			super.positionne(infoRandom, dimfen);
    }
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CDroiteOm pt;
    if (p.getClass() == this.getClass()) {
      pt = (CDroiteOm) p;
   		return (rep == pt.rep) && (O == pt.O) && m.confonduAvec(pt.m);
    }
    else return false;
  }        

  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (O == ancienPoint) O = nouveauPoint;
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new CoefficientDirecteurDlg(frame, this, true);
  }
  
  @Override
  public String informationModification() {
  	return getString("chaineOutil44");
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(rep) || rep.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(O) || O.estDefiniParObjetDans(listeOb))
		&&
		m.estDefiniParObjetDans(listeOb);
	}    

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
    O = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    m = new CValeurAssocieeAVariable();
    m.chargeDepuisFlux(inps, list);
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
    O = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    m = new CValeurAssocieeAVariable();
    m.read(inps, list);
  } 
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(rep);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(O);
    oups.writeInt(ind2);
    m.write(oups, list);
  }
  
  @Override
  public boolean contientParDefinition(CPointAncetre po)
  {
    return (O == po);
  }
}