/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;

public class CDroiteParallele extends CDroite {
	public CPointAncetre A;	// Le point par lequel la droite passe
	public CDroiteAncetre D;				// La droite  laquelle elle est parallle

  /** Creates new CDroiteParallele */
  public CDroiteParallele() {
    super(); // Pour que lengthvecteur soit cr
  }
  
  // Constructeur pour clonage
  public CDroiteParallele(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      StyleTrait styleInit, double abscisseNomInit, CPointAncetre AInit, CDroiteAncetre DInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
            dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, styleInit, abscisseNomInit);
    A = AInit;
    D = DInit;
  }
  // Constructeur plus court pour objets visuels
  public CDroiteParallele(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit,
    Color couleurInit, StyleTrait styleInit, CPointAncetre AInit, CDroiteAncetre DInit, int indiceTailleNomInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, false, styleInit, indiceTailleNomInit);
    A = AInit;
    D = DInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(A);
    int ind2 = listeSource.indexOf(D);
    int ind3 = listeSource.indexOf(impProto);     
    CElementBase ptelb = new CDroiteParallele(listeCible,(CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y,
      masque, nom, indiceTailleNom, style, abscisseNom,
      (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CDroiteAncetre)(listeCible.get(ind2, CDroiteAncetre.class)));
    return ptelb;
  }
 
  /**
   * Fonction renvoyant la chane  afficher dans la bote de dialogue d'historique
   * pour dcrire l'objet.
   * @return
   */
  @Override
  public String infoPourHistorique() {
  	return nom() + " : " + getString("chinfo5") + " " + D.nom() + "\n" + getString("chinfo196") + " " + A.nom();
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo5") + " " + D.nom() + " " + getString("chinfo196") + " " + A.nom();
	}

  /**
   * Fonction ajoutant  list les lments qui ont gnr l'objet.
   * @param liste
   */
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(A);
    liste.add(D);
  }  
  
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	return memoriseDependance(super.dependDe(p) || A.dependDe(p) || D.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || A.dependDePourReclassement(p) || D.dependDePourReclassement(p));
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	return ((p == this) || A.dependDePourBoucle(p) || D.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = (D.existe) && (A.existe);
    if (!existe) return;
    point_x = A.x;
    point_y = A.y;
    D.vecteur.setCopy(this.vecteur);
    super.positionne(infoRandom, dimfen);
  }  
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CDroiteParallele pt;
    if (p.getClass() == this.getClass()) {
      pt = (CDroiteParallele) p;
      return ((A==pt.A) && (D==pt.D));
    }
    else return false;
  }  

  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (A == ancienPoint) A = nouveauPoint;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(A) || A.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(D) || D.estDefiniParObjetDans(listeOb));
	}
  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    A = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    D = (CDroiteAncetre) (list.get(ind2,CDroiteAncetre.class));
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    A = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    D = (CDroiteAncetre) (list.get(ind2,CDroiteAncetre.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(A);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(D);
    oups.writeInt(ind2);
  }     

  @Override
  public boolean contientParDefinition(CPointAncetre po)
  {
    return (A == po);
  }
  
  @Override
  public boolean estParalleleParDefinition(CDroiteAncetre dte){
  	return (dte == this) || (dte == D) || D.estParalleleParDefinition(dte) || dte.estParalleleParDefinition(D);
  }
  
  @Override
  public boolean estPerpendiculaireParDefinition(CDroiteAncetre dte) {
  	return D.estPerpendiculaireParDefinition(dte) ||dte.estPerpendiculaireParDefinition(D);
  }
  
}
