	
/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.util.ArrayList;
import java.util.ListIterator;
import java.util.Vector;
import mathgraph32.MtgInternalFrame;
import types.FonteNom;
import types.NatureCalcul;
import types.NatureObjet;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 * 
 * @author Yves
 */
public class CImplementationProto extends CElementBase {
	public String nomProto; // Le nom du prototype qui l'a gnr
	public int nbIntermediaires;
	public int nbFinaux;
	public CSousListeObjets listeSources;

	/** Creates a new instance of CImplementationProto */
	public CImplementationProto() {
		listeSources = new CSousListeObjets();
		existe = true; // Ajout version 4.8
	}

	/**
	 * Constructeur pour clonage
	 */
	public CImplementationProto(CListeObjets liste, String nomProtoInit, int nbIntermediairesInit,
			int nbFinauxInit, CSousListeObjets listeSourcesInit) {
		super(liste, null, false);
		nomProto = nomProtoInit;
		nbIntermediaires = nbIntermediairesInit;
		nbFinaux = nbFinauxInit;
		listeSources = listeSourcesInit;
		existe = true; // Ajout version 4.8
	}

	/**
	 * Constructeur pour implmentation  partir d'un protoype Avanat appel les
	 * lments gnriques du prototype ont t mis  pointer sur les objets
	 * sources dans liste
	 * 
	 * @param liste
	 * @param nomProtoInit
	 * @param nbIntermediairesInit
	 * @param nbFinauxInit
	 */
	public CImplementationProto(CListeObjets liste, CPrototype proto) {
		super(liste, null, false);
		nomProto = proto.nom;
		listeSources = new CSousListeObjets();
		int nbSources = proto.nbObjSources;
		for (int i = 0; i < nbSources; i++) {
			listeSources.add(((CElementGenerique) proto.get(i)).elementAssocie);
		}
		nbFinaux = proto.nbObjFinaux;
		nbIntermediaires = proto.size() - nbSources - nbFinaux;
	}

  @Override
	public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
		CSousListeObjets listeSourcesClone = new CSousListeObjets();
		listeSourcesClone.setImage(listeSources, listeSource, listeCible);
		CElementBase ptelb = new CImplementationProto(listeCible, nomProto, nbIntermediaires, nbFinaux,
				listeSourcesClone);
		return ptelb;
	}

	public final CElementBase dernierObjetFinal() {
		int ind = listeProprietaire.indexOf(this);
		return listeProprietaire.get(ind + nbIntermediaires + nbFinaux);
	}

	@Override
	public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
		return memoriseDependance(super.dependDe(p) || listeSources.dependDe(p));
	}

	// Modifi version 4.8
	@Override
	public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || listeSources.dependDePourReclassement(p));
	}

	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
		return getString("chinfo138") + " " + nomProto;
	}

	@Override
	public long getNature() {
		return NatureObjet.NImplementationProto;
	}

  @Override
	public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
		listeSources.remplacePoint(ancienPoint, nouveauPoint);
	}

	/**
	 * Implmentation dans la liste qui contiendra l'implmentation du protoype,
	 * suivie des objets intermdiares et des objets finaux. Pendant cette
	 * implmentation les listes listeSources, listeIntermediaires et listeFinaux
	 * sont mises  jour. Renvoie une chane vide si l'implmentation est possible
	 * et sinon le message d'erreur  afficher Avant l'appel les lments
	 * gnriques du prototype proto ont vu leur pointeur elementAssocie pointer
	 * sur l'objet source correspondant de la liste propritaire de tous les
	 * objets
	 * Version 4.5 : Ajout d'un troisime paramtre cadre
	 * Modifi version 4.8 : on passe un paramtre ideb qui est le premier indice qu'on rajoute aux noms de calculs ou de points pour gnrer
	 * un nouveau nom
	 */
	public void implemente(CListeObjets listePourImp, CPrototype proto, MtgInternalFrame cadre, int ideb) {
		int i;
		ArrayList<CElementGraphique> listePointsOuDroitesARenommer = new ArrayList<CElementGraphique>();
		ArrayList<CValeurDynamique> listeCalculsARenommer = new ArrayList<CValeurDynamique>();
		CListeObjets listeEltsAjoutes = new CListeObjets();
		// nomProto = proto.nom; // Inutile. Supprim version 4.8
		proto.setListePourImplementation(listeProprietaire);
		// On ajoute l'objet implementationPrototype  la liste
		// Ncessaire d'utiliser un clone car lors des implmentations itratives plusieurs prototypes de mme nature sont utiliss
		CImplementationProto imp = (CImplementationProto) this.getClone(listeProprietaire, listeProprietaire);
		listeProprietaire.add(imp);
		// Comme c'est un clone qui a t ajout et pas this, on donne quand mme  this la valeur de index
		// Pour compatibilit avec les version antrieures.
		this.index = imp.index;																														//
		// Important que que les indices soient corrects
		proto.indicePremierObjetPourImplementation = listeProprietaire.size();
		// On implmente la liste d'objets sources gre par l'implmentation de
		// prototype
		// en recherchant dans la liste des lments sources les points et droites qui ont un nom
		// dans le prototype et on leur gnre un nom pour que les ventuelles mesures
		// et calculs l'utilisant aient leur formules ractualises
		for (i = 0; i < proto.nbObjSources; i++) {
			CElementGenerique ptgen = (CElementGenerique) proto.get(i);
			CElementBase elb = ptgen.elementAssocie;
			// listeSources.add(elb);
			if (elb.estDeNature(NatureObjet.NToutPoint|NatureObjet.NDroite)) {
				CElementGraphique ptp = (CElementGraphique) elb;
				String nomGen = ptgen.nom;
				// On regarde si l'lment correspondant tait nomm.
				// Si oui on rajout cet lment  la liste des points devant tre
				// nomms
				if ((nomGen.length() != 0) && (ptp.rendNom().length() == 0)) {
					ptp.donneNom(nomGen);
					listePointsOuDroitesARenommer.add(ptp);
				}
			}
		}
		// Puis on ajoute des clones des objets intermdiaires et finaux du
		// prototype
		for (i = proto.nbObjSources; i < proto.size(); i++) {
			CElementBase elb = proto.get(i);
			CElementBase clone = elb.getClone(proto, listeProprietaire);
			clone.estElementFinal = elb.estElementFinal;
			clone.impProto = imp; // Important
			listePourImp.add(clone);
			listeEltsAjoutes.add(clone);
			// On regarde s'il y a des points ou des droites dans les lments
			// intermdiaires
			// ou finaux ayant le mme nom que d'autres dj dfinis et si oui on les
			// renomme.
			// Modifi version 4.8
			if (clone.estDeNature(NatureObjet.NToutPoint)) {
				CPointAncetre ptp = (CPointAncetre) clone;
				if (!ptp.rendNom().isEmpty()) { // Ajout version 4.8
					if (listeProprietaire.existeMemeNomNonIntermediaire(ptp, ptp.rendNom()))
						listePointsOuDroitesARenommer.add(ptp);
				} // Fin ajout version 4.8
				// Ajout version 4.9.5.2 : On donne la taille de nom par dfauts aux points finaux
				if (ptp.estElementFinal) ptp.indiceTailleNom = cadre.frame.pref_indiceTaillePoliceNom;
			} else {
				if (clone.getNature() == NatureObjet.NDroite) {
					CDroite ptd = (CDroite) clone;
					if (!ptd.rendNom().isEmpty()) { // Ajout version 4.8
						if (listeProprietaire.existeMemeNomNonIntermediaire(ptd, ptd.rendNom()))
								listePointsOuDroitesARenommer.add(ptd); // Ajout version 4.8
							// ptd.donneNom(listeProprietaire.genereNomPourPointOuDroite('D')); Modifi version 4.8
					}
				} else {
					if (clone.estDeNatureCalcul(NatureCalcul.NToutCalculNomme)) {
						CValeurDynamique vald = (CValeurDynamique) clone;
						if (listeProprietaire.existeCalculOuVariableOuFonctionOuParametreMemeNom(null, vald.nom(), false))
							listeCalculsARenommer.add(vald);
					}
				}
			}
		}
		// On renomme maintenant les points ou droites qui ont besoin d'tre renomms
 		ListIterator<CElementGraphique> li1 = listePointsOuDroitesARenommer.listIterator();
		while (li1.hasNext()) {
			CElementGraphique ptp = li1.next();
			String ancienNom = ptp.rendNom();
			String nouveauNom = listeProprietaire.genereNomPourPointOuDroite(ancienNom.charAt(0), ptp, ideb);
			ptp.donneNom(nouveauNom);
			// listeEltsAjoutes.MetAJourFormulesCalculsDependantDe(ptp); Ne pas appeler  ce niveau
			// car des fomules peuvent dpendre des objets sources
		}
		// On renomme les calculs intermdiaires qui ont besoin d'tre renomms
		// en mettant  jour les affichages de commentaires dynamiques qui en
		// dpendent
		ListIterator<CValeurDynamique> li2 = listeCalculsARenommer.listIterator();
		while (li2.hasNext()) {
			CValeurDynamique val = li2.next();
			String ancienNom = val.nom();
			// Amlioration par rapport au C++. On gnre un nouveau nom avec
			// seulement les premiers caractres qui ne sont pas des chiffres
			String nouveauNom = listeProprietaire.genereNomPourCalcul(FonteNom
					.premiersCaracteresNonChiffres(ancienNom), val, ideb);
			listeEltsAjoutes.remplaceNomValeurDynamiqueDansCommentairesOuLatex(ancienNom, nouveauNom);
			val.donneNom(nouveauNom);
		}
		// On reconstitue les formules des calculs ou fonctions intermdiaires et
		// finaux dpendant des objets sources
		ListIterator<CElementBase> li3 = listeEltsAjoutes.listIterator();
		while (li3.hasNext()) {
			CElementBase elb = li3.next();
			if (elb.estDeNatureCalcul(NatureCalcul.NCalculOuFonctionNonConstante)) {
				CCalcul calc = (CCalcul) elb;
				calc.reconstruitChaineCalcul();
			}
		}
		for (i = 0; i < proto.nbObjSources; i++) {
			CElementBase elb = ((CElementGenerique) proto.get(i)).elementAssocie;
			if (elb.estDeNatureCalcul(NatureCalcul.NTouteValeurPourCommentaireDynamique)) {
				CElementGenerique gen = (CElementGenerique) proto.get(i);
				CValeurDynamique val = (CValeurDynamique) elb;
				listeEltsAjoutes.remplaceNomValeurDynamiqueDansCommentairesOuLatex(gen.nom, val.nom());
			}
		}
		// On met  jour les commentaires des lments rajouts au cas o ils
		// contiendraient
		// des affichages de valeurs dynamiques
		listeEltsAjoutes.determineDependancesCommentaires();
		listeEltsAjoutes.metAJourEtPositionne(true, cadre.dimf); // Modifi version 4.8. True car on permet les calculs alatoires dans les objets
	}

	/* A garder. J'abandonne le renommage desobjest crs ou des calculs crs
	public void implementePourRecur(CListeObjets listePourImp, CPrototype proto, MtgInternalFrame cadre, int ideb, int nbFinaux) {
		int i;
		ArrayList<CElementGraphique> listePointsOuDroitesARenommer = new ArrayList<CElementGraphique>();
		ArrayList<CValeurDynamique> listeCalculsARenommer = new ArrayList<CValeurDynamique>();
		CListeObjets listeEltsAjoutes = new CListeObjets();
		// nomProto = proto.nom; Supprim version 4.8. Inutile
		proto.setListePourImplementation(listeProprietaire);
		// On ajoute l'objet implementationPrototype  la liste
		// Pour ne pas ajouter  la liste de multiples implmentations de mme CImplementationProt, on utilise un clone
		CImplementationProto imp = (CImplementationProto) this.getClone(listeProprietaire, listeProprietaire);
		imp.nomProto = proto.nom;
		listeProprietaire.add(imp); 	// Ncessaire d'utiliser un clone car lors des implmenntations rcursives
																																	// plusieus prototypes de mme nature sont utiliss
		// Important que que les indices soient corrects
		proto.indicePremierObjetPourImplementation = listeProprietaire.size();
		// On implmente la liste d'objets sources gre par l'implmentation de
		// prototype
		// en recherchant dans la liste des lments sources les points et droites qui ont un nom
		// dans le prototype et on leur gnre un nom pour que les ventuelles mesures
		// et calculs l'utilisant aient leur formules ractualises
		for (i = 0; i < proto.nbObjSources; i++) {
			CElementBase elb = ((CElementGenerique) proto.get(i)).elementAssocie;
			// listeSources.add(elb);
			if (elb.estDeNature(NatureObjet.NToutPoint|NatureObjet.NDroite)) {
				CElementGenerique ptgen = (CElementGenerique) proto.get(i);
				CElementGraphique ptp = (CElementGraphique) elb;
				String nomGen = ptgen.nom;
				// On regarde si l'lment correspondant tait nomm.
				// Si oui on rajout cet lment  la liste des points devant tre
				// nomms
				if ((nomGen.length() != 0) && (ptp.rendNom().length() == 0)) {
					ptp.donneNom(nomGen);
					listePointsOuDroitesARenommer.add(ptp);
				}
			}
		}
		// Puis on ajoute des clones des objets intermdiaires et finaux du
		// prototype  concurrence de nbFinaux
		int nbFinauxTotal = 1;
		for (i = proto.nbObjSources; (i < proto.size()) && (nbFinauxTotal <= nbFinaux); i++) {
			CElementBase elb = proto.get(i);
			CElementBase clone = elb.getClone(proto, listeProprietaire);
			boolean b = elb.estElementFinal;
			clone.estElementFinal = b;
			if (b) nbFinauxTotal++;
			clone.impProto = imp; // Important
			listePourImp.add(clone);
			listeEltsAjoutes.add(clone);
			// On regarde s'il y a des points ou des droites dans les lments
			// intermdiaires
			// ou finaux ayant le mme nom que d'autres dj dfinis et si oui on les
			// renomme.
			// Modifi version 4.8
			if (clone.estDeNature(NatureObjet.NToutPoint)) {
				CPointAncetre ptp = (CPointAncetre) clone;
				if (!ptp.rendNom().isEmpty()) { // Ajout version 4.8
					if (listeProprietaire.existeMemeNomNonIntermediaire(ptp, ptp.rendNom()))
						listePointsOuDroitesARenommer.add(ptp);
				} // Fin ajout version 4..8
			} else {
				if (clone.getNature() == NatureObjet.NDroite) {
					CDroite ptd = (CDroite) clone;
					if (!ptd.rendNom().isEmpty()) { // Ajout version 4.8
						if (listeProprietaire.existeMemeNomNonIntermediaire(ptd, ptd.rendNom()))
								listePointsOuDroitesARenommer.add(ptd); // Ajout version 4.8
							// ptd.donneNom(listeProprietaire.genereNomPourPointOuDroite('D')); Modifi version 4.8
					}
				} else {
					if (clone.estDeNatureCalcul(NatureCalcul.NToutCalculNomme)) {
						CValeurDynamique vald = (CValeurDynamique) clone;
						if (listeProprietaire.existeCalculOuVariableOuFonctionOuParametreMemeNom(null, vald.nom(), false))
							listeCalculsARenommer.add(vald);
					}
				}
			}
		}
		// On renomme maintenant les points ou droites qui ont besoin d'tre renomms
 		ListIterator<CElementGraphique> li1 = listePointsOuDroitesARenommer.listIterator();
		while (li1.hasNext()) {
			CElementGraphique ptp = li1.next();
			String ancienNom = ptp.rendNom();
			String nouveauNom = listeProprietaire.genereNomPourPointOuDroite(ancienNom.charAt(0), ptp, ideb);
			ptp.donneNom(nouveauNom);
			// listeEltsAjoutes.MetAJourFormulesCalculsDependantDe(ptp); Ne pas appeler  ce niveau
			// car des fomules peuvent dpendre des objets sources
		}
		// On renomme les calculs intermdiaires qui ont besoin d'tre renomms
		// en mettant  jour les affichages de commentaires dynamiques qui en
		// dpendent
		ListIterator<CValeurDynamique> li2 = listeCalculsARenommer.listIterator();
		while (li2.hasNext()) {
			CValeurDynamique val = li2.next();
			String ancienNom = val.nom();
			// Amlioration par rapport au C++. On gnre un nouveau nom avec
			// seulement les premiers caractres qui ne sont pas des chiffres
			String nouveauNom = listeProprietaire.genereNomPourCalcul(FonteNom
					.premiersCaracteresNonChiffres(ancienNom), val, ideb);
			listeEltsAjoutes.remplaceNomValeurDynamiqueDansCommentairesOuLatex(ancienNom, nouveauNom);
			val.donneNom(nouveauNom);
		}
		// On reconstitue les formules des calculs ou fonctions intermdiaires et
		// finaux dpendant des objets sources
		ListIterator<CElementBase> li3 = listeEltsAjoutes.listIterator();
		while (li3.hasNext()) {
			CElementBase elb = li3.next();
			if (elb.estDeNatureCalcul(NatureCalcul.NCalculOuFonctionNonConstante)) {
				CCalcul calc = (CCalcul) elb;
				calc.reconstruitChaineCalcul();
			}
		}
		for (i = 0; i < proto.nbObjSources; i++) {
			CElementBase elb = ((CElementGenerique) proto.get(i)).elementAssocie;
			if (elb.estDeNatureCalcul(NatureCalcul.NTouteValeurPourCommentaireDynamique)) {
				CElementGenerique gen = (CElementGenerique) proto.get(i);
				CValeurDynamique val = (CValeurDynamique) elb;
				listeEltsAjoutes.remplaceNomValeurDynamiqueDansCommentairesOuLatex(gen.nom, val.nom());
			}
		}
		// On met  jour les commentaires des lments rajouts au cas o ils
		// contiendraient
		// des affichages de valeurs dynamiques
		listeEltsAjoutes.determineDependancesCommentaires();
		listeEltsAjoutes.metAJourEtPositionne(true, cadre.dimf); // Modifi version 4.8. True car on permet les calculs alatoires dans les objets
	}
*/
	/**
	 * Appel pour les impmentations rcursives de constructions
	 * @param listePourImp
	 * @param proto
	 * @param cadre
	 * @param nbFinaux : le nombre d'obejts fianux rel  implmenter
	 */
	public void implementePourRecur(CListeObjets listePourImp, CPrototype proto, MtgInternalFrame cadre, int nbFinaux) {
		int i;
		CListeObjets listeEltsAjoutes = new CListeObjets();
		proto.setListePourImplementation(listeProprietaire);
		// On ajoute l'objet implementationPrototype  la liste
		// Pour ne pas ajouter  la liste de multiples implmentations de mme CImplementationProt, on utilise un clone
		CImplementationProto imp = (CImplementationProto) this.getClone(listeProprietaire, listeProprietaire);
		imp.nomProto = proto.nom;
		// Ajout version 4.9.9 : Pour le reclassement d'objets il faut connatre le nombre rel d'objets finaux
		imp.nbFinaux = nbFinaux;
		//
		listeProprietaire.add(imp); 	// Ncessaire d'utiliser un clone car lors des implmenntations rcursives
																																	// plusieus prototypes de mme nature sont utiliss
		// Important que que les indices soient corrects
		proto.indicePremierObjetPourImplementation = listeProprietaire.size();
		// On implmente la liste d'objets sources gre par l'implmentation de
		// prototype
		// Puis on ajoute des clones des objets intermdiaires et finaux du
		// prototype  concurrence de nbFinaux
		int nbFinauxTotal = 1;
		nbIntermediaires = 0;
		for (i = proto.nbObjSources; (i < proto.size()) && (nbFinauxTotal <= nbFinaux); i++) {
			CElementBase elb = proto.get(i);
			CElementBase clone = elb.getClone(proto, listeProprietaire);
			boolean b = elb.estElementFinal;
			clone.estElementFinal = b;
			if (b) nbFinauxTotal++; else nbIntermediaires++;
			clone.impProto = imp; // Important
			listePourImp.add(clone);
			listeEltsAjoutes.add(clone);
			// On regarde s'il y a des points ou des droites dans les lments
			// intermdiaires
			// ou finaux ayant le mme nom que d'autres dj dfinis et si oui on les
			// renomme.
			// Modifi version 4.8
		}
		nbFinaux = nbFinauxTotal;
		listeEltsAjoutes.positionne(true, cadre.dimf); // Modifi version 4.8. True car on permet les calculs alatoires dans les objets
	}
	
	public void implementePourIter(CListeObjets listePourImp, CPrototype proto, MtgInternalFrame cadre) {
		int i;
		CListeObjets listeEltsAjoutes = new CListeObjets();
		proto.setListePourImplementation(listeProprietaire);
		// On ajoute l'objet implementationPrototype  la liste
		// Pour ne pas ajouter  la liste de multiples implmentations de mme CImplementationProt, on utilise un clone
		CImplementationProto imp = (CImplementationProto) this.getClone(listeProprietaire, listeProprietaire);
		imp.nomProto = proto.nom;
		listeProprietaire.add(imp); 	// Ncessaire d'utiliser un clone car lors des implmenntations rcursives
																																	// plusieus prototypes de mme nature sont utiliss
		// Important que que les indices soient corrects
		proto.indicePremierObjetPourImplementation = listeProprietaire.size();
		// On implmente la liste d'objets sources gre par l'implmentation de
		// prototype
		// Puis on ajoute des clones des objets intermdiaires et finaux du
		// prototype
		for (i = proto.nbObjSources; i < proto.size(); i++) {
			CElementBase elb = proto.get(i);
			CElementBase clone = elb.getClone(proto, listeProprietaire);
			clone.estElementFinal = elb.estElementFinal;
			clone.impProto = imp; // Important
			listePourImp.add(clone);
			listeEltsAjoutes.add(clone);
			// On regarde s'il y a des points ou des droites dans les lments
			// intermdiaires
			// ou finaux ayant le mme nom que d'autres dj dfinis et si oui on les
			// renomme.
			// Modifi version 4.8
		}
		listeEltsAjoutes.positionne(true, cadre.dimf); // Modifi version 4.8. True car on permet les calculs alatoires dans les objets
	}

/**
 * 	 * Ajout version 4.8
	 * Implmente le prototype proto de faon rcursive dans la liste listePourImp

 * @param listePourImp : la liste cible des implmentations
 * @param proto: le prototype (construction)  implmenter
 * @param cadre: le CFrame possdant la figure
 * @param listeSourcesConst: la liste forme des lments sources constants
 * @param listeSourcesVar: la liste forme des lments variant  chaque itration
 * @param pasRec:le pas de rcursion qui est le nombre d'objets  sauter avant de rimplmenter la construction
 * @param nbImp: le nombre de fois que la construction doit tre itre pour un mme niveau
 * @param profond: le niveau de rcursion (1 pour le niveau initial)
 * @param profondMax; le niveau maximum de rcursion autoris
 * @param nbTotImp: Le nombre total d'implmentations de la construction qui ont t implmentes
 */
	public void implementeRecursif(CListeObjets listePourImp, Vector<CPrototype> tableProto, MtgInternalFrame cadre, CListeObjets listeSourcesConst,
			CListeObjets listeSourcesVar, int pasRec, int nbImp, boolean creationNiveauMax, int profond, int profondMax, Vector <CNoeudPointeurSurPoint> listePoints) {
		// On commence par implmenter une premire fois la construction
		CPrototype proto = tableProto.get(profond);
		int ls = listeSourcesConst.size();
		int sizeInit = listePourImp.size();
		for (int i = 0; i < ls; i++) {
			((CElementGenerique) proto.get(i)).elementAssocie = listeSourcesConst.get(i);
		}
		int j = 0;
		for (int i = ls; i < proto.nbObjSources; i++) {
			((CElementGenerique) proto.get(i)).elementAssocie = listeSourcesVar.get(j);
			j++;
		}
		int nbObjFinaux = creationNiveauMax ? ((profond == profondMax) ? proto.nbObjFinaux : listeSourcesVar.size()+pasRec*(nbImp-1)): proto.nbObjFinaux; 
		implementePourRecur(listePourImp, proto, cadre, nbObjFinaux);
		if (profond == profondMax) {
			if (listePoints != null) {
				// On rajoute  listePoints les points de gnration ultime
				for (int i = sizeInit; i < listePourImp.size(); i++) {
					CElementBase elb = listePourImp.get(i);
					if (elb.estElementFinal && elb.estDeNature(NatureObjet.NToutPoint))
						listePoints.add(new CNoeudPointeurSurPoint(listePourImp, (CPointAncetre)elb));
				}
			}
			return;
		}
		// On cre une liste forme de tous les lments finaux crs par la premire implmentation
		CListeObjets listef = new CListeObjets();
		for (int i = sizeInit; i < listePourImp.size(); i++) {
			CElementBase elb = listePourImp.get(i);
			if (elb.estElementFinal  && (elb.estDeNature(NatureObjet.NToutObjet)
					|| !elb.estDeNatureCalcul(NatureCalcul.NAucunCalcul))) listef.add(elb);
		}
		// On applique la rcursion 
		for (int k = 0; k < nbImp; k++) {
			j = 0;
			CListeObjets newlisteSourcesVar = new CListeObjets();
			for (int i = ls; i < proto.nbObjSources; i++) {
				newlisteSourcesVar.add(listef.get(j + pasRec*k));
				j++;
			}			
			implementeRecursif(listePourImp, tableProto, cadre, listeSourcesConst, newlisteSourcesVar, pasRec, nbImp, creationNiveauMax, profond+1, profondMax, listePoints);
		}
	}

	/**
	 * Fonction renvoyant le nombre d'objets finaux de type calcul ayant un nom
	 * Est utilis lors de la fin de l'implmentation d'une construction pour
	 * proposer de renommer ces calculs
	 * 
	 * @return
	 */
	public int nombreObjetsFinauxTypeCalculNommes() {
		int res = 0;
		int ind = listeProprietaire.indexOf(this);
		for (int i = ind + 1; i <= ind + nbFinaux + nbIntermediaires; i++) {
			CElementBase elb = listeProprietaire.get(i);
			if (elb.estElementFinal && elb.estDeNatureCalcul(NatureCalcul.NToutCalculNommeSaufConst))
				res++;
		}
		return res;
	}
	
	/**
	 * Fonction rajout version 3.6.1 pour corriger des bugs lors du reclassement d'objets
	 * @return
	 */
	// Supprim version 4.9.2. Inutile.
	/*
	public void  determineNombreObjetsIntermediaires() {
		int indimp = listeProprietaire.indexOf(this);
		int nb = 0;
		for (int i = indimp+1; i < listeProprietaire.size(); i++) {
			CElementBase elb = listeProprietaire.get(i);
			if ((elb.impProto == this) && !elb.estElementFinal) nb++;
			else break;
		}
		nbIntermediaires = nb;
	}
	*/	
	/**
	 * Ajout version 4.9.2 : Donne  tous les points finaux la taille de caractres d'indice indTaille
	 * Utilis quand on cre une figure avec une figure prdfinie.
	 * @param indTaille
	 */
	public final void donneTailleNomsFinaux(int indTaille) {
		int ind = listeProprietaire.indexOf(this);
		for(int i = ind + 1; i <= ind + nbFinaux + nbIntermediaires; i++) {
			CElementBase elb = listeProprietaire.get(i);
			if (elb.estElementFinal && elb.estDeNature(NatureObjet.NToutPoint)) {
				CPointAncetre pt = (CPointAncetre) elb;
				pt.indiceTailleNom = indTaille;
			}
		}
	}
	// C++ : chargeDepuisFlux a t ajout  la version 2.5 pour cause d'implmentation
	// de prototype
  @Override
	public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
		super.chargeDepuisFlux(inps, list);
		nomProto = inps.readCString();
		nbIntermediaires = inps.readCInt();
		nbFinaux = inps.readCInt();
		listeSources.chargeDepuisFlux(inps, list);
	}
  
  // Ajout version 5.4
  /**
   * Fonction renvoyant le premier objet ce cette implmentation de prototype qui n'est pas un objet final
   * et qui est de nature nat
   * @param nat
   * @return
   */
  public CElementBase premierFinal(long nat) {
  	CListeObjets list = listeProprietaire;
  	for (int i = 1; i <= nbFinaux + nbIntermediaires; i++) {
  		CElementBase el = list.get(this.index + i);
  		if (!el.estElementIntermediaire() && el.estDeNature(nat)) return el;
  	}
  	return null;
  }
  
  // Ajout version 5.4
  /**
   * Fonction renvoyant le premier objet cette implmentation de prototype  partir de la fin qui n'est pas un objet finel
   * et qui est de nature nat
   * @param nat
   * @return
   */

  public CElementBase dernierFinal(long nat) {
  	CListeObjets list = listeProprietaire;
  	for (int i = nbFinaux + nbIntermediaires; i>0; i--) {
  		CElementBase el = list.get(this.index + i);
  		if (!el.estElementIntermediaire() && el.estDeNature(nat)) return el;
  	}
  	return null;
  }


  @Override
	public void read(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
		super.read(inps, list);
		nomProto = inps.readUTF();
		nbIntermediaires = inps.readInt();
		nbFinaux = inps.readInt();
		listeSources.read(inps, list);
		// Pour palier bug sur certaines figures
	}

  @Override
	public void write(MathGraphDataOutputStream oups, CListeObjets list) throws IOException {
		super.write(oups, list);
		oups.writeUTF(nomProto);
		oups.writeInt(nbIntermediaires);
		oups.writeInt(nbFinaux);
		listeSources.write(oups, listeProprietaire);
	}
}