/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.*;


import types.*;

import entreesSorties.*;
import glob.*;

// Modification pour la version 2.5 : la classe n'est plus abstract car il
// faut pouvoir l'instancier dans CListeObjets.get(int indice, Class cla)
import mathgraph32.MtgInternalFrame;
public class CMacro extends CAffichageLieAPoint {
	// Ajout version 5.1 : Sert  informer la macro quand elle s'excute de la CMacroSuiteMacro ventuelle qui l'a lance.
	// Si elle a t lance directement, sera null.
	protected CMacroSuiteMacros macroLanceuse = null;
	// Ajout version 4.9.9 pour gnrer un nom
	public static int indice = 0;
	//
	public String intitule;
  public String commentaireMacro;
  public boolean executionEnCours;
  public long timeDebut; // Le temps de dbut de l'animation (s'il y en a une de dure fixe)
  public long timeFin; // Le temps de fin d'animation (idem)

  // Les constantes suivantes permettent de savoir quel type d'excution recquiert la macro
  // quand elle a t clique : aucune animation, animation par thread ou animation par timer
  public static final int sansAnimation = 0;
  public static final int animationParTimer = 1;
  public static final int animationParThread = 2;
  /** Creates new CMacro */
  public CMacro() {
  	executionEnCours = false; // Ajout version 4.9.3
  }
  
  // Constructeur pour clonage
  public CMacro(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, CPointAncetre pointLieInit,
      int indiceTaillePoliceInit, boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
      int alignementHorizontalInit, int alignementVerticalInit) {
  	super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit,
          StyleEncadrement.Effet3D, effacementFondInit, couleurFondInit, alignementHorizontalInit, alignementVerticalInit);
    intitule = new String(intituleInit);
    commentaireMacro = new String(commentaireMacroInit);
  	executionEnCours = false; // Ajout version 4.9.3
  }
  
  // Il faut redfinir setClone  cause de l'affichage en relief
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    xEncadrement = ((CMacro)ptel).xEncadrement;
    yEncadrement = ((CMacro)ptel).yEncadrement;
    widthEncadrement = ((CMacro)ptel).widthEncadrement;
  }
   
  @Override
  public long getNature() {
    return NatureObjet.NMacro;
  }

  /**
   * Ajout version 4.8 : Renvoie true si la macro est une CMacroConstructionIterative ou une CMacroConstructionRecursive
   * ou une CMacroLSuiteMacros ne contenant que des macros du type prcdent.
   * Redfini pour CMacroConstructionIteartive, CMacroConnstructionRecursive et CMacroSuiteMacros
   * @return
   */
  public boolean utilisablePourDemarrage() {
  	return false;
  }
  /**
   * Fonction renvoyant true seulement si la macro descend de CMacroAvecListe
   * et est une macro  laquelle on peut ajouter ou retirer des objets
   * avec les outils d'jout ou de retrait d'objet graphqique  une macro
   * Redfini seulement pour CMacroAvecListe (true) et CMacroSuiteMacro (false);
   * @return
   */
  public boolean ajoutOuRetraitObjetsPossible() {
  	return false;
  }
  
  /**
   * Ajout pour la version 2.4.0 : renverra true si la macro est une macro de modification de variable,
   * d'affectation de valeur  une variable ou une CMacroSuiteMacros ne contenant que des macros du type prcdent
   * @return
   */
  public boolean peutEtreMacroFinBoucle() {
  	return false;
  }
  
  // Trace de l'intitul avec un effet de relief
  // Java : amlior par rapport au C++ en appelant la mthode de l'anctre
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
  	// Les macros qui sont masques ne doivent pas re affiches quand on active l'iutil rideau
  	// de dmasquage d'objet
  	if (masque) return;
    super.trace(pDC, couleurFond, transparence);
    int x1 = xEncadrement + widthEncadrement + 1;
    int y1 = yEncadrement + heightEncadrement + 1;
    pDC.drawLine(xEncadrement + 1, y1, x1, y1);
    pDC.drawLine(x1, yEncadrement + 1, x1, y1);
  }

  @Override
  protected String rendChaineAffichage() {
    return (char)8658 + intitule;
  }
 
  // MacroEnCours renvoie this except pour les macros de
  // type CMacroSuiteMacro
  public CMacro macroEnCours() {
    return this;
  }
  
  // Cette mthode doit tre appele  la fin de chaque termineAction() des descendants
  public final void DetruitTimersThread(MtgInternalFrame cadre) {
    cadre.timerTask = null;
    if (cadre.timer != null){
      cadre.timer.cancel();
      cadre.timer.purge();
      cadre.timer = null;
    }
    if (cadre.runner != null) {
    	cadre.runner = null;
    }
    Thread.yield();	// Pour viter une concurrentModificationException parce que la boucle tourne encore
		// et une liste est en cours de parcours par un ListIterator
  }
  // Mthode charge de crer les tier ou thread dont la macro a besoin pour s'excuter
  public void initialiseInterruptions(MtgInternalFrame cadre) {
    switch (typeAnimation()) {
      case CMacro.animationParThread :
        cadre.runner = new Thread(cadre);
        //applet.runner.setPriority(Thread.MAX_PRIORITY);
        cadre.runner.start();
        break;
      case CMacro.animationParTimer :
        cadre.timerTask = new MathGraphTimerTask(cadre);
        cadre.timer = new java.util.Timer();
        cadre.timer.scheduleAtFixedRate(cadre.timerTask, 0l, macroEnCours().frequenceAnimation());
      default :
    } 
  }
  // Java : Renvoie le type d'animation utilis par la macro quand elle est excute.
  // Voir les types dfinis dans les premires lignes pas dfini en C++)
  public int typeAnimation() {
    return sansAnimation;
  }
  // Java : les macros de dure d'animation fixe devront redfinir cette procdure.
  public long dureePourAnimation() {
    return 0;
  }
  
  /**
   * renvoie la frquence d'animation pour les macros d'animation
   * Doit tre redfini pour les macros utilisant une animation
   * @return
   */
  public int frequenceAnimation() {
    return 0;
  }
  
  public boolean executionPossible() {
    return true;
  }
 
  /**
   * Fonction renvoyant true si la macro est prvue pour tre arrte par un clic de souris.
   * Si la rponse est true et si la macro est contneue dans une macro excutant une suite de macros
   * un clic de souris fera passer  la macro suivante.
   * @return
   */
  public boolean arretParClic() {
  	return false;
  }
  // Redfini pour toutes les macros utilisant un thread.
  public void executePourThread(MtgInternalFrame cadre) {
  }

  // Redfini pour les macros utilisant un timer
  public void executePourTimer(MtgInternalFrame cadre, boolean repaint) {
  }
  
  // InitialisePourLancement ne fera rien et ne sera redfini que pour les macros excutant une suite de macros
  public void initialisePourLancement(MtgInternalFrame cadre) {
  }
  
  
  // Si lors de actionDansPaint(), appel dans paint(), la macro affiche d'abord
  // l'image de fond, cette procdure renvoie true (c'est le dfaut).
  // Renverra fausse pour les macros de trace et de trace automatique
  // ainsi que pour les macros de boucle avec trace d'objet
  public boolean affichageSurImFond() {
    return true;
  }
  
  // Le changement effectif de la figure doit se faire dans paint et non pas dans executePoutThread()
  // C'est ce que se charge de faire ActionDansPaint();  redfinir pour toutes les macros utilisant
  // un thread
  public void actionDansPaint(MtgInternalFrame cadre) {
  }
  
  /** Mthode charge de remettre la figure en tat initial */
  public void termineAction(MtgInternalFrame cadre) {
    DetruitTimersThread(cadre);
    executionEnCours = false;
    cadre.frame.indication("");
    /* Modifi version 5.1
    if (cadre.macroEnCours == this) cadre.macroEnCours = null;
    */
    if (macroLanceuse == null) cadre.macroEnCours = null;
    // Ajout version 5.1
    else {
    	macroLanceuse.passageMacroSuiv(cadre);
    }
    //
  }
  
 
  /**
   * Fonction redfinie seulement pour les macros pouvant tre excutes en dbut ou fin de boucle
   * @param cadre
   */
  public void executeSimple(MtgInternalFrame cadre) {
    // Redfini pour les descendants pouvant tre excuts dans une macro de boucle
                          // en dbut ou fin de boucle
  }
  
  // Java : deux mthodes de plus pour les macros qui seront pour certaines excutes par des thread
    
  public void actionAvantExecution(MtgInternalFrame cadre) {
  	// Modification version 5.1 pour informer cadre du fait que c'est cette macro qui est en cours d'excution
  	cadre.macroEnCours = this;
  	//
    executionEnCours = true;
    if (arretParClic()) cadre.frame.indication(getString("ch108"));
    else if ((typeAnimation() == CMacro.animationParThread) || (typeAnimation() == CMacro.animationParTimer))
    	cadre.frame.indication(getString("ch132"));
    else cadre.frame.indication("");
  }
  
  public boolean agitSurVariable(CVariableBornee var) { // Renvoie faux par dfaut. A redfinir pour les descendants
                              // agissant sur une variable
    return false;
  }
  
  /**
   * Ajout version 2.4.0 : Renvoie true si la macro affecte val ou un calcul dpendant de val  une variable
   * A redfinir pour les macros d'affectation de valeur  variable, de modification de variable
   * et d'excution d'une suite de macros
   * @param val
   * @return
   */
  public boolean affecteValeurAVariable(CValeurDynamique val) {
  	return false;
  }
 
// Nouveaut par rapport au C++ : certaines macros maintiennent une liste. Par exemple la macro
// de modification d'une variable contient une liste de pointeurs sur les objets de la liste
// dpendant de la variable. Cette liste ne peut pas tre initialise au moment o la macro est
// lus depuis lengthflux, puisque les objets suivants n'ont pas t chargs. D'o la dfinition
// d'une mthode etablitListeInterne charge de mettre  jour cette  liste. Par dfaut, ne fait
// rien sauf pour les macros qui la redfinissent.
  
  public void etablitListesInternes() {
  }
  
  @Override
  public String chaineDesignation() {
    return getString("designationMacro");
  }
   
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    // Modification pour la version 1.9.5 : filtarge des caractres grecs
    intitule = inps.readCStringAvecFiltrage();
    commentaireMacro = inps.readCString();
    // Pour compatibilit d'affichage avec C++
    y_nom += Kernel.decalagePourCompatibilite(intitule);
  }
  
  // Ajout version 4.8
  @Override
  public String informationModification() {
  	return getString("chaineOutil120");
  }
	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
    return nom() + " : " + getString("chinfo224") + " " + this.intitule;
	}
	/**
	 * Ajout version 5.1
	 * Fonction utilise dans ModificationMacroDlg. Est redfini seulement pour CMacroSuiteMacro
	 * @return
	 */
	public String infoPourDlg() {
		return infoPourHistorique();
	}

  // Ajout version 4.9.9
  public String genereNom() {
  	indice++;
  	return getString("rmac") + indice;
  }
  
  // Ajout version 5.1
  /**
   * Fonction servant  tablir macroLanceuse qui sert lors de l'excution d'une macro
   * Si macroLancante est null, c'est que la macro a t lance directement.
   * Sinon macroLancante pointe sur la CMacroSuiteMacros qui a lanc l'excution de ma macro
   * et, une fois celle-ci excuten, elle doit rendre la main  la CMacroSuiteMacros qui l'a lance
   * @param mac
   */
  public void setMacroLanceuse(CMacroSuiteMacros mac) {
  	macroLanceuse = mac;
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    intitule = inps.readUTF();
    commentaireMacro = inps.readUTF();
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeUTF(intitule);
    oups.writeUTF(commentaireMacro);
  }
}