/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;
import dialogs.MacroAffectationPointLieDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMacroAffectationPointLie extends CMacro {
	public CPointLie pointLieAPositionner;
	public CPointLie pointLieAssocie;

  /** Creates new CMacroAffectationPointLie */
  public CMacroAffectationPointLie() {
  }
  
  // Constructeur pour clonage
  public CMacroAffectationPointLie(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit,
      boolean effacementFondInit, Color couleurFondInit, String intituleInit,
      String commentaireMacroInit, CPointLie pointLieAPositionnerInit, CPointLie pointLieAssocieInit,
    	int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit,
          effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit,
					alignementHorizontalInit, alignementVerticalInit);
       pointLieAPositionner = pointLieAPositionnerInit;
       pointLieAssocie = pointLieAssocieInit;
  }
 
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(pointLieAPositionner);
    int ind2 = listeSource.indexOf(pointLieAssocie);
    int ind3 = listeSource.indexOf(pointLie);
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMacroAffectationPointLie(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
      (CPointAncetre)listeCible.get(ind3, CPointAncetre.class), indiceTaillePolice,
      effacementFond, couleurFondAffichage, intitule, commentaireMacro, (CPointLie)(listeCible.get(ind1, CPointLie.class)),
      (CPointLie)(listeCible.get(ind2, CPointLie.class)), alignementHorizontal, alignementVertical);
    return ptelb;
  }  

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || pointLieAPositionner.dependDe(p) || pointLieAssocie.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || pointLieAPositionner.dependDePourReclassement(p) || pointLieAssocie.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (super.dependDePourBoucle(p) || pointLieAPositionner.dependDePourBoucle(p)
      || pointLieAssocie.dependDePourBoucle(p));
  }

  @Override
  public boolean executionPossible() {
    return pointLieAPositionner.existe && pointLieAssocie.existe;
  }

  @Override
  public void actionDansPaint(MtgInternalFrame cadre) {
    // boolean infr = cadre.infRand;
    // DimensionFenetre dimfen = cadre.dimf;
    pointLieAPositionner.donneAbscisse(pointLieAssocie.abscisse);
    // Modification version 2.0
    /*
    CMathGrafDoc doc = cadre.document;
    cadre.listePr.effaceTout(cadre.gbuffer, dimfen, true, doc.couleurFond, doc.transparencePourSurfaces);
    listeProprietaire.positionne(infr, dimfen);
    cadre.afficheToutDansgbuffer();
    */
    cadre.redessineAvecCalcul();
    executionEnCours = false;
  }

  /*
   * Ajout version 4.9.3 : On ne peut pas supprimer une liaison entre un point li et un objet si ce point li
   * sert dans une telle macro.
   */
  @Override
	public boolean estGenereParPointLie(CPointLie pointlie) {
		return ((pointLieAssocie == pointlie) || (pointLieAPositionner == pointlie));
	}

  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
  	if (pointLieAssocie == ancienPoint) pointLieAssocie = (CPointLie) nouveauPoint;
  	if (pointLieAPositionner == ancienPoint) pointLieAPositionner = (CPointLie) nouveauPoint;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = nom() + " : " + getString("chinfo126") + " " + pointLieAPositionner.nom() + "\n" + getString("chinfo230") + " " + pointLieAssocie.nom();
    ch += "\n" + getString("macroApparitionDlg2") + " " + intitule;
  	if (pointLie != null) ch += "\n" + getString("chinfo200") + " " + pointLie.nom();
  	if (effacementFond) ch += "\n" + getString("paneAffichage1");
  	return ch;
  }
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
    return nom() + " : " + getString("chinfo126") + " " + pointLieAPositionner.nom() + " " + getString("chinfo230") + " " + pointLieAssocie.nom();
	}

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(pointLieAPositionner);
    liste.add(pointLieAssocie);
  }    
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 4.5.1
  	// return !estElementFinal;
  	return true;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroAffectationPointLieDlg(frame, this, true);
  }
  
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
		return false;
	}  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    pointLieAPositionner = (CPointLie) (list.get(ind1,CPointLie.class));
    int ind2 = inps.readCInt();
    pointLieAssocie = (CPointLie) (list.get(ind2,CPointLie.class));
  }
  /* Lecture depuis dans un flux java */  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    pointLieAPositionner = (CPointLie) (list.get(ind1,CPointLie.class));
    int ind2 = inps.readInt();
    pointLieAssocie = (CPointLie) (list.get(ind2,CPointLie.class));
  }  
  /* Enregistrement dans un flux java */
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(pointLieAPositionner);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(pointLieAssocie);
    oups.writeInt(ind2);
  }
}
