/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.*;


import dialogs.MacroAnimationDlg;
import dialogs.MtgDlg;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
/**
 * Dans la version logiciel java, listeDep est un membre dont les lments sont initialiss  la fin
 * du chargement d'une liste ou lors du clonage pour permettre le multitche.
 * @author Yves
 *
 */
public class CMacroAnimationPointLie extends CMacro {
	public boolean animationCyclique;
  public int frequenceAnimationMacro;
  public int nombrePointsPourAnimation;
	public int dureeAnimation; // Dure en diximes de seconde (0 si animation
											// jusqu' clic de souris)
	public CPointLie pointLieAssocie;
  private double abscisseInitiale;
  private CListeObjets listeDep;
  private double abscisse; // l'abscisse en cours du point li qui bouge
  private double abscisseMini;
  private double abscisseMaxi;
  private double pas;
  // Ajouts version 4.9.8
  public boolean retourDepart;
  public boolean unCycle;
  public boolean inverserSens;
  
  // L'animation se faisant par un thread, il faut momoriser l'abscisse du point li
  // gnrant l'animation pour la lui restituer  la fin (dans abscisseActuelle).
  // Au dbut de l'animation, le membre executionEnCours sera mis  true.
  // execute() sera lanc par le thread.
  // Quand l'utilisateur cliquera sur la figure ou en dehors, le membre animation
  // en cours sera mis  false via le membre macroEnCours de l'Applet et la boucle de execute()
  // cessera.

  /** Creates new CMacroAnimationPointLie */
  public CMacroAnimationPointLie() {
  }
  
  // Version 4.9.8. On change le numro de version.
  // On rajoute la possibilit de demander que le point mobile ne soit pas remis  son emplacement de dpart  la fin de l'animation,
  // la possibilit de demander un seul cycle complet et de demander un dplacement dans le sens inverse.
  @Override
  public int numeroVersion() {
  	return 2;
  }  

  
  // Constructeur pour clonage
  public CMacroAnimationPointLie(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit,
      boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
      boolean animationCycliqueInit, int frequenceAnimationMacroInit, int nombrePointsPourAnimationInit,
      	int dureeAnimationInit, CPointLie pointLieAssocieInit, int alignementHorizontalInit, int alignementVerticalInit,
      	boolean retourDepartInit, boolean unCycleInit, boolean inverserSensInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit,
          effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit,
					alignementHorizontalInit, alignementVerticalInit);
        animationCyclique = animationCycliqueInit;
        frequenceAnimationMacro = frequenceAnimationMacroInit;
        nombrePointsPourAnimation = nombrePointsPourAnimationInit;
        dureeAnimation = dureeAnimationInit;
        pointLieAssocie = pointLieAssocieInit;
        retourDepart = retourDepartInit;
        unCycle = unCycleInit;
        inverserSens = inverserSensInit;
        listeDep = new CListeObjets(liste);
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(pointLieAssocie);
    int ind2 = listeSource.indexOf(pointLie);    
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMacroAnimationPointLie(listeCible,
    	(CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
      (CPointAncetre)listeCible.get(ind2, CPointAncetre.class), indiceTaillePolice,
      effacementFond, couleurFondAffichage, intitule, commentaireMacro, animationCyclique, frequenceAnimationMacro,
      nombrePointsPourAnimation, dureeAnimation, (CPointLie)(listeCible.get(ind1, CPointLie.class)),
      alignementHorizontal, alignementVertical, retourDepart, unCycle, inverserSens);
    ((CMacroAnimationPointLie)ptelb).listeDep = new CListeObjets(listeCible);
    return ptelb;
  }

  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	return memoriseDependance(super.dependDe(p) || pointLieAssocie.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || pointLieAssocie.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	return super.dependDePourBoucle(p) || pointLieAssocie.dependDePourBoucle(p);
  }

  @Override
  public int typeAnimation() {
    return CMacro.animationParTimer;
  }

  @Override
  public int frequenceAnimation() {
    return frequenceAnimationMacro;
  }

  @Override
  public long dureePourAnimation() {
    return dureeAnimation;
  }
  
  @Override
  public boolean arretParClic() {
  	return (dureeAnimation == 0);
  }
  
  @Override
  public void metAJour() {
  	super.metAJour();
  	etablitListesInternes();
  }
  
  /**
   * Fonction prparant les listes dont la macro a besoin pour s'excuter
   */
  @Override
  public void etablitListesInternes() {
  	listeDep.retireTout();
    listeDep.ajouteObjetsDependantsSauf(pointLieAssocie, listeProprietaire, this);
  }
  
  @Override
  public void actionAvantExecution(MtgInternalFrame cadre) {
  	super.actionAvantExecution(cadre);  // Met executioEnCours  true
    // On mmorise la position actuelle du point li
    abscisseInitiale = pointLieAssocie.abscisse;
    abscisseMini = pointLieAssocie.abscisseMinimale();
    abscisseMaxi = pointLieAssocie.abscisseMaximale();
    if (abscisseMini > abscisseMaxi) {
      double d = abscisseMini;
      abscisseMini = abscisseMaxi;
      abscisseMaxi = d;
    }
    // Ligne suivante modifie pour la version 1.8
    pas = (abscisseMaxi-abscisseMini)/(nombrePointsPourAnimation - 1);
    // Modifi version 4.9.8
    abscisse = unCycle ? (inverserSens ? abscisseMaxi : abscisseMini) : abscisseInitiale;
    if (inverserSens) pas = -pas;
    //
    timeDebut = System.currentTimeMillis();
    if (dureeAnimation != 0) {
      timeFin = timeDebut + dureeAnimation*100; // La dure est en diximes de secondes
    }
  }

  // Cette fonction a t optimise pour la version 1.9.5
  // Corrig version 4.9.9 car si inverserSens tait true ne marchait pas pour les droites
  @Override
  public void executePourTimer(MtgInternalFrame cadre, boolean repaint) {
    if (!executionEnCours) return;
    abscisse = abscisse + pas;
    if (Kernel.zero13(abscisse - abscisseMaxi)) abscisse = abscisseMaxi;
    else
    	if (Kernel.zero13(abscisse - abscisseMini)) abscisse = abscisseMini;
    
    if (animationCyclique) {
      // Modifi version 4.9.9.2
    	if (inverserSens) {
    		if (abscisse < abscisseMini) abscisse = abscisseMaxi;
    	}
    	else if (abscisse > abscisseMaxi) abscisse = abscisseMini;
    }
    else
    {
      if (abscisse > abscisseMaxi)
      {
        pas = -pas;
        abscisse = abscisseMaxi;
      }
      else
      {
        if (abscisse < abscisseMini)
         {
           pas = - pas;
           abscisse = abscisseMini;
        }
      }
    }
    if (repaint) cadre.paneFigure.repaint();
  }  
  @Override
  public void actionDansPaint(MtgInternalFrame cadre) {
    boolean infor = cadre.infRand;
    DimensionFenetre dimfen = cadre.dimf;
    // Modification version 2.0
    // CMathGrafDoc doc = cadre.document;
    // listeDep.effaceTout(cadre.gbuffer, dimfen, true, doc.couleurFond, doc.transparencePourSurfaces);
    cadre.effacegbuffer();
    pointLieAssocie.donneAbscisse(abscisse);
    listeDep.positionne(infor, dimfen);
    cadre.afficheToutDansgbuffer();
    if (unCycle) {
    	if (inverserSens) {
    		if (Kernel.zero13(abscisse - abscisseMini) || (abscisse < abscisseMini)) termineAction(cadre);
    	}
    	else if (Kernel.zero13(abscisse - abscisseMaxi) || (abscisse > abscisseMaxi)) termineAction(cadre);
    }
    //
  }

  // Mthode charge de remettre la figure en tat initial aprs excution du thread s'il y en a un
  @Override
  public void termineAction(MtgInternalFrame cadre) {
    super.termineAction(cadre);
    // listeDep.effaceTout(cadre.gbuffer, dimfen, true, doc.couleurFond, doc.transparencePourSurfaces);
    if (retourDepart) pointLieAssocie.donneAbscisse(abscisseInitiale);
    // Ligne suivante supprime version 2.0
    // cadre.dessinegbuffer();
    cadre.repaintFigureAvecCalcul();
  }
  
  @Override
  public boolean executionPossible() {
    return pointLieAssocie.existe;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = nom() + " : " + getString("chinfo124") + " " + pointLieAssocie.nom();
    ch += "\n" + getString("macroApparitionDlg2") + " " + intitule;
  	if (pointLie != null) ch += "\n" + getString("chinfo200") + " " + pointLie.nom();
  	if (effacementFond) ch += "\n" + getString("paneAffichage1");
  	return ch;
  }
  
 	/**
 	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
 	 * A redfinir pour les descendants.
 	 * @return
 	 */
   @Override
 	public String info() {
     return nom() + " : " + getString("chinfo124") + " " + pointLieAssocie.nom();
 	}

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(pointLieAssocie);
  }  
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
  	if (pointLieAssocie == ancienPoint) pointLieAssocie = (CPointLie) nouveauPoint;
    // Suprim version 2.0 car provoques des stackoverflow
  	// listeDep.remplacePoint(ancienPoint, nouveauPoint);
  }  
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
		return 	super.estDefiniParObjetDans(listeOb)
			&& (listeOb.contains(pointLieAssocie) || pointLieAssocie.estDefiniParObjetDans(listeOb));
	}
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 4.5.1
  	// return !estElementFinal;
  	return true;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroAnimationDlg(frame, this, true);
  }
  
	/**
	 * Fonction renvoyant true si l'objet dpend des positions gnres pas les dplacements d'un point li
	 * A redfinir pour les lieux de points et lieux d'objets et macros utilisant les dplacements d'un point li
	 * @param pointlie
	 * @return
	 */
  @Override
	public boolean estGenereParPointLie(CPointLie pointlie) {
		return pointLieAssocie == pointlie;
	}
 
    
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    animationCyclique = inps.readBoolean();
		frequenceAnimationMacro = inps.readCInt();
    // Frquence  vitesse maxi remplace par 1/1000 de seconde
    if (frequenceAnimationMacro == 0) frequenceAnimationMacro = 1;
		nombrePointsPourAnimation = inps.readCInt();
		dureeAnimation = inps.readCInt();
    int ind1 = inps.readCInt();
    pointLieAssocie = (CPointLie) (list.get(ind1,CPointLie.class));
		// Ajout version 4.9.8
  	inverserSens = false;
  	retourDepart = true;
  	unCycle = false;
		//
    if (list.getClass() != CPrototype.class) {
    	listeDep = new CListeObjets(listeProprietaire);
    }
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    animationCyclique = inps.readBoolean();
		frequenceAnimationMacro = inps.readInt();
    // Frquence  vitesse maxi remplace par 1/1000 de seconde
    if (frequenceAnimationMacro == 0) frequenceAnimationMacro = 1;
		nombrePointsPourAnimation = inps.readInt();
		dureeAnimation = inps.readInt();   
    int ind1 = inps.readInt();
    pointLieAssocie = (CPointLie) (list.get(ind1,CPointLie.class));
    if (nVersion < 2) {
    	inverserSens = false;
    	retourDepart = true;
    	unCycle = false;
    }
    else {
    	inverserSens = inps.readBoolean();
    	retourDepart = inps.readBoolean();
    	unCycle = inps.readBoolean();
    }
    if (unCycle) dureeAnimation = 0;
    if (list.getClass() != CPrototype.class) {
    	listeDep = new CListeObjets(listeProprietaire);
    }
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeBoolean(animationCyclique);
    oups.writeInt(frequenceAnimationMacro);
    oups.writeInt(nombrePointsPourAnimation);
    oups.writeInt(dureeAnimation);
    int ind1 = list.indexOf(pointLieAssocie);
    oups.writeInt(ind1);
    // Ajout version 4.9.8
    oups.writeBoolean(inverserSens);
    oups.writeBoolean(retourDepart);
    oups.writeBoolean(unCycle);
  }
}
