/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.Color;

import dialogs.MacroApparitionDlg;
import dialogs.MtgDlg;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMacroApparition extends CMacroAvecListe {
  public boolean clicPourObjetSuivant;
  int indiceDernierObjetTraite; // sera incrment si clicPourObjetSuivant est true
  /** Creates new CMacroApparition */
  public CMacroApparition() {
  }
  
  // Constructeur pour clonage
  public CMacroApparition(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
    CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
    boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
    CSousListeObjets listeAssocieeInit, boolean clicPourObjetSuivantInit, int alignementHorizontalInit, int alignementVerticalInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
      dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit, encadrementInit,
      effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit, listeAssocieeInit,
      alignementHorizontalInit, alignementVerticalInit);
    clicPourObjetSuivant = clicPourObjetSuivantInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(pointLie);
    int ind2 = listeSource.indexOf(impProto);
    CSousListeObjets listeAssocieeClone = new CSousListeObjets();
    listeAssocieeClone.setImage(listeAssociee, listeSource, listeCible);
    CElementBase ptelb = new CMacroApparition(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, masque,
      (CPointAncetre)listeCible.get(ind1, CPointAncetre.class), indiceTaillePolice, encadrement,
      effacementFond, couleurFondAffichage, intitule, commentaireMacro, listeAssocieeClone, clicPourObjetSuivant,
      alignementHorizontal, alignementVertical);
    return ptelb;
  }
  
  @Override
  public void actionAvantExecution(MtgInternalFrame cadre) { // par dfaut, met executionEnCours  true;
  	super.actionAvantExecution(cadre);  // Met executioEnCours  true
    if (clicPourObjetSuivant) {
      indiceDernierObjetTraite = -1;
    }
  }
  
  /**
   * Version java . Pour la gestion des clics par l'outil d'excution de macro
   */
  @Override
  public void termineAction(MtgInternalFrame cadre) {
    super.termineAction(cadre);
  	if ((clicPourObjetSuivant) && (indiceDernierObjetTraite != listeAssociee.size() - 1)) executionEnCours = true;
  	else executionEnCours = false;
  }

  
  @Override
  public void actionDansPaint(MtgInternalFrame cadre) {
    if (clicPourObjetSuivant) {
      indiceDernierObjetTraite ++;

      CElementGraphique el = (CElementGraphique) listeAssociee.get(indiceDernierObjetTraite);
      if (!el.masque) {
        while(indiceDernierObjetTraite < listeAssociee.size()) {
          el = (CElementGraphique) listeAssociee.get(indiceDernierObjetTraite);
          if (el.masque) break;
          else indiceDernierObjetTraite++;
        }
        if (indiceDernierObjetTraite == listeAssociee.size()) {
          indiceDernierObjetTraite--;
          executionEnCours = false;
          cadre.frame.indication("");
          return;
        }
      }
           
      el.montre();
      executionEnCours = (indiceDernierObjetTraite != listeAssociee.size() - 1);
      if (executionEnCours) cadre.frame.indication(getString("ch107"));
      else cadre.frame.indication("");

    } else {
      listeAssociee.montreTout(true);
      executionEnCours = false;
    }
    // Ligne suivante ajoute version 2.0
    cadre.effacegbuffer();
    cadre.afficheToutDansgbuffer();
  }
    
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 4.5.1
  	// return !estElementFinal;
  	return true;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroApparitionDlg(frame, this, true);
  }
      
   @Override
   public String infoPourHistorique() {
     String ch = nom() + " : " + getString("chinfo119") + " " + getString("ch31") + "\n" + listeAssociee.listeNoms();
     ch += "\n" + getString("macroApparitionDlg2") + " " + intitule;
   	if (pointLie != null) ch += "\n" + getString("chinfo200") + " " + pointLie.nom();
   	if (effacementFond) ch += "\n" + getString("paneAffichage1");
   	return ch;
   }
   
  	/**
  	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
  	 * A redfinir pour les descendants.
  	 * @return
  	 */
    @Override
  	public String info() {
      return nom() + " : " + getString("chinfo119") + " " + listeAssociee.listeNoms();
  	}

   @Override
   public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
     liste.ajouteElementsDe(listeAssociee);
   }    
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    if (listeProprietaire.numeroVersion == 1) clicPourObjetSuivant = false;
    else clicPourObjetSuivant = inps.readBoolean();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    clicPourObjetSuivant = inps.readBoolean();
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeBoolean(clicPourObjetSuivant);
  }  
}
