/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.util.*;
import java.awt.Color;

import dialogs.MacroSuiteMacrosDlg;
import dialogs.MtgDlg;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

public class CMacroSuiteMacros extends CMacroAvecListe {
  private int indiceMacroEnCours; // Indice de la macro en cours d'excution

  /** Creates new CMacroSuiteMacros */
  public CMacroSuiteMacros() {
  }
  
  // Constructeur pour clonage
  public CMacroSuiteMacros(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
      boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
      CSousListeObjets listeAssocieeInit, int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit, encadrementInit,
          effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit, listeAssocieeInit,
          alignementHorizontalInit, alignementVerticalInit);
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(pointLie);
    int ind2 = listeSource.indexOf(impProto);        
    CSousListeObjets listeAssocieeClone = new CSousListeObjets();
    listeAssocieeClone.setImage(listeAssociee, listeSource, listeCible);
    CElementBase ptelb = new CMacroSuiteMacros(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur,x_nom, y_nom, dec_x, dec_y, 
      masque, (CPointAncetre)listeCible.get(ind1, CPointAncetre.class), indiceTaillePolice, encadrement,
      effacementFond, couleurFondAffichage, intitule, commentaireMacro, listeAssocieeClone, alignementHorizontal, alignementVertical);
    return ptelb;
  }
  
  /**
   * Ajout version 4.8 : Renvoie true si la macro est une CMacroConstructionIterative ou une CMacroConstructionRecursive
   * ou une CMacroLSuiteMacros ne contenant que des macros du type prcdent.
   * Redfini pour CMacroConstructionIteartive, CMacroConnstructionRecursive et CMacroSuiteMacros
   * @return
   */
  public boolean utilisablePourDemarrage() {
    boolean resultat = true;
    ListIterator<CElementBase> li = listeAssociee.listIterator();
    while (li.hasNext()) {
      CMacro ptMacro = (CMacro) (li.next());
      resultat = resultat && ptMacro.utilisablePourDemarrage();
    }
    return resultat;
  }

  // Ajout version 4.8
	@Override
	public boolean dependDeProto(CPrototype proto) {
    boolean resultat = false;
    ListIterator<CElementBase> li = listeAssociee.listIterator();
    while (li.hasNext()) {
      CMacro ptMacro = (CMacro) (li.next());
      resultat = resultat || ptMacro.dependDeProto(proto);
    }
    return resultat;
	}

  // MacroEnCours renvoie this except pour les macros de
  // type CMacroSuiteMacro
  @Override
  public CMacro macroEnCours() { 
    if (indiceMacroEnCours < listeAssociee.size()) 
       // return (CMacro)(listeAssociee.get(indiceMacroEnCours)); Moodifi version 5.1
    	return ((CMacro)(listeAssociee.get(indiceMacroEnCours))).macroEnCours();
    else return this;
  }

  // Ajout version 5.1
  @Override
  public void setMacroLanceuse(CMacroSuiteMacros mac) {
  	super.setMacroLanceuse(mac);
  	indiceMacroEnCours = 0;
  }

  @Override
  public void initialisePourLancement(MtgInternalFrame cadre) {
  	CMacro ptMacro = (CMacro)(listeAssociee.get(0));
  	ptMacro.setMacroLanceuse(this); // Ajout version 5.1
  	ptMacro.initialisePourLancement(cadre); // Si la premire macro est une CMacroSuiteMacro il faut l'informer de sa macro laneuse
    indiceMacroEnCours = 0;
    // macroEnCours().executionEnCours = true; // A revoir
  }
 
  @Override
  public int typeAnimation() {
    if (indiceMacroEnCours < listeAssociee.size()) 
       return macroEnCours().typeAnimation();
    else return CMacro.sansAnimation;
  }
  
  public void passageMacroSuiv(MtgInternalFrame cadre) {
  	// Modification java : si la macro en cours est une macro d'apparition d'objets avec clic pour
  	// passer  l'objet suivant, on ne passe  la macro suivante qui si le dernier objet a t affich.
  	// Sinon on fait apparaitre l'objet suivant
  	if (macroEnCours().getClass() == CMacroApparition.class) {
  		CMacroApparition macroapp = (CMacroApparition) macroEnCours();
  		if (macroapp.clicPourObjetSuivant && (macroapp.indiceDernierObjetTraite < macroapp.listeAssociee.size() - 1)) {
  			cadre.paneFigure.repaint();
  			return;
  		}
  	}
    indiceMacroEnCours++;
     if (indiceMacroEnCours < listeAssociee.size()) {
    	// Modifi version 4.9.7.1
    	// Version 5.1 : Si la macro suivante est une CMacroSuiteMacro il faut appeller initialise pour que les macros qu'elle appellent sachent qu'elle est la macro appelante
    	 ((CMacro)listeAssociee.get(indiceMacroEnCours)).setMacroLanceuse(this);
    	((CMacro)listeAssociee.get(indiceMacroEnCours)).initialisePourLancement(cadre);
    	CMacro mac = macroEnCours(); // Attention : Cette macro ne peut pas tre une CMacroSuiteMacro
    	cadre.macroEnCours = mac; // Ajout version 5.1
      mac.actionAvantExecution(cadre);
      mac.initialiseInterruptions(cadre);// Si la macro suivante ne marche pas par interruptions
      // il faut envoyer un repaint
      if (mac.typeAnimation() == CMacro.sansAnimation) cadre.paneFigure.repaint();
      executionEnCours = true;
    }
    // Modification version 5.1
   //  else executionEnCours = false;
    else {
    	executionEnCours = false;
    	if (macroLanceuse != null) 
    		macroLanceuse.passageMacroSuiv(cadre);
    	else cadre.macroEnCours = null; // Ajout version 5.0
    }
  }
    
  @Override
  public boolean agitSurVariable(CVariableBornee var) {
    boolean resultat = false;
    ListIterator<CElementBase> li = listeAssociee.listIterator();
    while (li.hasNext()) {
      CMacro ptMacro = (CMacro) (li.next());
      resultat = resultat || ptMacro.agitSurVariable(var);
    }
    return resultat;
  }
  
  @Override
  public boolean affecteValeurAVariable(CValeurDynamique val) {
    boolean resultat = false;
    ListIterator<CElementBase> li = listeAssociee.listIterator();
    while (li.hasNext()) {
      CMacro ptMacro = (CMacro) (li.next());
      resultat = resultat || ptMacro.affecteValeurAVariable(val);
    }
    return resultat;
  }
  
  /**
   * Une macro excutant une suite de macros ne peut tre une macro de fin de boucle pour une macro de boucle
   * que si elle ne contient que des macros pouvant elles-mmes tre des macros de fin de boucle.
   */
  @Override
  public boolean peutEtreMacroFinBoucle() {
    boolean resultat = true;
    ListIterator<CElementBase> li = listeAssociee.listIterator();
    while (li.hasNext()) {
      CMacro ptMacro = (CMacro) (li.next());
      resultat = resultat && ptMacro.peutEtreMacroFinBoucle();
    }
    return resultat;
  }
  

  // Nouveau version 3.0
  @Override
  public void actionDansPaint(MtgInternalFrame cadre) {
    while (indiceMacroEnCours < listeAssociee.size()) {
      CMacro ptMacro = (CMacro) (listeAssociee.get(indiceMacroEnCours));
    	if (ptMacro.typeAnimation() == CMacro.sansAnimation) {
    		//Modifi version 2.4.0
    		ptMacro.actionDansPaint(cadre);
    		indiceMacroEnCours++;
    	}
    	else break;
    }
    if (indiceMacroEnCours == listeAssociee.size()) executionEnCours = false;
  }
  
  @Override
  public void executeSimple(MtgInternalFrame cadre) {
    ListIterator<CElementBase> li = listeAssociee.listIterator();
    while (li.hasNext()) {
      CMacro ptMacro = (CMacro) (li.next());
      ptMacro.executeSimple(cadre);
    }
  }
  
  
  /**
   * Fonction renvoyant true seulement si la macro descend de CMacroAvecListe
   * et est une macro  laquelle on peut ajouter ou retirer des objets
   * avec les outils d'jout ou de retrait d'objet graphqique  une macro
   * Redfini seulement pour CMacroAvecListe (true) et CMacroSuiteMacro (false);
   * @return
   */
  public boolean ajoutOuRetraitObjetsPossible() {
  	return false;
  }
  
  @Override
  public boolean executionPossible() {
    boolean resultat = true;
    ListIterator<CElementBase> li = listeAssociee.listIterator();
    while (li.hasNext()) {
      CMacro ptMacro = (CMacro) (li.next());
      resultat = resultat && ptMacro.executionPossible();
    }
    return resultat;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch =  nom() + " : " + getString("chinfo135") + "\n" + listeAssociee.listeNoms();
		ch += "\n" + getString("macroApparitionDlg2") + " " + intitule;
		if (pointLie != null) ch += "\n" + getString("chinfo200") + " " + pointLie.nom();
		if (effacementFond) ch += "\n" + getString("paneAffichage1");
		return ch;
  }
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
    return nom() + " : " + getString("chinfo135") + " " + listeAssociee.listeNoms();
	}

	/**
	 * Ajout version 5.1
	 * Fonction utilise dans ModificationMacroDlg. Est redfini seulement pour CMacroSuiteMacro
	 * @return
	 */
  @Override
	public String infoPourDlg() {
    String ch =  nom() + " : " + getString("chinfo135") + " : ";
    for (int i = 0; i < listeAssociee.size(); i++) {
    	ch += ((CMacro)listeAssociee.get(i)).intitule;
    	if (i != listeAssociee.size()-1)     	ch += " , ";
    }
		ch += "\n" + getString("macroApparitionDlg2") + " " + intitule;
		if (pointLie != null) ch += "\n" + getString("chinfo200") + " " + pointLie.nom();
		if (effacementFond) ch += "\n" + getString("paneAffichage1");
		return ch;
	}
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new MacroSuiteMacrosDlg(frame, this, true);
  }
  
   @Override
   public void termineAction(MtgInternalFrame cadre) {
     super.termineAction(cadre);
     CMacro mac = macroEnCours();
     if (mac != this) mac.termineAction(cadre);
   }
  
}
