/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

import mathgraph32.MtgFrame;

//Version 5.0. L'objet descend maintenant de CCalculAncetre
// public class CMesureAbscisse extends CValeurDynamique {
public class CMesureAbscisse extends CCalculAncetre {
	double abscisse;
	CPointAncetre origine;
	CPointAncetre extremite;
	CPointAncetre pointMesure;
	private final String debutNom = "abs"; // Ajout version 5.0
	
  /** Creates new CMesureAbscisse */
  public CMesureAbscisse() {
  }
  
  /**
   * Version 5.0. Lors de la cration, l'utilisateur affecte un nom  la mesure d'o un changement de version
   */
  @Override
  public int numeroVersion() {
  	return 2;
  }  

  // Constructeur pour clonage
  /* Modifi version 5.0. Un paramtre de plus pour le nom
  public CMesureAbscisse(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPointAncetre origineInit, CPointAncetre extremiteInit, CPointAncetre pointMesureInit) {
    super(liste, impProtoInit, estElementFinalInit);
    origine = origineInit;
    extremite = extremiteInit;
    pointMesure = pointMesureInit;
  }
  */
  public CMesureAbscisse(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPointAncetre origineInit, CPointAncetre extremiteInit, CPointAncetre pointMesureInit, String nomCalculInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit);
    origine = origineInit;
    extremite = extremiteInit;
    pointMesure = pointMesureInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(origine);
    int ind2 = listeSource.indexOf(extremite);   
    int ind3 = listeSource.indexOf(pointMesure);   
    int ind4 = listeSource.indexOf(impProto);    
    // Si le nom est vide, c'est que la liste source est un CPrototype. On gnre alors un nom
    String nomCalc = (nomCalcul.isEmpty()) ? listeCible.genereNomPourCalcul(debutNom, true) : nomCalcul;
    CElementBase ptelb = new CMesureAbscisse(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)), (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)), nomCalc);
    return ptelb;    
  }

  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureAbscisseSurDroite;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch =  nom() + " : " + getString("chinfo31") + " " + pointMesure.nom() + "\n" + getString("chinfo32") 
            + " (" + origine.nom() + "," + extremite.nom() + ")";
    if (existe) ch = ch + "\n" + getString("chinfo97") + Kernel.chaineNombre(abscisse, 16);
    else ch = ch + "\n" + getString("ch18");
    return ch;
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nom() + " : " + getString("chinfo31") + " " + pointMesure.nom() + " " + getString("chinfo32") 
        + " (" + origine.nom() + "," + extremite.nom() + ")";
	}
  
  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(origine);
    liste.add(extremite);
    liste.add(pointMesure);
  } 
  
  
  @Override
  public void setClone(CElementBase ptel) {
    abscisse = ((CMesureAbscisse)ptel).abscisse;
  }  
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || origine.dependDe(p) || extremite.dependDe(p) || pointMesure.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || origine.dependDePourReclassement(p)
    	|| extremite.dependDePourReclassement(p) || pointMesure.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || origine.dependDePourBoucle(p) || extremite.dependDePourBoucle(p)
      || pointMesure.dependDePourBoucle(p);
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = origine.existe && extremite.existe
			&& pointMesure.existe && !(origine.egal(extremite));
    existe = existe && Kernel.aligne(origine.x,origine.y,extremite.x,
			extremite.y,pointMesure.x,pointMesure.y);
    if (!existe) return;
    // L'orientation sur l'cran est l'inverse de celle du repre
		// choisi.Il faut donc permuter les rles de u et v}
    VecteurDirecteur v = new VecteurDirecteur(origine,extremite);
    VecteurDirecteur u = new VecteurDirecteur(origine,pointMesure);
		// Nouvelle modif
    if (Math.abs(v.x)>Math.abs(v.y)) abscisse = u.x/v.x;
    else abscisse = u.y/v.y;
  }

  @Override
  public boolean nomIndispensable(CElementGraphique el) {
    return ((el == origine) || (el == extremite) || (el == pointMesure));
  }
  
  @Override
  public double rendValeur() {
    return abscisse;
  }

  /* Supprim version 5.0. La mesure d'abscisse a maintenant un nom
  @Override
  public String nom() {
    return pointMesure.nom + "(" + origine.nom + "," + extremite.nom + ")";
  }
  */
  
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CMesureAbscisse pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMesureAbscisse) p;
      return ((origine == pt.origine)
			&& (extremite == pt.extremite
			&& (pointMesure == pt.pointMesure)));
    }
    else return false;
  }    

  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (origine == ancienPoint) origine = nouveauPoint;
    if (extremite == ancienPoint) extremite = nouveauPoint;
    if (pointMesure == ancienPoint) pointMesure = nouveauPoint;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(origine) || origine.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(extremite) || extremite.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(pointMesure) || pointMesure.estDefiniParObjetDans(listeOb));
	}  
      
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     int ind3 = inps.readCInt();
     origine = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     extremite = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
     pointMesure = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
   	if (listeProprietaire.getClass() == CPrototype.class) nomCalcul = "";
   	else nomCalcul = listeProprietaire.genereNomPourCalcul(debutNom, true);
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
  	// Modifi version 5.0
    if (nVersion>=2) super.read(inps, list);
    else {
    	super.readSansNom(inps, list);
    	// Attention : Si la liste propritaire est un CPrototype, on ne peut pas lui demander  ce
    	// niveau de gnrer un nom.
    	// Ce sera fait dans getclone() en testant si le nom est vide.
    	if (listeProprietaire.getClass() == CPrototype.class) nomCalcul = "";
    	else nomCalcul = listeProprietaire.genereNomPourCalcul(debutNom, true);
    }
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    int ind3 = inps.readInt();
    origine = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    extremite = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    pointMesure = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(origine);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(extremite);
    oups.writeInt(ind2);
    int ind3 = list.indexOf(pointMesure);
    oups.writeInt(ind3);
  }  
}
