/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.IOException;

import mathgraph32.MtgFrame;
import types.Complexe;
import types.DimensionFenetre;
import types.ErreurCalculException;
import types.NatureCalcul;
import dialogs.MtgDlg;
import dialogs.SuiteRecurrenteComplexe3Dlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CSuiteRecComplexe3 extends CSuiteRecComplexe {
  public CValeurAssocieeComp deuxiemeTerme;
  Complexe w = new Complexe();
  
  public CSuiteRecComplexe3() {
  }
  
  /**
   * Constructeur pour cration d'objets
   * @param liste
   * @param nombreTermesInit
   * @param premierTermeInit
   */
  public CSuiteRecComplexe3(CListeObjets liste, CValeurAssocieeAVariable nombreTermesInit, CValeurAssocieeComp premierTermeInit,
  		CValeurAssocieeComp deuxiemeTermeInit) {
  	super(liste, nombreTermesInit, premierTermeInit);
  	deuxiemeTerme = deuxiemeTermeInit;
  }
  
  // Constructeur pour clonage
  public CSuiteRecComplexe3(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomCalculInit, CFoncComplexeNVar fonctionAssocieeInit,
    CValeurAssocieeAVariable nombreTermesInit, CValeurAssocieeComp premierTermeInit, CValeurAssocieeComp deuxiemeTermeInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit, fonctionAssocieeInit, nombreTermesInit, premierTermeInit);
    deuxiemeTerme = deuxiemeTermeInit;
  }
    
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(fonctionAssociee);
    int ind2 = listeSource.indexOf(impProto);
    CValeurAssocieeComp premierTermeClone =
      (CValeurAssocieeComp) premierTerme.getClone(listeSource, listeCible);
    CValeurAssocieeComp deuxiemeTermeClone =
        (CValeurAssocieeComp) deuxiemeTerme.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable nombreTermesClone =
      (CValeurAssocieeAVariable) nombreTermes.getClone(listeSource, listeCible);
    CElementBase ptelb = new CSuiteRecComplexe3(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CFoncComplexeNVar)(listeCible.get(ind1, CFoncComplexeNVar.class)), nombreTermesClone,
      premierTermeClone, deuxiemeTermeClone);
    return ptelb;
  }

  // Ajout version 4.8
  @Override
  public void initialisePourDependance() {
  	super.initialisePourDependance();
  	deuxiemeTerme.initialisePourDependance();
  }

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || deuxiemeTerme.dependDe(p));
  }
  
  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || deuxiemeTerme.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return super.dependDePourBoucle(p) || deuxiemeTerme.dependDePourBoucle(p);
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		super.estDefiniParObjetDans(listeOb)
		&&
		(deuxiemeTerme.estDefiniParObjetDans(listeOb));
	}  


  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    premierTerme.positionne(infoRandom, dimfen);
    deuxiemeTerme.positionne(infoRandom, dimfen);
    nombreTermes.positionne(infoRandom, dimfen);
    existe = fonctionAssociee.existe && premierTerme.existe() && deuxiemeTerme.existe() && nombreTermes.existe();
    if (!existe) return;
    double dnbt = Math.floor(nombreTermes.rendValeur() + 0.5);
    existe = (dnbt >= 2) && (dnbt <= NombreMaxiTermesSuiteRec);
    if (!existe) return;
    if ((int) dnbt != nbt) {
      nbt = (int) dnbt;
      metAJourNombreTermes();
    }    
    premierTerme.rendValeurComplexe(u);
    valeurs[0].set(u);
    deuxiemeTerme.rendValeurComplexe(v);
    valeurs[1].set(v);
    indiceDernierTermeExistant = 0;
    int i = 2;
    while (i < nbt) {
      try {
        listeProprietaire.initialiseNombreIterations();
        // fonctionAssociee.calcul.resultatFonctionComplexe(infoRandom, u, v);
        Complexe[] arg = new Complexe[3];
        arg[0] = new Complexe();
        arg[0].x = i - 2;
        arg[0].y = 0;
        arg[1] = new Complexe(v);
        arg[2] = new Complexe(u);
        
        fonctionAssociee.calcul.resultatFonctionComplexe(true, arg, w);
        if (w.isNanOrInfinite())  break;
        u.set(v);
        v.set(w);
        valeurs[i].set(w);
        i++;
      } catch (ErreurCalculException e) {
        break;
      }
    }
    i--;
    indiceDernierTermeExistant = i;
  }
  
  @Override
	public boolean confonduAvec(CElementBase p) {
    CSuiteRecComplexe3 pt;
    if (p.getClass() == this.getClass()) {
      pt = (CSuiteRecComplexe3) p;
      return (fonctionAssociee == pt.fonctionAssociee) && (premierTerme.confonduAvec(pt.premierTerme)
      		&& (deuxiemeTerme.confonduAvec(pt.deuxiemeTerme)) && (nombreTermes.confonduAvec(pt.nombreTermes)));
    }
    else return false;
	}
 
  // Modifi version 4.9.9
  @Override
  public String infoPourHistorique() {
  	String ch = nomCalcul + " : " + getString("chinfo187") + " \n"+ getString("chinfo84") + " " + fonctionAssociee.nom() + "\n" +
  	getString("chinfo85") + " " + premierTerme.chaineInfo() + "\n" +
  	getString("chinfo186") + " " + deuxiemeTerme.chaineInfo() + "\n" +
  	getString("chinfo86") + " " + nombreTermes.chaineInfo();
    if (!existe) ch = ch + "\n" + getString("ch18");
    return ch;
  }
 	// Ajout version 4.9.9
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
  	return nomCalcul + " : " + getString("chinfo187")+ " " + getString("chinfo84") + " " + fonctionAssociee.nom();
	}
    
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new SuiteRecurrenteComplexe3Dlg(frame, this, true);
  }  
    
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NSuiteRecurrenteComplexe3;
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
 	 super.read(inps, list);
 	 deuxiemeTerme = new CValeurAssocieeComp();
 	 deuxiemeTerme.read(inps, list);  	 
  }
 
  @Override
 public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
   super.write(oups, list);
   deuxiemeTerme.write(oups, list);  	 
 }

}
