/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.*;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

import mathgraph32.MtgFrame;


public class CSurfaceLieuDeuxPoints extends CSurface {
  // Les deux points dlimitant le lieu
	CPointAncetre point1;
	CPointAncetre point2;
	private int[] abscisses;
  private int[] ordonnees;
	double x1,y1,x2,y2,x3,y3;
	CPointBase pointDebutLieu;
	CPointBase pointFinLieu;
	CSegment segmentDebut;
	CSegment segmentFin;
	CSegment segmentDroite;
  private int nombrePointsLigne;

  /** Creates new CSurfaceLieuDeuxPoints */
  public CSurfaceLieuDeuxPoints() {
    pointDebutLieu = new CPointBase();
    pointFinLieu = new CPointBase();
    segmentDebut = new CSegment();
    segmentFin = new CSegment();
    segmentDroite = new CSegment();
  }

  // Constructeur pour clonage
  public CSurfaceLieuDeuxPoints(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, int styleRemplissageInit, CElementGraphique bordInit,
      CPointAncetre point1Init, CPointAncetre point2Init) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleRemplissageInit, bordInit);
    point1 = point1Init;
    point2 = point2Init;
    pointDebutLieu = new CPointBase();
    pointFinLieu = new CPointBase();
    segmentDebut = new CSegment(liste, pointDebutLieu, point1);
    segmentFin = new CSegment(liste, point2, pointFinLieu);
    segmentDroite = new CSegment(liste, point1, point2);
    // On cre les tableaux devat recevoir les coordonnes. Le + 3 est du aux deux points projets
    // sur la droite
    metAJourTableaux();
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(bord);
    int ind2 = listeSource.indexOf(point1);
    int ind3 = listeSource.indexOf(point2);
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CSurfaceLieuDeuxPoints(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, masque, styleRemplissage, (CLieuDeBase)listeCible.get(ind1, CLieuDeBase.class),
      (CPointAncetre)listeCible.get(ind2, CPointAncetre.class), (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)));
    return ptelb;
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    if (ptel.existe) {
	    nombrePointsLigne = ((CSurfaceLieuDeuxPoints)ptel).nombrePointsLigne;
	    // Modifi pour la version 1.9.5
	    // int nbPointsADecaler = nombrePointsLigne + 3;
	    int nbPointsADecaler = nombrePointsLigne + 2;    
	    System.arraycopy(((CSurfaceLieuDeuxPoints)ptel).abscisses, 0, abscisses, 0, nbPointsADecaler);
	    System.arraycopy(((CSurfaceLieuDeuxPoints)ptel).ordonnees, 0, ordonnees, 0, nbPointsADecaler);
    }
  }
  
  /**
   * Fonction appele quand on change le nombre de points du lieu qui est le bord de la surface
   */
  @Override
  public void metAJourTableaux() {
  	// + 3 et non +2 car si le lieu est transform en un lieu ferm il aura un point de plus
    abscisses = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 3];
    ordonnees = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 3];  	
  }
  
  @Override
  public boolean aPourBord(CLieuDeBase lieu) {
  	return (lieu == bord);
  }  
  
 	// Ajout version 4.9.9
  // Modifi version 5.3
	/**
	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
	 * A redfinir pour les descendants.
	 * @return
	 */
  @Override
	public String info() {
    return nom() + ":" + getString("chinfo60") + " " + bord.nom() + ", " +
    		point1.nom() + " " + getString("chinfo101") + " " + point2.nom();
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(bord);
    liste.add(point1);
    liste.add(point2);
  }   
  
  @Override
  public boolean confonduAvec(CElementBase p) {
  	boolean res = super.confonduAvec(p);
   	if (!res) return false;
  	CSurfaceLieuDeuxPoints pt = (CSurfaceLieuDeuxPoints) p;
  	return (point1 == pt.point1) && (point2 == pt.point2);
  }
  
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || point1.dependDe(p) || point2.dependDe(p));
  }

  // Modifi version 4.8
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
  	if (elementTestePourDependDePourRec == p) return dependDeElementTestePourRec;
    return memoriseDependancePourRec(super.dependDePourReclassement(p) || point1.dependDePourReclassement(p) || point2.dependDePourReclassement(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return super.dependDePourBoucle(p) || point1.dependDePourBoucle(p) || point2.dependDePourBoucle(p);
  }

  @Override
  public boolean dependDePourCapture(CElementBase p) {
    return super.dependDePourCapture(p) || point1.dependDePourCapture(p) || point2.dependDePourCapture(p);
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double x4, y4, xaux, yaux;

    CLieuDeBase lieu = (CLieuDeBase) bord;
    existe = lieu.existe && point1.existe && point2.existe && lieu.contientUneSeuleLigne(); // Modifi version 4.8.1
    if (!existe) return;

    // On recopie les coordonnes des points du lieu
    int ideb = lieu.infoLignes[lieu.indicePremiereLigne].indicePremierPoint;
    nombrePointsLigne = lieu.infoLignes[lieu.indicePremiereLigne].nombrePoints;
    System.arraycopy(lieu.abscissesCoordonneesPoints, ideb, abscisses, 0, nombrePointsLigne);
    System.arraycopy(lieu.ordonneesCoordonneesPoints, ideb, ordonnees, 0, nombrePointsLigne);

    int indicePremierPoint = lieu.infoLignes[lieu.indicePremiereLigne].indicePremierPoint;
    int indiceDernierPoint = lieu.infoLignes[lieu.indicePremiereLigne].indicePremierPoint
      + lieu.infoLignes[lieu.indicePremiereLigne].nombrePoints-1 ;

    x1 = lieu.abscissesCoordonneesPoints[indicePremierPoint];
    y1 = lieu.ordonneesCoordonneesPoints[indicePremierPoint];
    x4 = lieu.abscissesCoordonneesPoints[indiceDernierPoint];
    y4 = lieu.ordonneesCoordonneesPoints[indiceDernierPoint];
    pointDebutLieu.placeEn(x1,y1);
    pointFinLieu.placeEn(x4,y4);
    x2 = point1.x;
    y2 = point1.y;
    x3 = point2.x;
    y3 = point2.y;
    // Dans la mesure du posible, on s'arrange pour que le lieu soit 
    // convexe
    if ((Kernel.distancePointPoint(x1,y1,x3,y3 )< Kernel.distancePointPoint(x1,y1,x2,y2))
      && (Kernel.distancePointPoint(x4,y4,x2,y2) < Kernel.distancePointPoint(x4,y4,x3,y3)))
    {
      xaux = x3;
      yaux = y3;
      x3 = x2;
      y3 = y2;
      x2 = xaux;
      y2 = yaux;
      segmentDebut.point2 = point2;
      segmentFin.point1 = point1;
    };
    abscisses[nombrePointsLigne] = (int) Math.round(x3);
    ordonnees[nombrePointsLigne] = (int) Math.round(y3);
    abscisses[nombrePointsLigne+1] = (int) Math.round(x2);
    ordonnees[nombrePointsLigne+1] = (int) Math.round(y2);
    // Supprim pour la version 1.9.5. Inutie car le polygone
    // se referme lors du trac
    // abscisses[nombrePointsLigne+2] = (int) Math.round(x1);
    // ordonnees[nombrePointsLigne+2] = (int) Math.round(y1);
    segmentDebut.positionne(infoRandom,dimfen);
    segmentDroite.positionne(infoRandom,dimfen);
    segmentFin.positionne(infoRandom,dimfen);
    existe = (segmentDebut.existe) && (segmentDroite.existe) && (segmentFin.existe);
  }

  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    int styleRemplissageEffectif;
    if (masque) styleRemplissageEffectif = StyleRemplissage.QuadrillageCroise;
    else styleRemplissageEffectif = styleRemplissage;
    if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePleinTransp) {
    	if (transparence != 1) {
    		pDC.setColor(couleur);
  	    Composite originalComposite = pDC.getComposite();
  	    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparence));
  	    pDC.fillPolygon(abscisses, ordonnees, nombrePointsLigne + 2);
  	    pDC.setComposite(originalComposite);    		
    	}
    	else {
    		pDC.setColor(couleur);
    		pDC.fillPolygon(abscisses, ordonnees, nombrePointsLigne + 2);
    	}
    }
    else {
      if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePlein) {
    		pDC.setColor(couleur);
    		pDC.fillPolygon(abscisses, ordonnees, nombrePointsLigne + 2);
      }
      else {
      	TexturePaint tp = StyleRemplissage.getTexturePaint(couleur, styleRemplissageEffectif, listeProprietaire, couleurFond);
      	pDC.setPaint(tp);
      	pDC.fillPolygon(abscisses, ordonnees, nombrePointsLigne + 2);
      }
    }     
  }

  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
  	double dist;
  	CLieuDeBase lieu;

  	if (!existe || (masquage && masque)) return -1;
  	lieu = (CLieuDeBase) bord;
  	dist = lieu.distancePointPourSurface(xp,yp,masquage);
		if ((dist != -1) && (dist < listeProprietaire.distanceMiniPourEtreProche)) return dist;
  	dist = segmentDebut.distancePoint(xp,yp,masquage);
		if ((dist != -1) && (dist < listeProprietaire.distanceMiniPourEtreProche)) return dist;
  	dist = segmentDroite.distancePoint(xp,yp,masquage);
		if ((dist != -1) && (dist < listeProprietaire.distanceMiniPourEtreProche)) return dist;
  	dist = segmentFin.distancePoint(xp,yp,masquage);
		if ((dist != -1) && (dist < listeProprietaire.distanceMiniPourEtreProche)) return dist;
  	return -1;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		super.estDefiniParObjetDans(listeOb)
		&&
		(listeOb.contains(point1) || point1.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(point2) || point2.estDefiniParObjetDans(listeOb));
	}           
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    segmentDebut.point1 = pointDebutLieu;
    segmentDebut.point2 = point1;
    segmentFin.point1 = point2;
    segmentFin.point2 = pointFinLieu;
    segmentDroite.point1 = point1;
    segmentDroite.point2 = point2;    // On cre les tableaux devat recevoir les coordonnes.
    // sur la droite et aux lieux ferms ventuels
     // Modifi pour la version 1.9.5. Seulement deux points de rajouts car le polygone
    // se referme tout seul lors du trac
    // abscisses = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 4];
    // ordonnees = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 4];
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
      abscisses = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 2];
      ordonnees = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 2];
    }  
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    segmentDebut.point1 = pointDebutLieu;
    segmentDebut.point2 = point1;
    segmentFin.point1 = point2;
    segmentFin.point2 = pointFinLieu;
    segmentDroite.point1 = point1;
    segmentDroite.point2 = point2;    
    if (list.getClass() != CPrototype.class) {
      abscisses = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 2];
      ordonnees = new int[((CLieuDeBase)bord).infoLieu.nombreDePoints + 2];
    }  
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(point1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(point2);
    oups.writeInt(ind2);
  }
  
  /**
   * Ajout version 4.8.1
   */
  @Override
  public String tikz(DimensionFenetre dimf, boolean nomaff, double coefmult, boolean bu) {
    CLieuDeBase lieu = (CLieuDeBase) bord;

    // On recopie les coordonnes des points du lieu

    return "\\fill" + tikzFillStyle() + lieu.tikzOnePath(dimf, nomaff, coefmult, bu) + " -- (" +
		Kernel.chaineNombre(listeProprietaire.tikzXcoord(x3, dimf, bu), 3) + "," +
		Kernel.chaineNombre(listeProprietaire.tikzYcoord(y3, dimf, bu), 3) + ")--(" +
		Kernel.chaineNombre(listeProprietaire.tikzXcoord(x2, dimf, bu), 3) + "," +
		Kernel.chaineNombre(listeProprietaire.tikzYcoord(y2, dimf, bu), 3) + ")--cycle;";	
  }

}