/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.*;

import types.*;

import dialogs.MtgDlg;
import dialogs.VariableDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;

public class CVariableBornee extends CValeurDynamique {
  public String nom;
  public double valeurActuelle;
  public double valeurMini;
  public double valeurMaxi;
  public double valeurPas;
	public String chaineValeurMini;
	public String chaineValeurMaxi;
	public String chaineValeurPas;
	public boolean dialogueAssocie;

  /** Creates new CVariableBornee */
  public CVariableBornee() {
  }
  
  // Constructeur pour clonage
  public CVariableBornee(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomInit, double valeurActuelleInit, double valeurMiniInit, double valeurMaxiInit, double valeurPasInit, 
    String chaineValeurMiniInit,String chaineValeurMaxiInit,
    String chaineValeurPasInit, boolean dialogueAssocieInit) {
      super(liste, impProtoInit, estElementFinalInit);
      nom = nomInit;
      valeurActuelle = valeurActuelleInit;
      valeurMini = valeurMiniInit;
      valeurMaxi = valeurMaxiInit;
      valeurPas = valeurPasInit;
      chaineValeurMini = chaineValeurMiniInit;
      chaineValeurMaxi = chaineValeurMaxiInit;
      chaineValeurPas = chaineValeurPasInit;
      dialogueAssocie = dialogueAssocieInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CVariableBornee(listeCible, (CImplementationProto)listeCible.get(ind1, CImplementationProto.class),
      estElementFinal, nom, valeurActuelle, valeurMini, valeurMaxi, valeurPas,
      chaineValeurMini, chaineValeurMaxi, chaineValeurPas, dialogueAssocie);
    return ptelb;
  }

  @Override
  public void setClone(CElementBase ptel) {
    valeurActuelle = ((CVariableBornee) ptel).valeurActuelle;
    valeurMini = ((CVariableBornee) ptel).valeurMini;
    valeurMaxi = ((CVariableBornee) ptel).valeurMaxi;
    valeurPas = ((CVariableBornee) ptel).valeurPas;
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NVariable;
  }

  @Override
  public void donneNom(String  nouveauNom) {
  	nom = nouveauNom;
  }

  // version Java : inutile de redfinir dependDe et dependDePourBoucle

  @Override
  public double rendValeur() {
  	return valeurActuelle;
  }

  public void incremente() {
    double nouv = valeurActuelle + valeurPas;
    if ((nouv >= valeurMini) && (nouv <= valeurMaxi))
      valeurActuelle = nouv;
    else Toolkit.getDefaultToolkit().beep();
  }

  public void decremente() {
    double nouv = valeurActuelle - valeurPas;
    if ((nouv >= valeurMini) && (nouv <= valeurMaxi))
      valeurActuelle = nouv;
    else Toolkit.getDefaultToolkit().beep();
  }

	public void donneValeur(double valeur) {
    valeurActuelle = valeur;
  }
  
	public void donneValeurs (CVariableBornee autreVariable) {
    valeurActuelle = autreVariable.valeurActuelle;
    valeurMini = autreVariable.valeurMini;
    valeurMaxi = autreVariable.valeurMaxi;
    valeurPas = autreVariable.valeurPas;
  }

  @Override
  public String nom() {
    return nom;
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Version 3.5 : on peut modifier une variable mme quand elle est un objet final de construction
  	return true;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new VariableDlg(frame, this, true);
  }
  
   // Inutile de redfinir informationModification() pour les objets de type calcul
  
   // Modifi version 4.9.9
   @Override
   public String infoPourHistorique() {
     return nom() + " : " + getString("chinfo76") + "\n" 
     + getString("chinfo77") + " " + glob.Kernel.chaineNombre(valeurMini, 12) + "\n"
     + getString("chinfo78") + " " + glob.Kernel.chaineNombre(valeurMaxi, 12) + "\n"
     + getString("chinfo79") + " " + glob.Kernel.chaineNombre(valeurPas, 12) + "\n"
     + getString("chinfo80") + " " + glob.Kernel.chaineNombre(valeurActuelle, 12);
   }
  // Ajout version 4.9.9
 	/**
 	 * Fonction renvoyant la chane courte dcrivant l'objet dans certaines listes de botes de dialogues.
 	 * A redfinir pour les descendants.
 	 * @return
 	 */
   @Override
 	public String info() {
     return nom() + " : " + getString("chinfo76");
    }
   
  
   @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    nom = inps.readCString();
 		valeurActuelle = inps.readCDouble();
		valeurMini = inps.readCDouble();
		valeurMaxi = inps.readCDouble();
		valeurPas = inps.readCDouble();
		dialogueAssocie = inps.readBoolean();
		chaineValeurMini = inps.readCString();
		chaineValeurMaxi = inps.readCString();
		chaineValeurPas = inps.readCString();
  }
  
   @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    nom = inps.readUTF();
 		valeurActuelle = inps.readDouble();
		valeurMini = inps.readDouble();
		valeurMaxi = inps.readDouble();
		valeurPas = inps.readDouble();
		dialogueAssocie = inps.readBoolean();
		chaineValeurMini = inps.readUTF();
		chaineValeurMaxi = inps.readUTF();
		chaineValeurPas = inps.readUTF();
  }
  
   @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeUTF(nom);
    oups.writeDouble(valeurActuelle);
    oups.writeDouble(valeurMini);
    oups.writeDouble(valeurMaxi);
    oups.writeDouble(valeurPas);
    oups.writeBoolean(dialogueAssocie);
    oups.writeUTF(chaineValeurMini);
    oups.writeUTF(chaineValeurMaxi);
    oups.writeUTF(chaineValeurPas);
  }
   
   // Version 4.7.1 : Correction d'un epetit bug en ajoutant cette fonction.
   // Si un objet final tait par exemple un affichage LaTeX affichant la valeur de la variable celui-ci ne pouvait pas
   // tre choisi comme objet final mme si la variable tait dans les objets sources.
   @Override
  public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return (listeOb.contains(this));
  }
}
