/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;

import java.awt.Point;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CCercle;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CPointBase;
import objets.CPointInterieurCercle;
import objets.CSurfaceDisque;
import types.Couleur;
import types.NatureObjet;
import types.StyleRemplissage;

public class OutilCreationLiaisonInterieurCercle extends Outil {
	private CPointBase ptBase;
	private CCercle cercle1;
	private CPointInterieurCercle pointInterieurCercle;
	
	public OutilCreationLiaisonInterieurCercle(MtgFrame frameInit) {
		super(frameInit, true);
	}

  @Override
	public void select() {
		super.select();
		ptBase = null;
		cercle1 = null;
		CListeObjets liste = frame.cadreActif().listePr;
		frame.listeClignotante.ajouteObjetsParNature(liste, NatureObjet.NPointBase);		
		frame.outilPointageActif = frame.outilPointageObjetClignotant;
		frame.outilPointageActif.designationDe(NatureObjet.NPointBase);
		frame.indication(getString("creationLiaison1"));
		resetClignotement();		
  }
  
  @Override
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets liste = frame.cadreActif().listePr;      	
    if (ptBase == null) {
      ptBase = (CPointBase) elg;
      annuleClignotement();
      ajouteClignotementDe(ptBase);
      resetClignotement();
      frame.outilPointageActif = frame.outilPointageCre;
      frame.outilPointageActif.designationDe(NatureObjet.NCercle);
      frame.outilPointageActif.reset();
  		frame.indication(getString("creationLiaison2"));
    }
    else {
    	if (cercle1 == null) {
    		// Il faut regarder si l'objet auquel on veut lier le point libre dpend de ce point libre
    		if (elg.appartientABlocDependantPourReclassement(ptBase)) {
    			frame.dialogueAvertissement(getString("ch122"));
    			ptBase = null;
    			annuleClignotement();
    			frame.activeOutilCapture();
    			return;
    		}
        cercle1 = (CCercle) elg;
        ajouteClignotementDe(cercle1);
        // L'utilisateur doit maintenant cliquer  l'intrieur du polygone
        frame.outilPointageActif = frame.outilPointageClic;
        frame.indication(getString("creationLiaisonCercle1"));
        resetClignotement();
        cadre.videDesignation();
        ajouteObjetsVisuels();
    	}
    	else {
        int x = pointClique.x;
        int y = pointClique.y;
        // On regarde si le point cliqu est bien intrieur au polygone
        if (cercle1.pointInterieur(x, y)) {
      		pointInterieurCercle = new CPointInterieurCercle(liste, null, false, ptBase.couleur,
      				ptBase.nomMasque, ptBase.dec_x, ptBase.dec_y, ptBase.masque, ptBase.rendNom(), ptBase.indiceTailleNom,
      				ptBase.motif, true, cercle1, 0, 0, ptBase.marquePourTrace);
      		pointInterieurCercle.placeEn(x, y);
       		pointInterieurCercle.positionne(false, cadre.dimf);
	      	creeObjet();
        }
        else frame.dialogueAvertissement(getString("ch178"));
    	}
    }
  }
  
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
  	CSurfaceDisque surf = new CSurfaceDisque(liste, null, false, Couleur.TraduitCouleur(7), false,
  			StyleRemplissage.QuadrillagePleinTransp, cercle1);
  	frame.ajouteObjetVisuel(surf);
  }  
  
  
  @Override
  public void creeObjet() {
    CListeObjets liste = frame.cadreActif().listePr;      	
		int indicePointRemplace = liste.indexOf(ptBase);
		int indiceObjetCible;
		if (cercle1.estElementFinal) indiceObjetCible = liste.indexOf(cercle1.impProto.dernierObjetFinal());
		else indiceObjetCible = liste.indexOf(cercle1);
		frame.remplaceElement(ptBase, pointInterieurCercle);
		liste.remplacePoint(ptBase, pointInterieurCercle);
		// Si l'objet auquel on doit lier le point a t cr aprs le
		// point, il faut dcaler le point ainsi que tous les objets en 
		// dpendant et ayant t crs entre ce point et l'objet
		// immdiatement aprs cet objet}
		// Attention : decaleDependants remet  jour les index des objets
		if (indicePointRemplace < indiceObjetCible) liste.decaleDependants(indicePointRemplace, indiceObjetCible);
		else pointInterieurCercle.index = indicePointRemplace; // Rajout version 4.9.3
		MtgInternalFrame cadre = frame.cadreActif();
		
    cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil97"));
    cadre.repaintFigureAvecCalcul();
    frame.activeOutilCapture();
    cadre.videListesPourConstructions();  // On remet  jour une construction ventuellement commence  	
    // cadre.listePr.initialiseDependances(); // Ajout version 4.8 Supprim version 4.9.2. Fait par GestionnaireEdition.sauvegardeFigureEnCours()
  }
}
