/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Color;
import java.awt.Point;

import objets.CAffichageLieAPoint;
import objets.CCalcul;
import objets.CCalculAncetre;
import objets.CCommentaire;
import objets.CElementGenerique;
import objets.CElementGraphique;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CPointAncetre;
import objets.CPointAncetrePointsMobiles;
import objets.CPointLieDroite;
import objets.CPrototype;
import objets.CResultatValeur;
import objets.CSegment;
import objets.CValeurAffichee;
import objets.CValeurAssocieeAVariable;
import objets.CValeurDynamique;
import types.MotifPoint;
import types.NatureObjet;
import types.StyleEncadrement;
import types.StyleTrait;
import dialogs.CurseurDlg;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

// Modifi version 5.4
public class OutilCurseur extends Outil {
	CurseurDlg dlg;
	private CPointAncetre point1;
	public static String nomCurseur = "MathGraph32 Cursor"; // Ajout version 4.5

	/** Constructeur */
	public OutilCurseur(MtgFrame frameInit) {
		super(frameInit, true); // True pour gestion clignotement
	}

	@Override
	public void select() {
		super.select();
		point1 = null;
		frame.outilPointageActif = frame.outilPointageCreationCurseur;
		frame.outilPointageActif.designationDe(NatureObjet.NToutPoint);
		frame.indication(getString("curseur1"));
	}

	@Override
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
		if (point1 == null) {
			point1 = (CPointAncetre) elg;
			ajouteClignotementDe(elg);
			CListeObjets liste = frame.cadreActif().listePr;
			// Avant l'affichage du dialogue on cre un message qui sera ajout  la
			// liste des objets clignotants
			// pour matrialiser l'emplacement de l'affichage
			int x = pointClique.x;
			int y = pointClique.y;
			/*
			 * Modifi version 5.0 CCommentaire comClignotant = new
			 * CCommentaire(liste, null, false, frame.couleurActive(), x, y, 7d, -6d,
			 * false, null, 2, StyleEncadrement.Sans, false,
			 * frame.cadreActif().couleurFond, getString("ch165"),
			 * CAffichageLieAPoint.alignementHorizontalGauche,
			 * CAffichageLieAPoint.alignementVerticalHaut);
			 */
			CCommentaire comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 7d, -6d, false,
					null, (int) (MtgFrame.coefMultForSystemFonts * 13), StyleEncadrement.Sans, false,
					frame.cadreActif().couleurFond, getString("ch165"), CAffichageLieAPoint.alignementHorizontalGauche,
					CAffichageLieAPoint.alignementVerticalHaut);
			comClignotant.positionne(false, frame.cadreActif().dimf);
			ajouteClignotementDe(comClignotant);
			resetClignotement();
			frame.cadreActif().videDesignation();
			dlg = new CurseurDlg(frame);
			dlg.setLocationRelativeTo(frame);
			dlg.setVisible(true);
			if (dlg.retourOK())
				creeObjet();
			else {
				frame.detruitDernierElement();
				frame.activeOutilCapture();
			}
		}
	}

	@Override
	public void creeObjet() {
		CValeurDynamique valeurMini; // La valeur qui sera associ au deuxime objet
																	// source
		CValeurDynamique valeurMaxi; // La valeur qui sera associ au troisime
																	// objet source
		CValeurDynamique valeurPas; // La valeur du pas si le curseur demand est
																// discret
		CValeurDynamique valeurPasGrad; // La valeur du pas de graduation si le
																		// curseur demand a une graduation

		MtgInternalFrame cadre = frame.cadreActif();
		cadre.videDesignation();
		CListeObjets liste = cadre.listePr;
		// On regarde d'abord si pour les valeurs mini et maxi l'utilisateur a fait
		// rfrence  une valeur
		// existante ou entr un calcul
		CValeurAssocieeAVariable valMin = dlg.valMin;
		CValeurAssocieeAVariable valMax = dlg.valMax;
		if (valMin.calcul.getClass() != CResultatValeur.class) {
			String nomMin = liste.genereNomPourCalcul("mini", false);
			CCalcul calculMin = new CCalcul(liste, null, false, nomMin, valMin.calcul.chaineCalculSansPar(null),
					valMin.calcul);
			frame.ajouteElement(calculMin);
			valeurMini = calculMin;
		} else {
			valeurMini = ((CResultatValeur) valMin.calcul).valeurAssociee;
		}
		if (valMax.calcul.getClass() != CResultatValeur.class) {
			String nomMax = liste.genereNomPourCalcul("maxi", false);
			CCalcul calculMax = new CCalcul(liste, null, false, nomMax, valMax.calcul.chaineCalculSansPar(null),
					valMax.calcul);
			frame.ajouteElement(calculMax);
			valeurMaxi = calculMax;
		} else {
			valeurMaxi = ((CResultatValeur) valMax.calcul).valeurAssociee;
		}
		// On regarde quel est le prototype  implmenter (ils ont t chargs dans
		// le constructeur du MtgFrame)
		CPrototype proto = null;
		// Version 5.0 : On tient compte des crans haute rsolution
		boolean discret = dlg.checkValeursDiscretes.isSelected();
		boolean gradue = dlg.checkAvecGrad.isSelected();
		String nomProto = "Curseur";
		if (discret) nomProto += "Discret";
		nomProto += dlg.radioHorizontal.isSelected() ? "Hor" : "Ver";
		if (gradue) nomProto += "Gradue";
		if (MtgFrame.screenIsHighResolution) nomProto += "HD";
		proto = frame.chargeConstructionDansDossierMtgFrame(nomProto + ".mgc");
		// On implmente maintenant ce prototype dans la figure
		int i = 0;
		((CElementGenerique) proto.get(i++)).elementAssocie = valeurMini;
		((CElementGenerique) proto.get(i++)).elementAssocie = valeurMaxi;
		if (discret) {
			CValeurAssocieeAVariable valPas = dlg.valPas;
			if (valPas.calcul.getClass() != CResultatValeur.class) {
				String nomPas = liste.genereNomPourCalcul(getString("step"), false);
				CCalcul calculPas = new CCalcul(liste, null, false, nomPas, valPas.calcul.chaineCalculSansPar(null),
						valPas.calcul);
				frame.ajouteElement(calculPas);
				valeurPas = calculPas;
			} else {
				valeurPas = ((CResultatValeur) valPas.calcul).valeurAssociee;
			}
			((CElementGenerique) proto.get(i++)).elementAssocie = valeurPas;
		}
		if (gradue) {
			CValeurAssocieeAVariable valPasGrad = dlg.valPasGrad;
			if (valPasGrad.calcul.getClass() != CResultatValeur.class) {
				String nomPasGrad = liste.genereNomPourCalcul(getString("stepgrid"), false);
				CCalcul calculPasGrad = new CCalcul(liste, null, false, nomPasGrad, valPasGrad.calcul.chaineCalculSansPar(null),
						valPasGrad.calcul);
				frame.ajouteElement(calculPasGrad);
				valeurPasGrad = calculPasGrad;
			} else {
				valeurPasGrad = ((CResultatValeur) valPasGrad.calcul).valeurAssociee;
			}
			((CElementGenerique) proto.get(i++)).elementAssocie = valeurPasGrad;
		}
		((CElementGenerique) proto.get(i)).elementAssocie = point1;
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		impProto.implemente(liste, proto, cadre, 0); // Modifi version 4.8
		// On modifie le nom de l'implmentation (pour internationalisation)
		// Modifi version 4.5 : On garde le mme nom pour l'implmentation de
		// prototype pour pouvoir
		// lors du zoom dtecter les droites  direction fixe qui ne doivent pas
		// tre traites.
		/*
		 * if (dlg.radioHorizontal.isSelected()) { if
		 * (dlg.checkValeursEntieres.isSelected()) impProto.nomProto =
		 * getString("curseurEntierHorizontal"); else impProto.nomProto =
		 * getString("curseurHorizontal"); } else { if
		 * (dlg.radioVertical.isSelected()) { if
		 * (dlg.checkValeursEntieres.isSelected()) impProto.nomProto =
		 * getString("curseurEntierVertical"); else impProto.nomProto =
		 * getString("curseurVertical"); } }
		 */
		// Version 4.5 :
		impProto.nomProto = nomCurseur;
		// On modifie l'avant dernier objet implment qui est un affichage de
		// valeur li  un point
		// suivant les choix de la bote de dialogue
		CValeurAffichee affval = (CValeurAffichee) impProto.dernierFinal(NatureObjet.NValeurAffichee);
		affval.enTete = dlg.editeurPreChaine.getText();
		affval.postChaine = dlg.editeurPostChaine.getText();
		affval.couleurFondAffichage = cadre.couleurFond;
		affval.effacementFond = true;
		// L'avant dernier objet reprsente le calcul du curseur
		CCalculAncetre calc = (CCalculAncetre) liste.get(liste.size() - 2);
		calc.donneNom(dlg.editeurNom.getText());
		// On modifie le style de trait et de couleur des objets finaux
		Color col = frame.couleurActive();
		StyleTrait styleTrait = frame.styleTraitActif();

		if (dlg.checkExtremitesPunaisees.isSelected()) {
			if ((point1.getNature() & NatureObjet.NPointMobile) != 0)
				((CPointAncetrePointsMobiles) point1).pointLibre = false;
		}
		CPointLieDroite pt = (CPointLieDroite) impProto.premierFinal(NatureObjet.NPointLie);
		pt.donneCouleur(col);
		pt.indiceTailleNom = frame.pref_indiceTaillePoliceNom; // Ajout version
																														// 4.9.1.1
		pt.pointLibre = !dlg.checkExtremitesPunaisees.isSelected();
		CSegment seg = (CSegment) impProto.premierFinal(NatureObjet.NSegment);
		seg.donneCouleur(col);
		seg.donneStyle(styleTrait);
		CPointAncetre ptb = (CPointAncetre) impProto.dernierFinal(discret ? NatureObjet.NPointBaseEnt : NatureObjet.NPointLie);
		ptb.donneCouleur(col);
		ptb.donneMotif(MotifPoint.Rond);
		ptb.indiceTailleNom = frame.pref_indiceTaillePoliceNom; // Ajout version
																														// 4.9.1.1
		CValeurAffichee valaff = (CValeurAffichee) impProto.dernierFinal(NatureObjet.NValeurAffichee);
		valaff.donneCouleur(col);

		// Modification version 4.7.2 : Si on a demand un curseur de grande taille,
		// on multiplie l'abscisse du point li par 2
		if (dlg.radioGdeTaille.isSelected())
			pt.donneAbscisse(pt.abscisse * 2);

		if (!dlg.checkValeursDiscretes.isSelected())
			affval.nombreDecimales = dlg.paneDecimales.nombreDecimales();
		cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil123"));
		cadre.repaintFigureAvecCalcul();
		frame.activeOutilCapture();
	}

}