/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import java.util.ListIterator;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import dialogs.MesureAirePolygoneDlg;
import objets.CAffichageLieAPoint;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CMesureAire;
import objets.CNoeudPointeurSurPoint;
import objets.CPointAncetre;
import objets.CPolygone;
import objets.CValeurAffichee;
import types.NatureObjet;
import types.StyleEncadrement;

public class OutilMesureAirePolygone extends Outil {
	CPolygone poly;
	
	public OutilMesureAirePolygone(MtgFrame frameInit) {
		super(frameInit, true);
	}

  @Override
	public void select() {
		super.select();
		poly = null;
		frame.outilPointageActif = frame.outilPointageCre;
		frame.outilPointageActif.designationDe(NatureObjet.NPolygone);
		frame.indication(getString("mesureAirePolygone1"));
    frame.outilPointageObjetClignotant.reset();
	}
	
	public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (poly == null) {
      poly = (CPolygone) elg;
      creeObjet();
    }
	}
	
  @Override
	public void creeObjet() {
		MtgInternalFrame cadre = frame.cadreActif();
		cadre.videDesignation();
		CListeObjets liste = cadre.listePr;
		CMesureAire mesure = new CMesureAire(liste,null, false, poly, "");
		MesureAirePolygoneDlg dlg = new MesureAirePolygoneDlg(frame, mesure);
		dlg.setLocationRelativeTo(frame);
		dlg.setVisible(true);
		if (dlg.retourOK()) {
			if (poly.collectionPoints.size() <= 50) { // Modification version 4.8 sinon trop long pour les constructions rcursives et itratives
				// On nomme les points de la ligne qui n'ont pas de nom
				ListIterator<CNoeudPointeurSurPoint> li = poly.collectionPoints.listIterator();
				while (li.hasNext()) {
					CNoeudPointeurSurPoint noeud = li.next();
					CPointAncetre point = noeud.pointeurSurPoint;
					if (point.rendNom().length() == 0) {
						point.donneNom(liste.genereNomPourPoint(false));
					}
				}
			}
      if (frame.ajouteElementAvecVerification(mesure))
    		if (frame.pref_affichageMesuresNiveau[frame.pref_niveau]) {
     			CValeurAffichee affval = new CValeurAffichee(liste, null, false, frame.couleurActive(),
    					0d, 0d, 0d, -6d, false, poly.collectionPoints.get(0).pointeurSurPoint, frame.pref_IndiceTaillePoliceAffichages,
    					StyleEncadrement.Sans, true, cadre.couleurFond, mesure, mesure.nom() + " = ", "", 1,
    					CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalBas);
     			frame.ajouteElement(affval);
     			cadre.repaintFigureAvecCalcul();
     		}
        cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil144"));

		}
		select();
		
	}
	
  @Override
  public boolean activationValide() {
  	CListeObjets liste = frame.cadreActif().listePr;
    return ((liste.pointeurLongueurUnite != null) && liste.nombreObjetsParNatureVisibles(NatureObjet.NPolygone) != 0);
  }
  
	@Override
	public String messageActivationNonValide() {
  	CListeObjets liste = frame.cadreActif().listePr;
		String ch = "";
		if (liste.pointeurLongueurUnite == null) ch = ch + getString("messagePasDeLongueurUnite") + "\n";
		if (liste.nombreObjetsParNatureVisibles(NatureObjet.NPolygone)==0) ch = ch + getString("messagePasDePolygone");
		return ch;
	}		
}
