/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author Yves
 * Outil de pointage utilis pour la capture de points mobiles
 */
package outils;


import glob.Kernel;

import java.awt.Graphics2D;
import java.awt.geom.*;

import objets.*;
import types.*;
import dialogs.ChoixTypeDlg;
import dialogs.PremierDernierDlg;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

// Modifi version 4.5
public class OutilPointageCapt extends OutilPointage {
  /* pointPredesigne contient un point mobile qui a t choisi lors d'un dialogue de choix
   * de type ou du type premier-dernier
  */
  private CElementGraphique objetPredesigne;
  public boolean deplacementEnCours;
  public CElementGraphique elementCapture;
  public long natureElementCapture; // Nature du point proche du pointeur souris
  // Le champ x de pointabs contient la nouvelle abscisse d'un point
  // li qui doit tre dplac
  public Doub pointabs = new Doub();
  // pointres contient les nouvelles coordonnes d'un point libre qui doit tre dplac
  public Point2D.Double pointres = new Point2D.Double();
  // Anciennes coordonnes du pointeur souris quand on drague la souris
  private int xInitialSouris;
  private int yInitialSouris;
  private double xAffInitial; // Sert pour le dclage d'un affichage li  un point
  private double yAffInitial; // idem
  // Variables servant lors du dcalage d'un afiichage li  un point
	private double xdecalageInitialRelatifAffichage;
	private double ydecalageInitialRelatifAffichage;
	// Ajout version 4.9.9 pour pouvoir dplacer un nom
	private double xnom, ynom;
  private CElementGraphique eltAyantNomCapturable;  
  
  /** Creates a new instance of OutilPointageCapt */
  public OutilPointageCapt(MtgFrame frameInit) {
    super(frameInit);
    // reset();
  }
  @Override
  public void reset() {
    super.reset();
    deplacementEnCours = false;
    objetPredesigne = null;
    elementCapture = null;
  }
  
  // Ajout pout version 2.0
  @Override
  public void actionDansPaint(MtgInternalFrame cadre) {
		cadre.effacegbuffer();
		cadre.afficheToutDansgbuffer();
		aRedessinerDansPaint = false;
  }
  
  @Override
  public void mouseClicked(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
  }
  @Override
  public void mouseMoved(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
  	xnom = mouseEvent.getX();
  	ynom = mouseEvent.getY();
    InfoProx info = frame.infoProx;
    CListeObjets liste = listeUtilisee();
    int nbObjetsProches = liste.procheDePourCapture(aDesigner, mouseEvent.getPoint(), info,
      frame.listeExclusionDesignation, true);
    // boolean designationDejaAffichee = (cadre.commentaireDesignation().chaineCommentaire.length() != 0);
    // true si une dsignation est dj affiche prs du pointeur
    if ((nbObjetsProches > 0) && !cadre.ctlKeyPressed) {
      setActiveCursor(MtgFrame.handCursor);
      // cadre.commentaireDesignation().placeEn(0, 0, -6, -6); // Supprim version 4.5.2
      aRedessinerDansPaint = true;
      cadre.paneFigure.repaint();
    }
    else {
      // Si aucun point ou aucune droite n'est proche du pointeur souris
      // on regarde si un nom de point ou de droite est proche de celui-ci
      eltAyantNomCapturable = liste.premierObjetNomProcheDe((Graphics2D) cadre.getGraphics(),
              NatureObjet.NObjetNommable, mouseEvent.getPoint());
      // Si un nom est proche on transforme le curseur en curseur main
      if (eltAyantNomCapturable != null) setActiveCursor(MtgFrame.handCursor);
      else setActiveCursor(MtgFrame.defaultCursor);
      cadre.videDesignation();
 
      // setActiveCursor(MtgFrame.crossHairCursor);
      // frame.cadreActif().videDesignation();
      // aRedessinerDansPaint = true;
      // cadre.paneFigure.repaint();
    }
  }
  @Override
  public void mousePressed(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
    if (elementCapture != null) return;
    // Ajout version 4.9.9
    if (cadre.nameEditor.isVisible()) cadre.nameEditor.cache();
    //
    xInitialSouris = mouseEvent.getX();
    yInitialSouris = mouseEvent.getY();
    CListeObjets liste = cadre.listePr;
    InfoProx info = frame.infoProx;
    if (info.nombreObjetsProches > 0) {
      if (objetPredesigne != null) {
        if (objetPredesigne.distancePoint(xInitialSouris, yInitialSouris, true) <= 4) {
          elementCapture = objetPredesigne;
          objetPredesigne = null;	        
        }
        else return; // Ligne rajoute version 5.4
      }
      else {   	
	      if (info.premierPointVoisin != null) { // A revoir si une macro est en cours d'excution
	        // Priorit aux points mobiles
	        if (info.nombrePointsVoisins == 1) {
	          elementCapture = (CPointAncetrePointsMobiles) info.premierPointVoisin;
	        }
          else {
          	if (info.nombreTypesPointsMobilesProches() == 1) {
	            PremierDernierDlg dlg = new PremierDernierDlg(frame);
	        		dlg.setLocationRelativeTo(frame);
	            dlg.setVisible(true);
	            if (!dlg.retourOK()) return;
	            // On regarde quelle a t la rponse de l'utilisateur
	            if (dlg.getPremierChoisi()) objetPredesigne = info.premierPointVoisin;
	            else objetPredesigne = info.dernierPointVoisin;
	            return;
          	}
          	else {
          		// On restreint le choix de type aux types de point si on est proche d'un point
  	          ChoixTypeDlg dlgct = new ChoixTypeDlg(frame, info.typesProches & NatureObjet.NPointMobile);
  	      		dlgct.setLocationRelativeTo(frame);
  	          dlgct.setVisible(true);
  	          if (!dlgct.retourOK()) return;    
  	          int indType = dlgct.indiceTypeChoisi;
  	          // Il faut regarder si dans le type choisi un seul objet est proche ou plusieurs
  	          if (info.infoParType[indType].nombreVoisins == 1) {
  	            objetPredesigne = info.infoParType[indType].premierVoisin;
  	            return;
  	          }
  	          else { // Plusieurs objets du type choisi sont proches du curseur
  	            PremierDernierDlg dlg = new PremierDernierDlg(frame);
  	        		dlg.setLocationRelativeTo(frame);
  	            dlg.setVisible(true);
  	            if (!dlg.getRetourOK()) return;
  	            // On regarde quelle a t la rponse de l'utilisateur
  	            if (dlg.getPremierChoisi()) objetPredesigne = info.infoParType[indType].premierVoisin;
  	            else objetPredesigne = info.infoParType[indType].dernierVoisin;
  	            return;
  	          }
          	}
          }
	      }
	      else {
	        if (info.nombreTypesProches == 1) {
	          int i;
	          for (i =0; i < NatureObjet.nombreTypesDesignables; i++) {
	            if (info.infoParType[i].nombreVoisins > 0) break;
	          }
	          // i contient l'indice du type d'objets proches
	          if (info.infoParType[i].nombreVoisins >= 2) {
	            // On ouvre un dialogue pour demander de choisir entre le premier et le dernier objet
	            PremierDernierDlg dlg = new PremierDernierDlg(frame);
	        		dlg.setLocationRelativeTo(frame);
	            dlg.setVisible(true);
	            if (!dlg.retourOK()) return;
	            // On regarde quelle a t la rponse de l'utilisateur
	            if (dlg.getPremierChoisi()) objetPredesigne = info.infoParType[i].premierVoisin;
	            else objetPredesigne = info.infoParType[i].dernierVoisin;
	            return;
	          }
	          else {
	            elementCapture = info.infoParType[i].premierVoisin;
	          }
	        }
	        else { // Plusieurs objets de diffrents types sont proches
	          ChoixTypeDlg dlgct = new ChoixTypeDlg(frame, info.typesProches);
	      		dlgct.setLocationRelativeTo(frame);
	          dlgct.setVisible(true);
	          if (!dlgct.retourOK()) return;    
	          int indType = dlgct.indiceTypeChoisi;
	          // Il faut regarder si dans le type choisi un seul objet est proche ou plusieurs
	          if (info.infoParType[indType].nombreVoisins == 1) {
	            objetPredesigne = info.infoParType[indType].premierVoisin;
	            return;
	          }
	          else { // Plusieurs objets du type choisi sont proches du curseur
	            PremierDernierDlg dlg = new PremierDernierDlg(frame);
	        		dlg.setLocationRelativeTo(frame);
	            dlg.setVisible(true);
	            if (!dlg.getRetourOK()) return;
	            // On regarde quelle a t la rponse de l'utilisateur
	            if (dlg.getPremierChoisi()) objetPredesigne = info.infoParType[indType].premierVoisin;
	            else objetPredesigne = info.infoParType[indType].dernierVoisin;
	            return;
	          }
	        }  	
	      }
      }
      natureElementCapture = elementCapture.getNature();
      if (elementCapture.estDeNature(NatureObjet.NAffichageLieAPoint)) {
				// Les deux lignes suivantes rajoute version 2.2.3 pour que quand on capture un affichage un ventuel lieu d'objets
				// de cet affichage soit aussi recalcul
        cadre.listeDependantsDansPrincipale.retireTout();
        cadre.listeDependantsDansPrincipale.setListeDependantePourCapture(liste, elementCapture);
      	CAffichageLieAPoint aff = (CAffichageLieAPoint) elementCapture;
      	xdecalageInitialRelatifAffichage = aff.dec_x;
      	ydecalageInitialRelatifAffichage = aff.dec_y;
      	xAffInitial = aff.x_nom;
      	yAffInitial = aff.y_nom;
      }
      else {
      	if (elementCapture.estDeNature(NatureObjet.NLieuObjet)) {
      		// Si ce lieu d'objet est capturable c'est qu'il est gnr par un CCommentaire ou un CLatex
      		CLieuObjetAncetre lieu = (CLieuObjetAncetre) elementCapture;
      		CAffichageLieAPoint aff = (CAffichageLieAPoint) lieu.elementAssocie;
        	xdecalageInitialRelatifAffichage = aff.dec_x;
        	ydecalageInitialRelatifAffichage = aff.dec_y;
        	xAffInitial = aff.x_nom;
        	yAffInitial = aff.y_nom;      		
      	}
      	else {
	        if (elementCapture.estDeNature(NatureObjet.NPointMobile)) {
	  	        cadre.listeDependantsDansPrincipale.retireTout();
	  	        cadre.listeDependantsDansPrincipale.setListeDependantePourCapture(liste, elementCapture);
	        }
      	}
      }
    }
  }
  
  @Override
  public void mouseDragged(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
    int x = mouseEvent.getX();
    int y = mouseEvent.getY();
    //  revoir
    if ((x > cadre.dimf.longueurAbscisses) || (y > cadre.dimf.longueurOrdonnees)) {
      return;
    }
    if (elementCapture != null){
      if (natureElementCapture == NatureObjet.NPointLie) {
        CPointLie ptPointLie = (CPointLie)(elementCapture);
        deplacementEnCours = 
          ptPointLie.testDeplacement(cadre.dimf,x,y,pointres,pointabs);
        if (deplacementEnCours) {
	        ptPointLie.donneAbscisse(pointabs.getValue());
	        cadre.listeDependantsDansPrincipale.positionne(cadre.infRand, cadre.dimf);
	        aRedessinerDansPaint = true;
	        cadre.paneFigure.repaint();
        }
      }
      else {
        if (natureElementCapture == NatureObjet.NPointBase) {
          CPointBase ptbase = (CPointBase) elementCapture;
          deplacementEnCours = ptbase.testDeplacement(cadre.dimf,x,y,pointres,pointabs);
          if (deplacementEnCours) {
  	        ptbase.placeEn(pointres.x, pointres.y);
  	        cadre.listeDependantsDansPrincipale.positionne(cadre.infRand, cadre.dimf);
  	        aRedessinerDansPaint = true;
  	        cadre.paneFigure.repaint();
          }
        }
        else {
        	if (natureElementCapture == NatureObjet.NPointBaseEnt) {
	          CPointBaseEnt ptbaseEnt = (CPointBaseEnt) elementCapture;
	          deplacementEnCours = ptbaseEnt.testDeplacement(cadre.dimf,x,y,pointres,pointabs);
	          if (deplacementEnCours){
	  	        ptbaseEnt.placeEn(pointres.x, pointres.y);
	  	        cadre.listeDependantsDansPrincipale.positionne(cadre.infRand, cadre.dimf);
	  	        aRedessinerDansPaint = true;
	  	        cadre.paneFigure.repaint();
	          }      
	        }
        	else {
        		if ((natureElementCapture & NatureObjet.NAffichageLieAPoint) != 0) {
        			deplacementEnCours = true;
        			CAffichageLieAPoint aff = (CAffichageLieAPoint) elementCapture;
    					CPointAncetre point = aff.pointLie;
    					if (point == null) aff.placeNom(xAffInitial + x - xInitialSouris, yAffInitial + y - yInitialSouris);
    					else aff.decaleNom(xdecalageInitialRelatifAffichage + x - xInitialSouris,
    										ydecalageInitialRelatifAffichage + y - yInitialSouris);
    					aff.positionne(cadre.infRand, cadre.dimf);
    	        cadre.listeDependantsDansPrincipale.positionne(cadre.infRand, cadre.dimf);
    					aRedessinerDansPaint = true;
    					cadre.paneFigure.repaint();
        		}
        		else {
        			if (natureElementCapture == NatureObjet.NLieuObjet) {
        				// Si on a pu caapturer le lieu d'objet c'est qu'il est gnr par un CCommentaire ou CLatex (graduation)
        				CLieuObjetAncetre lieu = (CLieuObjetAncetre) elementCapture;
        				CAffichageLieAPoint aff = (CAffichageLieAPoint) lieu.elementAssocie;
        				// On dcale l'affichage gnrant le lieu d'objet (CCOmmentaire ou CLatex)
      					CPointAncetre point = aff.pointLie;
      					if (point == null) aff.placeNom(xAffInitial + x - xInitialSouris, yAffInitial + y - yInitialSouris);
      					else aff.decaleNom(xdecalageInitialRelatifAffichage + x - xInitialSouris,
      										ydecalageInitialRelatifAffichage + y - yInitialSouris);
      					aff.positionne(cadre.infRand, cadre.dimf);
      					lieu.positionne(cadre.infRand, cadre.dimf);
      	        cadre.listeDependantsDansPrincipale.positionne(cadre.infRand, cadre.dimf);
      					aRedessinerDansPaint = true;
      					cadre.paneFigure.repaint();       				
        			}
        			else {
			    			if (natureElementCapture == NatureObjet.NMarqueAngle) {
			       			deplacementEnCours = true;        				
			    				CMarqueAngleAncetre marque = (CMarqueAngleAncetre) elementCapture;
			    				// Modification version 3.2.6 pour qu'une marque d'angle ne disparaisse pas
			    				double ray = glob.Kernel.distancePointPoint(x, y, marque.centre_x, marque.centre_y);
			    				if (ray > 3) marque.setRayon(ray);
			    				marque.positionne(cadre.infRand, cadre.dimf);
			    				aRedessinerDansPaint = true;
			    				cadre.paneFigure.repaint();
			    			}
			    			else {
			    				if (natureElementCapture == NatureObjet.NPointInterieurPolygone) {
			      				CPointInterieurPolygone pointint = (CPointInterieurPolygone) elementCapture;
			  	          deplacementEnCours = pointint.testDeplacement(cadre.dimf,x,y,pointres,pointabs);
			  	          if (deplacementEnCours){
			      					pointint.placeEn(pointres.x, pointres.y);
			      					pointint.positionne(cadre.infRand, cadre.dimf);
			  	  	        cadre.listeDependantsDansPrincipale.positionne(cadre.infRand, cadre.dimf);
			         				aRedessinerDansPaint = true;
			         				cadre.paneFigure.repaint();
			  	          }
				  	      }
			    				else {
			      				if (natureElementCapture == NatureObjet.NPointInterieurCercle) {
			        				CPointInterieurCercle pointint = (CPointInterieurCercle) elementCapture;
			    	          deplacementEnCours = pointint.testDeplacement(cadre.dimf,x,y,pointres,pointabs);
			    	          if (deplacementEnCours){
			        					pointint.placeEn(pointres.x, pointres.y);
			        					pointint.positionne(cadre.infRand, cadre.dimf);
			    	  	        cadre.listeDependantsDansPrincipale.positionne(cadre.infRand, cadre.dimf);
			           				aRedessinerDansPaint = true;
			           				cadre.paneFigure.repaint();
			    	          }
			  	  	      }
			    				}
			    			}
        			}
        		}
        	}
        }
      }
    }
    else {
    	int xn,yn;
    	double decxn,decyn;
    	xn = x;
    	yn = y;
      if (eltAyantNomCapturable != null) {
        if (eltAyantNomCapturable.estDeNature(NatureObjet.NToutPoint)) {
          decxn = xn - xnom + eltAyantNomCapturable.dec_x;
          decyn = yn - ynom + eltAyantNomCapturable.dec_y;
          if (decyn < -180) decyn = -180;
          else {
            if (decyn > 90) decyn = 90;
          }
          if (decxn < -180) decxn = -180;
          else {
            if (decxn > 90) decxn = 90;
          }
          eltAyantNomCapturable.decaleNom(decxn, decyn);
        }
        else {
          double nouvelleAbs;
          if (eltAyantNomCapturable.estDeNature(NatureObjet.NDroite)) {
            CDroite dte = (CDroite) eltAyantNomCapturable;
            Point2D.Double proj = new Point2D.Double();
            double x1 = eltAyantNomCapturable.x_nom + eltAyantNomCapturable.dec_x + xn - xnom;
            double y1 = eltAyantNomCapturable.y_nom + eltAyantNomCapturable.dec_y + yn - ynom;
            Kernel.projetteOrtho(xn, yn, dte.point_x, dte.point_y, dte.vecteur, proj);
            if (dte.xext1 != dte.xext2) nouvelleAbs = (proj.x-dte.xext1)/(dte.xext2-dte.xext1);
              else nouvelleAbs = (proj.y-dte.yext1)/(dte.yext2-dte.yext1);
            decxn = x1 - proj.x;
            decyn = y1 - proj.y;
            if (decyn < -90) decyn = -90;
            else {
              if (decyn > 90) decyn = 90;
            }
            if (decxn < -90) decxn = -90;
            else {
              if (decxn > 90) decxn = 90;
            }
            dte.decaleNom(decxn,decyn);
            dte.abscisseNom = nouvelleAbs;
          }      
        }
        xnom = mouseEvent.getX();
        ynom = mouseEvent.getY();
        eltAyantNomCapturable.positionne(cadre.infRand, cadre.dimf);
        cadre.repaintFigureSansCalcul();
      }
    	
    }
  }
  
  @Override
  public void mouseReleased(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
    // On sauvegarde l'tat de la figure avant capture
    if (elementCapture != null) frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil93"));      
    InfoProx info = frame.infoProx;
    int nbObjetsProches = listeUtilisee().procheDePourCapture(NatureObjet.NPointMobile | NatureObjet.NMacro, mouseEvent.getPoint(), info,
      frame.listeExclusionDesignation, true); // Priorit aux points pour la capture
    if (nbObjetsProches > 0) {
      if (info.premierPointVoisin != null) setActiveCursor(MtgFrame.handCursor);
      else setActiveCursor(MtgFrame.defaultCursor);
    }
    else setActiveCursor(MtgFrame.crossHairCursor);
    elementCapture = null;
    deplacementEnCours = false;
    cadre.listeDependantsDansPrincipale.retireTout();
    // On dcharge le garbage Collector
    // System.gc(); Supprim version 5.0.0.5
  }
  
  /**
   * liste  utilise pour savoir si on est proche d'une objet
   * Sera redfini par OutilPointageTraceObjet
   * @return
   */
  public CListeObjets listeUtilisee() {
  	return frame.cadreActif().listePr;
  }  
  
}
