/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;

import java.awt.Point;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CMarqueAngleOriente;
import objets.CPointAncetre;
import objets.CPointImage;
import objets.CSegment;
import objets.CSimilitude;
import objets.CTransformation;
import objets.CValeurAngle;
import objets.CValeurAssocieeAVariable;
import types.Couleur;
import types.Int;
import types.NatureObjet;
import types.StyleFleche;
import types.StyleMarqueAngle;
import types.StyleTrait;

import dialogs.AngleEtRapportSimilitudeDlg;

import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;

public class OutilSimilitude extends Outil {
  private CPointAncetre point;
  private CElementGraphique objet;
  private int nombreImagesCrees;
  private CSimilitude similitude;
  
  /** Creates a new instance of OutilRot */
  public OutilSimilitude(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  @Override
  public void select() {
    super.select();
    point = null;
    objet = null;
    similitude = null;
    nombreImagesCrees = 0;
    frame.outilPointageCre.designationDe(NatureObjet.NToutPoint);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.indication(getString("similitude1"));
  }

  /**
   * Il est ncessaire de regarder quand l'outil est dselectionn si on a vraiment cr des images
   * par une transformation. Sinon il faut supprimer la transformation
   */
  @Override
  public void deselect() {
    CListeObjets liste = frame.cadreActif().listePr;
    // Si on a utilis une similitude nouvelle et rien n'a t cr c'est le dernier lment
    if (similitude == liste.get(liste.size() - 1)){
    	frame.detruitDernierElement();
    }
    super.deselect();
  }  

  @Override
  public void creeObjet() {
    CListeObjets liste = frame.cadreActif().listePr;
    if (nombreImagesCrees == 0) {
    	CTransformation sim = liste.transformationConfondueAvec(similitude);
    	if (sim != null) {
    		similitude = (CSimilitude) sim;
     	}
    	else frame.ajouteElement(similitude);
    }
    Int nbImagesCrees = new Int();
    CElementGraphique elg = imageParTransformation(liste,  objet, similitude, nbImagesCrees);
    if (frame.ajouteElementsImageAvecVerification(elg, nbImagesCrees.getValue(), similitude)) {
    	frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil64"));
    	nombreImagesCrees++;
  		frame.cadreActif().repaintFigureAvecCalcul();    	
    }
  }

  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    CListeObjets liste = frame.cadreActif().listePr;
    if (point == null) {
      frame.indication(getString("similitude2"));
      point = (CPointAncetre) elg;
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      resetClignotement();
      CValeurAngle ang = new CValeurAngle(liste, 0, "0");
      CValeurAssocieeAVariable rap = new CValeurAssocieeAVariable(liste, 0);
      similitude = new CSimilitude(liste, null, false, point, ang, rap);
      // On cre un dialogue modal permettant d'entrer l'angle de la rotation
      AngleEtRapportSimilitudeDlg angleEtRapportDlg = new AngleEtRapportSimilitudeDlg(frame, similitude, false);
      // On redessine la figure pour liminer la dsignation ce point affiche pour le centre
      frame.cadreActif().videDesignation();
      frame.cadreActif().paneFigure.repaint();
  		angleEtRapportDlg.setLocationRelativeTo(frame);
      angleEtRapportDlg.setVisible(true);
      // On regarde si la bote de dialogue a bien t valide par OK
      if (angleEtRapportDlg.retourOK()) {
        ajouteObjetsVisuels();
        frame.outilPointageActif.designationDe(NatureObjet.NToutPoint | NatureObjet.NTouteDroiteSaufVecteur |
                NatureObjet.NCercle | NatureObjet.NArc | NatureObjet.NPolygone
                | NatureObjet.NLigneBrisee);
        resetClignotement();
        frame.indication(getString("similitude3"));

      }
      else {
        annuleClignotement();
        select(); 
      }      
    }
    else {
      objet = elg;
      creeObjet();
    }
  }
  
  /** Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
  	// On cre un segment en pointills joignant le centre au point souris
  	CSegment pts1 = new CSegment(liste, null, false, point, pane.pointSouris);
    frame.ajouteObjetVisuel(pts1);
    frame.ajouteObjetVisuel(similitude);
    CPointImage ptim = new CPointImage(liste, null, false, frame.couleurActive(),
    		frame.stylePointActif(), 13, pane.pointSouris, similitude);
    frame.ajouteObjetVisuel(ptim);
    // On cre un deuxime segment joigant le centre au point image
  	CSegment pts2 = new CSegment(liste, null, false, point, ptim);
    frame.ajouteObjetVisuel(pts2);
    // On cre une marque d'angle oriaent pour visualiser la rotation
    CMarqueAngleOriente ptmar = new CMarqueAngleOriente(liste, null, false,
      Couleur.TraduitCouleur(1), false, StyleTrait.traitFin, StyleMarqueAngle.marqueSimple,
      MtgFrame. screenIsHighResolution ? 36 : 18, StyleFleche.FlecheCourtePleine, pane.pointSouris, point, ptim);
    frame.ajouteObjetVisuel(ptmar);
  }  
}