/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import objets.CElementBase;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CPointAncetrePointsMobiles;
import types.NatureObjet;
import mathgraph32.MtgFrame;

public class OutilTraceObjets extends Outil {

	public CListeObjets ListeATracer; // Contiendra la liste des objets devant laisser une trace
	
  /** Creates a new instance of OutilCapture */
  public OutilTraceObjets(MtgFrame frameInit) {
    super(frameInit, true); // true car l'outil utilise les clignotements
    ListeATracer = new CListeObjets();
  }

  @Override
  public void select() {
    super.select();
    ListeATracer.retireTout();
    // frame.outilPointageCre.designationDe(NatureObjet.NToutObjet - NatureObjet.NMacro - NatureObjet.NPointBase);
    // modification version 3.2.5.4
    frame.outilPointageCre.designationDe(NatureObjet.NToutObjet - NatureObjet.NMacro);    
    frame.indication(getString("traceObjet1"));
    frame.outilPointageActif = frame.outilPointageCre;
    frame.outilPointageActif.reset();    
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	frame.listeClignotante.add(elg);
  	frame.listeExclusionDesignation.add(elg);
  	ListeATracer.add(elg);
  }
  
  public void etablitPointsMobilesAFaireClignoter() {
  	annuleClignotement();
  	frame.listeExclusionDesignation.retireTout(); // Ajout version 3.2.6
  	CListeObjets liste = frame.cadreActif().listePr;
  	for(int i = 0; i < ListeATracer.size(); i++) {
  		CElementBase elb1 = ListeATracer.get(i);
  		for (int j = 0; (j < liste.size()); j++) {
  			CElementBase elb2 = liste.get(j);
  			if (elb2.estDeNature(NatureObjet.NPointBase |NatureObjet.NPointLie)
  					&& elb1.dependDe(elb2) && !frame.listeClignotante.contains(elb2)) {
  				CPointAncetrePointsMobiles pt = (CPointAncetrePointsMobiles) elb2;
  				if ((!pt.masque) && pt.pointLibre) frame.listeClignotante.add(elb2); // Il faut que le point soit non punais et visible
  			}
  		}
  	}
  }
  
  @Override
  public void creeObjetApresClicDroit() {
  	etablitPointsMobilesAFaireClignoter();
  	frame.cadreActif().initialisetbuffer(); // Initialise le buffer de traces avec l'image de fond ventuelle
  	// Si les objets  dsigner ne dpendent d'aucun point mobile, message d'erreur et sortie
  	if (frame.listeClignotante.size() == 0) {
  		frame.dialogueAvertissement(getString("ch161"));
  		frame.activeOutilCapture();
  	}
  	else {
  		resetClignotement();
      frame.outilPointageTraceObjet.designationDe(NatureObjet.NPointBase | NatureObjet.NPointBaseEnt |NatureObjet.NPointLie);
      frame.indication(getString("traceObjet2"));
      frame.outilPointageActif = frame.outilPointageTraceObjet;
      frame.outilPointageActif.reset();    
  	}
  }
}
