/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;

import dialogs.CoordonneesVecteurTranslationDlg;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CPointImage;
import objets.CTransformation;
import objets.CTranslationParCoord;
import objets.CValeurAssocieeAVariable;
import objets.CVecteur;
import types.Couleur;
import types.Int;
import types.MotifPoint;
import types.NatureCalcul;
import types.NatureObjet;
import types.StyleFleche;
import types.StyleTrait;

public class OutilTranslationParCoord extends Outil {
  private CElementGraphique objet;
  private int nombreImagesCrees;
  private CTranslationParCoord trans;
  
  /** Creates a new instance of OutilRot */
  public OutilTranslationParCoord(MtgFrame frameInit) {
    super(frameInit, true);
  }
  
  @Override
  public void select() {
    super.select();
    frame.outilPointageActif = frame.outilPointageCre;
    frame.outilPointageActif.reset();
    objet = null;
    trans = null;
    nombreImagesCrees = 0;
    // Cet outil commence par le dialogue de choix des coordonnes du vecteur (et du repre)
    CListeObjets liste = frame.cadreActif().listePr;
    CValeurAssocieeAVariable xtrans = new CValeurAssocieeAVariable(liste, 0);
    CValeurAssocieeAVariable ytrans = new CValeurAssocieeAVariable(liste, 0);
    trans = new CTranslationParCoord(liste, null, false, null, xtrans, ytrans);
    CoordonneesVecteurTranslationDlg dlg = new CoordonneesVecteurTranslationDlg(frame, trans, false);
		dlg.setLocationRelativeTo(frame);
    dlg.setVisible(true);
    if (dlg.retourOK()) {
      frame.indication(getString("translation1"));  	
      frame.outilPointageActif.designationDe(NatureObjet.NToutPoint | NatureObjet.NTouteDroiteSaufVecteur |
              NatureObjet.NCercle | NatureObjet.NArc | NatureObjet.NPolygone
              | NatureObjet.NLigneBrisee);
      resetClignotement();
      ajouteObjetsVisuels();
    }
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    objet = elg;
    creeObjet();
  }
  
  @Override
  public void creeObjet() {
    CListeObjets liste = frame.cadreActif().listePr;
    if (nombreImagesCrees == 0) {
    	CTransformation tr = liste.transformationConfondueAvec(trans);
    	if (tr != null) {
    		trans = (CTranslationParCoord) tr;
     	}
    	else frame.ajouteElement(trans);
    }
    Int nbImagesCrees = new Int();
    CElementGraphique elg = imageParTransformation(liste,  objet, trans, nbImagesCrees);
    if (frame.ajouteElementsImageAvecVerification(elg, nbImagesCrees.getValue(), trans)) {
    	frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil19"));
    	nombreImagesCrees++;
  		frame.cadreActif().repaintFigureAvecCalcul();    	
    }
  }
  
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;  	
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
    CTranslationParCoord transx = new CTranslationParCoord(liste, null, false, trans.rep, trans.x, new CValeurAssocieeAVariable(liste, 0));
    CPointImage ptimx = new CPointImage(liste, null, false, frame.couleurActive(),
        MotifPoint.PetitRond, 13, pane.pointSouris, transx);
    ptimx.cache();
    frame.ajouteObjetVisuel(transx);
    frame.ajouteObjetVisuel(ptimx);
    CVecteur vecx = new CVecteur(frame.listeObjetsVisuels, null, false,
        Couleur.TraduitCouleur(2), false, StyleTrait.traitPointille,
        pane.pointSouris, ptimx, StyleFleche.FlecheCourtePleine);
    frame.ajouteObjetVisuel(vecx);
    CTranslationParCoord transy = new CTranslationParCoord(liste, null, false, trans.rep, new CValeurAssocieeAVariable(liste, 0), trans.y);
    frame.ajouteObjetVisuel(transy);
    CPointImage ptimy = new CPointImage(liste, null, false, frame.couleurActive(),
      MotifPoint.PetitRond, 13, ptimx, transy);
    ptimy.cache();
    frame.ajouteObjetVisuel(ptimy);
    frame.ajouteObjetVisuel(trans);
    CPointImage ptim = new CPointImage(liste, null, false, frame.couleurActive(),
       MotifPoint.PetitRond, 13, pane.pointSouris, trans);
    frame.ajouteObjetVisuel(ptim);
    CVecteur vecy = new CVecteur(frame.listeObjetsVisuels, null, false,
        Couleur.TraduitCouleur(3), false, StyleTrait.traitPointille,
        ptimx, ptim, StyleFleche.FlecheCourtePleine);
    frame.ajouteObjetVisuel(vecy);
    CVecteur vec = new CVecteur(frame.listeObjetsVisuels, null, false,
      Couleur.TraduitCouleur(1), false, StyleTrait.traitPointille,
      pane.pointSouris, ptim, StyleFleche.FlecheCourtePleine);
    frame.ajouteObjetVisuel(vec);
  }
  
  @Override
	public boolean activationValide() {
		return frame.cadreActif().listePr.nombreObjetsParNatureCalculNonIntermediaires(NatureCalcul.NRepere)>0;
	}
  
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDeRepere");
	}	  
  
}