/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package types;
/*
 * FonteNoms.java
 *
 * Created on 17 juin 2004, 16:59
 */

/**
 *
 * @author  Yves
 * @version 
 */
import java.awt.*;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import org.scilab.forge.jlatexmath.TeXConstants;
import org.scilab.forge.jlatexmath.TeXFormula;
import org.scilab.forge.jlatexmath.TeXIcon;

import objets.CLatex;

import java.awt.font.TextAttribute;
// Classe qui par ses mthodes de classe perlet de grer es fontes de caractres//
public class FonteNom {
  static final public int Normal   = 0;
  static final public int Gras   = 1;
  static final public int Italique   = 2;
  static final public int Underline   = 4; // Ajout version 5.0 pour le code #U
  // Ajout version 4.8.1 pour les caractres grecs utiliss
  // Modifi version 4.9.1.2 Pour caractre \forall oubli et \times replac par \forall
	public static final char[] grecHexa = {0x3b1,0x3b2,0x3b3,0x3b4,0x3b5,0x3b6,0x3b7,0x3b8,0x3bA,
	0x3bb,0x3bc,0x3bd,0x3be,0x3c0,0x3c1,0x3c3,0x3c4,0x3c6,0x3c7,
	0x3c8,0x3c9,0x393,0x394,0x39e,0x3A0,0x3A3,0x3A6,0x3A7,0x3A8,0x3A9};
	public static final String[] grecLatex = {"\\alpha ","\\beta ","\\gamma ","\\delta ","\\varepsilon ","\\zeta ","\\eta ","\\theta ","\\kappa ",
	"\\lambda ","\\mu ","\\nu ","\\xi ","\\pi ","\\rho ","\\sigma ","\\tau ","\\varphi ","\\chi ",
	"\\psi ","\\omega ","\\Gamma ","\\Delta ","\\Xi ","\\Pi ","\\Sigma ","\\Phi ","\\Chi ","\\Psi ","\\Omega "};
	public static final char[] mathHexa = {0x2264,0x2265,0x00d7,0x2200,0x2203,0x2205,0x2207,0x2209,0x220A,0x220D,0x220F,0x2211,0x221e,0x2227,0x2228,0x2229,
		0x222A,0x222b,0x223c,0x2245,0x2248,0x223C,0x2243,0x2260,0x2261,0x2282,0x2284,0x2283,0x2113,0x22A5,0x2295,0x2297,0x227A,0x227B,0x2202,0x2201};
	public static final String[] mathLatex = {"\\le ","\\ge ","\\times ","\\forall ","\\exists ","\\emptyset ","\\nabla ","\\notin ","\\in ","\\ni ","\\prod ","\\sum ","\\infty ",
		"\\wedge ","\\vee ","\\cap ","\\cup ","\\int ","\\sim ","\\cong ","\\approx ","\\sim","\\simeq","\\ne ","\\equiv ","\\subset ","\\not\\subset ",
		"\\supset ","\\ell ","\\perp ","\\oplus ","\\otimes ","\\prec ","\\succ ","\\partial ","\\complement "};
	public static final char[] flechesHexa = {0x2191,0x2190,0x2192,0x2193,0x2194,0x2195,0x21d4,
		0x21d2,0x21d0,0x21ce,0x21a6,0x21cf,0x21cd,0x21a9,0x21aa,0x21ba,0x21bb};
	public static final String[] flechesLatex = 	{"\\uparrow ","\\leftarrow ","\\rightarrow ","\\downarrow ","\\leftrightarrow ","\\updownarrow ","\\Leftrightarrow ",
		"\\Rightarrow ","\\Leftarrow ","\\nLeftrightarrow ","\\mapsto ","\\nRightarrow ","\\nLeftarrow ","\\hookleftarrow ",
		"\\hookrightarrow ","\\circlearrowleft ","\\circlearrowright "};


  /** Creates new FonteNoms */
    public FonteNom() {
    }
   // Mthode renvoyant la taille de caractres correspondant  l'indice de taille Indicede Taille//
    public static final int taille(int indiceDeTaille, double coef) {
      switch (indiceDeTaille)
      {
        case 0 : // Taille 8
          return (int)(coef*10);
        case 1 :	// Taille 9
          return (int)(coef*11);
        case 2 : // Taille 11
          return (int)(coef*13);
        case 3 : // Taille 12
          return (int)(coef*14);
        case 4 : // Taille 14
          return (int)(coef*17);
        // Ajouts version 4.7.5.2
        default :
        	return (int)(coef*(2*indiceDeTaille+9));
        // default : return (int)(coef*13);
      }
    }

    // Modification de la version java : on accepte les lettres grecques
  public static final boolean lettre(char car) {
    boolean res = ((car  >= 'a') && (car <= 'z')) || ((car >= 'A') && (car <= 'Z'));
    // Lettres grecques acceptes par le logiciel
    res = res || ((car >= 0x3b1) && (car <= (0x3b1 + 7)));
    res = res || (car == 0x3b1 + 10);
    res = res || (car == 0x3b1 + 11);
    res = res || (car == 0x3b1 + 13);
    // Modifi version 4.8.1
    // res = res || ((car >= 0x3b1 + 15) && (car <= (0x3b1 + 19)));
    res = res || ((car >= 0x3b1 + 15) && (car != 0x3b1 + 17) && (car <= (0x3b1 + 19)));
    res = res || ((car >= 0x3b1 + 21) && (car <= (0x3b1 + 24)));
    res = res || ((car >= 0x391 + 2) && (car <= (0x391 + 3)));
    res = res || (car == 0x391 + 15);
    res = res || ((car >= 0x391 + 18) && (car <= (0x391 + 19)));
    res = res || ((car >= 0x391 + 21) && (car <= (0x391 + 24)));
    // Ajout version 4.8.1
    res = res || (car == 0x3BA);    
    return res;
  }

  public static final boolean prime(char car) {
    return (car == '\'') || (car == '"');
  }
  public static final boolean chiffre(char car) {
    return (car >= '0') && (car <= '9');
  }
  
  // Mthode de classe renvoyant une fonte de la taille voulue //
  /* Modifi version 5.0 pour qu'on puisse mlanger les styles bold et italique
  public static final Font getFonteNom(int indiceTailleNom, int styleFonte, double coef ) {
  	if (coef == -1) coef = 1;
    switch (styleFonte) {
      case Normal :
        return new Font("Serif", Font.PLAIN, taille(indiceTailleNom, coef));
      case Gras :
        return new Font("Serif", Font.BOLD, taille(indiceTailleNom, coef));
      case Italique :
      default :
        return new Font("Serif", Font.ITALIC, taille(indiceTailleNom, coef));
    }
  }
  */
  // Version 5.0 ; indiceTailleNom est maintenant la taille relle de la police
  public static final Font getFonteNom(int indiceTailleNom, int styleFonte, double coef ) {
  	if (coef == -1) coef = 1;
  	int style = 0;
  	if (styleFonte == 0) style = Font.PLAIN;
  	else {
	  	if ((styleFonte & FonteNom.Gras) != 0) style |= Font.BOLD;
	  	if ((styleFonte & FonteNom.Italique) != 0) style |= Font.ITALIC;
  	}
  	if ((styleFonte & FonteNom.Underline) != 0) {
  		Map<TextAttribute, Integer> fontAttributes = new HashMap<TextAttribute, Integer>();
  		fontAttributes.put(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON);
  		// return new Font("Serif", style, taille(indiceTailleNom, coef)).deriveFont(fontAttributes); // Modifi version 5.0
  		return new Font("Serif", style, (int) (indiceTailleNom*coef)).deriveFont(fontAttributes);
  	}
  	// else return new Font("Serif", style, taille(indiceTailleNom, coef)); Modifi version 5.0
  	else return new Font("Serif", style, (int) (indiceTailleNom*coef));
  }

  public static final Font getFonteEdition(int indiceTailleNom, int styleFonte, double coef ) {
  	if (coef == -1) coef = 1;
  	int style = 0;
  	if (styleFonte == 0) style = Font.PLAIN;
  	else {
	  	if ((styleFonte & FonteNom.Gras) != 0) style |= Font.BOLD;
	  	if ((styleFonte & FonteNom.Italique) != 0) style |= Font.ITALIC;
  	}
  	if ((styleFonte & FonteNom.Underline) != 0) {
  		Map<TextAttribute, Integer> fontAttributes = new HashMap<TextAttribute, Integer>();
  		fontAttributes.put(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON);
  		// return new Font("Serif", style, taille(indiceTailleNom, coef)).deriveFont(fontAttributes); // Modifi version 5.0
  		return new Font("SansSerif", style, (int) (indiceTailleNom*coef)).deriveFont(fontAttributes);
  	}
  	// else return new Font("Serif", style, taille(indiceTailleNom, coef)); Modifi version 5.0
  	else return new Font("Serif", style, (int) (indiceTailleNom*coef));
  }

  
  public final static String premiersCaracteresNonChiffres(String ch) {
  	int len = ch.length();
  	int i = 0;
  	while (i < len) {
  		if (chiffre(ch.charAt(i))) break;
  		else i++;
  	}
  	if (i == len) return ch;
  	else return ch.substring(0, i);
  }
  
  // Ajout version 4.8.0.3
  public final static String premiersCaracteresNonChiffresNonPrime(String ch) {
  	int len = ch.length();
  	int i = 0;
  	while (i < len) {
  		if (chiffre(ch.charAt(i)) || prime(ch.charAt(i))) break;
  		else i++;
  	}
  	if (i == len) return ch;
  	else return ch.substring(0, i);
  }

  // Nouveauts pour la version 1.8
  // Fonction renvoyant une taille de police infrieure  celle de IndiceTaille
  // pour crire en exposant ou en indice
  // Modifi et renomm version 5.0
   public static final int taillePourExposantOuIndice(int taille)
  {
  	 return ((int) (taille*2/3));
  }
  // Nouveauts pour la version 1.8
  // Fonction renvoyant le dcalage en hauteur ou vers le bas lorsqu'on crit un exposant
  // ou un indice suivant l'indice de la taille de la police utilise
  // Dcalages augments par rapport au C++
  // Modifi version 5.0
  public static final int decalageLogiquePourExposantOuIndice(int taille, double coef)
  {
  	return (int) (taille/3);
  }
  
	/**
	 * Ajout version 4.8.1. Modifi version 4.9.1
	 * Pour l'exportation Tikz, il faut savoir si un nom commence par un caractre grec.
	 * Si oui il ne faut pas le traiter dans un \text{}
	 * @param car
	 * @return
	 */
  /*
	public static final boolean estCaractereGrec(char car) {
		// Caractres alpha, beta, gamma, delta, epsilon, zeta, eta, theta
		for (int i = 0; i <= 7; i++) if (car == 0x3b1 + i) return true;
		if (car == 0x3b1 + 10) return true; // lamda
		if (car == 0x3b1 + 11) return true; // mu
		if (car == 0x3b1 + 13) return true; // xi
		// Caractres pi, rho , sigma, tau
		for (int i = 15; i <= 19; i++) if  ((i != 17) && (car == 0x3b1 + i)) return true;
		// phi, chi, psi, omega
		for (int i = 21; i <= 24; i++) if (car == 0x3b1 + i) return true;
		if (car == 0x03BA) return true; // kappa
		// Gamma, Delta
		for (int i = 2; (i <= 3); i++) if (car == 0x391 + i) return true;
		if (car == 0x391 + 15) return true; // Pi
		// Sigma, Tau
		for (int i = 18; (i <= 19); i++) if (car == 0x391 + i) return true;
		// Phi, Chi, Psi, Omega
		for (int i = 21; (i <= 24); i++) if (car == 0x391 + i) return true;		
		return false;
	}
	*/
	public static final boolean estCaractereGrec(char car) {
		for (int i = 0; i < grecHexa.length; i++) {
			if (car == grecHexa[i]) return true;		
		}
		return false;		
	}

	
	/**
	 * Ajout version 4.8.1.Revu version 4.9.1
	 * Renvoie la chane de caractre LaTeX correspondant au caractre grec car
	 * @param car
	 * @return
	 */
	/*
	public final static String convertitLatex(char car) {
		String[] ch1= {"\\alpha","\\beta","\\gamma","\\delta","\\epsilon","\\zeta","\\eta","\\theta"};
		for (int i = 0; i <= 7; i++) if (car == 0x3b1 + i) return ch1[i];
		if (car == 0x3b1 + 10) return "\\lambda"; // lambda
		if (car == 0x3b1 + 11) return "\\mu"; // mu
		if (car == 0x3b1 + 13) return "\\xi"; // xi
		String[] ch2 = {"\\pi","\\rho","","\\sigma","\\tau"};
		for (int i = 15; i <= 19; i++) if  ((i != 17) && (car == 0x3b1 + i)) return ch2[i-15];
		String[] ch3 = {"\\phi","\\chi","\\psi","\\omega"};
		for (int i = 21; i <= 24; i++) if (car == 0x3b1 + i) return ch3[i-21];
		if (car == 0x03BA) return "\\kappa";
		String[] ch4 = {"\\Gamma","\\Delta"};
		for (int i = 2; (i <= 3); i++) if (car == 0x391 + i) return ch4[i-2];
		String[] ch5 = {"\\Sigma","\\Tau"};
		for (int i = 18; (i <= 19); i++) if (car == 0x391 + i) return ch5[i-18];
		String[] ch6 = {"\\Phi","\\Chi","\\Psi","\\Omega"};
		for (int i = 21; (i <= 24); i++) if (car == 0x391 + i) return ch6[i-21];
		return "";
	}
	*/
	public final static String convertitLatex(char car) {
		for (int i = 0; i < grecHexa.length; i++) {
			if (car == grecHexa[i]) return grecLatex[i];		
		}
		return "";
	}
	
	// Ajout version 4.8.1
	public static final String remplaceUnicodeParLatex(String ch) {
		String st = ch;
		boolean bstart$ = ch.startsWith("$");
		String dol = bstart$ ? "" : "$";
		for (int i = 0; i < grecHexa.length; i++) st = st.replace(Character.toString(grecHexa[i]), dol+grecLatex[i]+dol);
		for (int i = 0; i < mathHexa.length; i++) st = st.replace(Character.toString(mathHexa[i]), dol+mathLatex[i]+dol);
		for (int i = 0; i < flechesHexa.length; i++) st = st.replace(Character.toString(flechesHexa[i]), dol+flechesLatex[i]+dol);
		return st;
	}
	
	// Ajout version 4.8.1
	public static final String traiteCodesDiese(String st) {
		int ind1, ind2;
		String ch = st;
		while((ind1 = ch.indexOf("#G")) != -1) {
			int k = ch.indexOf("#", ind1+2);
			ind2 = (k == -1) ? ch.length() : k;
			ch = ch.substring(0, ind1) + "\\textbf{" + ch.substring(ind1+2, ind2) + "}" + ch.substring(ind2);
		}
		while((ind1 = ch.indexOf("#I")) != -1) {
			int k = ch.indexOf("#", ind1+2);
			ind2 = (k == -1) ? ch.length() : k;
			ch = ch.substring(0, ind1) + "\\textit{" + ch.substring(ind1+2, ind2) + "}" + ch. substring(ind2);
		}
		ch = ch.replace("#N", "");
		ch = ch.replace("#", "");
		return ch;
	}
	
	// Dplac version 4.9.1 depuis CCommentaire
	/**
	 * Renvoie l'indice de l'accolade fermannte correspondant  une accolade ouvrante.
	 * Lors de l'appel pdebut pointe sur le premier caractre suivant l'accolade fermante
	 * @param chaine
	 * @param pdebut
	 * @return
	 */
	public static final int parentheseFermante(String chaine, int pdebut) {
		int p;
		char ch;
		int somme;

		somme = 1;
		p = pdebut;
		int len = chaine.length();
		while ((somme  != 0) && (p <len))  {
			ch = chaine.charAt(p);
			if (ch == '(')
				somme++;
			else {
				if (ch == ')')
					somme--;
			}
			p++;
		}
		if (somme == 0) return p - 1; else return -1;
	}

	// Ajout version 4.9.1
	/**
	 * Renvoie l'indice de l'accolade fermannte correspondant  une accolade ouvrante.
	 * Lors de l'appel pdebut pointe sur le premier caractre suivant l'accolade fermante
	 * @param chaine
	 * @param pdebut
	 * @return
	 */
	/* Abandonn mais  garder au cas o
	public static final int accoladeFermante(String chaine, int pdebut) {
		int p;
		char ch;
		int somme;

		somme = 1;
		p = pdebut;
		int len = chaine.length();
		while ((somme  != 0) && (p <len))  {
			ch = chaine.charAt(p);
			if (ch == '{')
				somme++;
			else {
				if (ch == '}')
					somme--;
			}
			p++;
		}
		if (somme == 0) return p - 1; else return -1;
	}
	*/
	// Version 4.9.1
	/**
	 * Traite la chane de caractres st pour ajouter avant les appels non imbriqus de \frac un code \displaystyle 
	 * et ajoutant au numrateur un code \strut
	 * Avant appel la chane st doit tre correcte au point de vue sybtaxique pour le LaTeX
	 * @param st
	 * @return
	 */
	/* Abandonn. A garder au cas o
	public static final String traiteFractionsTickz(String st) {
		int i,deb,indacouv,indacfer;
		if (st.indexOf("\\frac") == -1) return st;
		String ch = "";
		try {
		deb = 0; 
			while ((i = st.indexOf("\\frac", deb)) != -1) {
				indacouv = st.indexOf("{", i + 4);
				indacfer = accoladeFermante(st, st.indexOf('{', accoladeFermante(st, indacouv + 1))+1);
				ch += st.substring(deb, i) + "\\displaystyle \\frac{ \\strut " + st.substring(indacouv+1, indacfer);
				deb = indacfer;
			}
			if (deb < st.length()) ch += st.substring(deb);
			return ch;
		}
		catch (Exception e) {
			return st;
		}
	}
	*/
	/**
	 * Ajout version 4.9.
	 * Renvoie true si la chane reprsente un nombre entier ou dcimal et false sinon.
	 * @param ch
	 * @return
	 */
	public static final boolean estDecimalCorrect(String ch) {
    boolean ptdec = false;
    for (int i = 0; i < ch.length(); i++) {
      char car = ch.charAt(i);
      if (car == '.') {
        if (ptdec) return false;
        else ptdec = true;
      }
      else if (!chiffre(car)) return false;
    }
    return true;
  }
	
	// Ajout version 4.9.8
	/**
	 * Renvoie true si ch est une cha^ne reprsnetant un caractre grec
	 * @param ch
	 * @return
	 */
	public static final boolean estNomGrec(String ch) {
		String ch2 = "\\" + ch + " ";
  	for (int i = 0; i < grecLatex.length; i++) {
  		if (grecLatex[i].equals(ch2)) return true;
  	}
  	return false;
  }
	
  // Renvoie la hauteur de la premire ligne  afficher dans le cas d'un commentaire
  // from d'une ou de plusieurs lignes. Ne doit pas comprendre GetDescent();
  public static int ascent(Graphics2D pDC, int indiceTaille, double coef, String ligne) {
    Font font = FonteNom.getFonteNom(indiceTaille, FonteNom.Normal, coef);    
    FontMetrics fm = pDC.getFontMetrics(font);
    int dec = 0;
    int hautfont;
  	hautfont = (int) (fm.getMaxAscent()*coef);
    if ((ligne.indexOf("#H(") != -1) || (ligne.indexOf("#L(") != -1))
    {
      int dec2 = FonteNom.decalageLogiquePourExposantOuIndice(indiceTaille, coef);
      if (dec2 > dec) dec = dec2;
    }
    hautfont += dec;
    // On regarde si la ligne contient un affichage Latex
    if (ligne.indexOf("$") != -1) {
      StringTokenizer st2 = new StringTokenizer(ligne, "#$",true);
      boolean latexEnCours = false;
      while (st2.hasMoreTokens()) {
        String ch = st2.nextToken();
        char premcar = ch.charAt(0);
        if ((premcar == '#') || (premcar == '$')) {
	        if ((premcar != '#')) {
			      if ((premcar == '$') && (latexEnCours)) latexEnCours = false;
			      else latexEnCours = true;
	        }
        }
        else {
	      	if (latexEnCours) {
        		TeXFormula formula =  CLatex.creeFormule(ch);
        		if (formula != null) {
	        		TeXIcon icon;
	        		try {
	        			// Ligne suivante modifi version 5.0
		        		icon = formula.createTeXIcon(TeXConstants.STYLE_DISPLAY, (float)(CLatex.tailleLatex(indiceTaille)*coef));
		           	int h = (int)(icon.getIconHeight()*icon.getBaseLine());
		           	if (h > hautfont) hautfont = h;
	        		}
	        		catch (Exception e) {
								// Si erreur on affichera le code LatTex donc on ne change pas la hauteur de la ligne
							}
        		}
        	}
      	}	      
      }    	
    }
    return hautfont;
  }
}
