/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package types;
/*
 * Pinceau.java
 * Created on 27 juin 2004, 18:35
 * Modifi version 5.0
 */

/**
 *
 * @author  Yves
 * @version 
 */
// Classe servant via ses mthodes de classe  crer des pinceaux de style de trait donn
// et de couleur donne
// Si basseDefinition est true on est en rsolution cran et sinon on est en rsolution mulstiplie par 100
// (10 dans chaque direction)
import java.awt.*;

import objets.CListeObjets;
public class Pinceau {
	/* Supprim version 5.0
	public static final float[] floatTraitPointille = new float[] {3,3};
	public static final float[] floatTraitTrait = new float[] {10,10};
	public static final float[] floatTraitPointPoint = new float[] {10,3,3,3,3,3};
	*/
	/* Supprim version 5.0
	private static final BasicStroke pinceauTraitFin = new BasicStroke(1.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);
	private static final BasicStroke pinceauTraitPointille = new BasicStroke(1.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1,floatTraitPointille,0f);
	private static final BasicStroke pinceauTraitEpais = new BasicStroke(2.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);
	private static final BasicStroke pinceauTraitTrait = new BasicStroke(1.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1,floatTraitTrait,0f);
	private static final BasicStroke pinceauTraitPointPoint = new BasicStroke(1.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1,floatTraitPointPoint,0f);
	private static final BasicStroke pinceauTraitTresEpais = new BasicStroke(3.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);
	private static final BasicStroke pinceauTraitPointilleEpais = new BasicStroke(2.0f,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1,floatTraitPointille,0f);
	*/
	
  /** Creates new Pinceau */
  public Pinceau() {
  }
  public static final BasicStroke CreePinceau(StyleTrait styleTrait, CListeObjets liste) {
  	float k3, k10;
  	int ep = styleTrait.epaisseur;
  	float k = (liste == null) ? 1 : (float) liste.documentProprietaire.coefForPointsAndMarks;
  	if ((liste == null) || liste.basseDefinition) {
	    switch (styleTrait.style) {
	      case StyleTrait.styleTraitContinu :
	        return new BasicStroke(ep,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);
	      case StyleTrait.styleTraitPointille :
	      	k3 = k*3;
	      	return new BasicStroke(ep,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1, new float[] {k3,k3},0f);
	      case StyleTrait.styleTraitTrait :
	      	k10 = k*10;
	        return new BasicStroke(ep,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1, new float[] {k10,k10},0f);
	      case StyleTrait.styleTraitPointPoint :
	      	k3 = k*3;
	      	k10 = k*10;
	      	return new BasicStroke(ep,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1, new float[] {k10,k3,k3,k3,k3,k3},0f);
	      case StyleTrait.styleTraitPointPointPoint :
	      	k3 = k*3;
	      	k10 = k*10;
	      	return new BasicStroke(ep,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1, new float[] {k10,k3,k3,k3,k3,k3,k3,k3},0f);
	      case StyleTrait.styleTraitTraitLongTrait :
	      	k3 = k*3;
	      	k10 = k*10;
	      	return new BasicStroke(ep,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1, new float[] {k10,k3,k*5,k3},0f);
	      default : return null;
	    }
  	} 
  	else {
  		float coef = (float) liste.coefMult*k;
  		// Si demand dans les prfrences on divise l'paisseur des traits par 2 pour un coefficient multiplicateur au moins gal  2
  		if (liste.bAttenuationEpaisseurTrait) {
  			if (coef < 2) coef = 1;
  			else coef /= 2;
  		}
	    switch (styleTrait.style) {
	      case StyleTrait.styleTraitContinu :
	        return new BasicStroke(coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);
	      case StyleTrait.styleTraitPointille :
	        return new BasicStroke(coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1, new float[] {3*coef,3*coef},0f);
	      case StyleTrait.styleTraitTrait :
	        return new BasicStroke(coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1, new float[] {10*coef,10*coef},0f);
	      case StyleTrait.styleTraitPointPoint :
	        return new BasicStroke(coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1 ,new float[] {10*coef,3*coef,3*coef,3*coef,3*coef,3*coef},0f);
	      case StyleTrait.styleTraitPointPointPoint :
	        return new BasicStroke(coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1 ,new float[] {10*coef,3*coef,3*coef,3*coef,3*coef,3*coef,3*coef,3*coef},0f);
	      case StyleTrait.styleTraitTraitLongTrait :
	        return new BasicStroke(coef,BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER,1 ,new float[] {10*coef,3*coef,5*coef,3*coef},0f);
	      default : return null;
	    }
  	}
  }    
}
